@@ -60,7 +60,6 @@ public:
 	virtual void show(void);
 	virtual void hide(void);
 	
-	XAP_FontPreview *				m_pFontPreview;	
 protected:
 	void							_releaseListener(void);
 
@@ -76,7 +75,7 @@ protected:
 	GtkWidget *						m_wToolbar;
 	GtkWidget * 					m_wHandleBox;
 	AP_UnixToolbar_Icons 			m_pUnixToolbarIcons;
-	UT_Vector						m_vecToolbarWidgets;
+	UT_GenericVector<_wd*>			m_vecToolbarWidgets;
 };
 
 #endif /* EV_UNIXTOOLBAR_H */
