@@ -62,16 +62,6 @@ utf8_char_len(uchar byte)
 	return (((0xE5000000 >> ((byte >> 3) & 0x1E)) & 3) + 1); 
 }
 
-const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
-										 const char* pszFontStyle,
-										 const char* pszFontVariant,
-										 const char* pszFontWeight,
-										 const char* pszFontStretch,
-										 const char* pszFontSize)
-{
-	return pszFontFamily;
-}
-
 GR_BeOSGraphics::GR_BeOSGraphics(BView *docview, XAP_App * app) {
 	m_pApp = app;
 	m_pShadowView = NULL;
@@ -273,6 +263,11 @@ void GR_BeOSGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 	UPDATE_VIEW
 }
 
+void GR_BeOSGraphics::drawGlyph(UT_uint32 Char,UT_sint32 xoff,UT_sint32 yoff)
+{
+	UT_ASSERT(UT_TODO);
+}
+
 BFont *findClosestFont(const char* pszFontFamily, 
 		 	const char* pszFontStyle, 
 			const char* pszFontWeight) {
@@ -377,7 +372,7 @@ GR_Font* GR_BeOSGraphics::findFont(const char* pszFontFamily,
 	BFont 	*aFont;
 	//int 	size = atoi(pszFontSize);
 	//UT_sint32 iHeight = convertDimension(pszFontSize);
-	int		size = convertDimension(pszFontSize);
+	double	size = UT_convertToPoints(pszFontSize);
 
 	DPRINTF(printf("GR Find Font:\n\tFamily: %s ", pszFontFamily));
 	DPRINTF(printf("\n\tStyle: %s ", pszFontStyle));
@@ -487,7 +482,7 @@ UT_uint32 GR_BeOSGraphics::getFontHeight(GR_Font *font)
 	return((UT_uint32)(fh.ascent + fh.descent + fh.leading + 0.5));
 }
 
-UT_uint32 GR_BeOSGraphics::measureUnRemappedChar(const UT_UCSChar c)
+UT_sint32 GR_BeOSGraphics::measureUnRemappedChar(const UT_UCSChar c)
 {
 	//We need to convert the string from UCS-X to UTF-8 before
 	//we use the BeOS string operations on it.
@@ -524,6 +519,11 @@ UT_uint32 GR_BeOSGraphics::measureUnRemappedChar(const UT_UCSChar c)
 	return 1.0f; // Shouldn't happen.
 }
 
+void GR_BeOSGraphics::getCoverage(UT_Vector& coverage)
+{
+	UT_ASSERT(UT_TODO);
+}
+
 UT_uint32 GR_BeOSGraphics::_getResolution() const
 {
 	return 72;
@@ -622,7 +622,7 @@ void GR_BeOSGraphics::invertRect(const UT_Rect* pRect)
 }
 
 
-void GR_BeOSGraphics::fillRect(UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
+void GR_BeOSGraphics::fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 						UT_sint32 w, UT_sint32 h)
 {
 	DPRINTF(printf("GR: Flll Rect\n"));
@@ -966,6 +966,28 @@ GR_Graphics::Cursor GR_BeOSGraphics::getCursor(void) const
 	return m_cursor;
 }
 
+void GR_BeOSGraphics::saveRectangle(UT_Rect & r, UT_uint32 iIndx)
+{
+	UT_ASSERT(UT_TODO);
+}
+
+void GR_BeOSGraphics::restoreRectangle(UT_uint32 iIndx)
+{
+	UT_ASSERT(UT_TODO);
+}
+
+UT_uint32 GR_BeOSGraphics::getDeviceResolution(void) const
+{
+	BScreen * pScreen = new BScreen(m_pShadowView->Window());
+	if (pScreen)
+	{
+		BRect r = pScreen->Frame();
+		return (UT_uint32)(r.Width()); // NOTE: assumes square pixels
+	}
+	else
+		return 800; // fallback
+}
+
 void GR_BeOSGraphics::setColor3D(GR_Color3D c)
 {
 	DPRINTF(printf("Set color 3D %d \n", c));
@@ -1022,3 +1044,33 @@ void GR_Font::s_getGenericFontProperties(const char * szFontName,
 	*pfp = FP_Unknown;
 	*pbTrueType = true;
 }
+
+UT_sint32 BeOSFont::measureUnremappedCharForCache(UT_UCSChar cChar) const
+{
+	// first of all, handle 0-width spaces ...
+	if(cChar == 0xFEFF || cChar == 0x200b || cChar == UCS_LIGATURE_PLACEHOLDER)
+		return 0;
+
+ 	//We need to convert the string from UCS-X to UTF-8 before
+ 	//we use the BeOS string operations on it.
+ 	char buffer[10];
+ 
+ 	BPoint escapementArray[1];
+ 	
+ 	char * utf8char;
+ 	utf8char =  UT_encodeUTF8char(cChar);
+ 	strcpy(buffer, utf8char);						
+ 
+ 	escapement_delta tempdelta;
+ 	tempdelta.space=0.0;
+ 	tempdelta.nonspace=0.0;
+ 	float fontsize=0.0f;
+ 
+ 	m_pBFont->SetSpacing(B_BITMAP_SPACING);
+ 
+ 	//Hope this works on UTF-8 characters buffers
+ 	m_pBFont->GetEscapements(buffer,1,&tempdelta,escapementArray);
+ 	fontsize=m_pBFont->Size();
+ 
+ 	return (UT_uint32)(escapementArray[0].x *fontsize);
+}
