@@ -508,15 +508,16 @@ extern UT_uint32 adobeDingbatsToUnicode(UT_uint32 iAdobe)
 #endif
 }
 
-const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
-										 const char* pszFontStyle,
-										 const char* pszFontVariant,
-										 const char* pszFontWeight,
-										 const char* pszFontStretch,
-										 const char* pszFontSize)
+const char* GR_UnixGraphics::findNearestFont(const char* pszFontFamily,
+											 const char* pszFontStyle,
+											 const char* pszFontVariant,
+											 const char* pszFontWeight,
+											 const char* pszFontStretch,
+											 const char* pszFontSize)
 {
 	xxx_UT_DEBUGMSG(("Find Font findNearestFont for %s \n",pszFontFamily));
-	XAP_UnixFont* pUnixFont = XAP_UnixFontManager::pFontManager->findNearestFont(pszFontFamily, pszFontStyle, pszFontVariant, pszFontWeight,
+	XAP_UnixFont* pUnixFont = XAP_UnixFontManager::pFontManager->findNearestFont(pszFontFamily, pszFontStyle,
+																				 pszFontVariant, pszFontWeight,
 																				 pszFontStretch, pszFontSize);
 	xxx_UT_DEBUGMSG(("Return Font name %s \n",pUnixFont->getName()));
 	//	UT_ASSERT(0);
@@ -758,17 +759,29 @@ static GdkCapStyle mapCapStyle ( GR_Graphics::CapStyle in )
     }
 }
 
-static GdkLineStyle mapLineStyle ( GdkGC* pGC, GR_Graphics::LineStyle in )
+static GdkLineStyle mapLineStyle ( GdkGC				  * pGC, 
+								   GR_Graphics::LineStyle 	in, 
+								   gint 					iWidth )
 {
+	iWidth = iWidth == 0 ? 1 : iWidth;
 	switch ( in )
     {
 		case GR_Graphics::LINE_ON_OFF_DASH :
+			{
+				gint8 dash_list[2] = { 4*iWidth, 4*iWidth };
+				gdk_gc_set_dashes(pGC, 0, dash_list, 2);
+			}
 			return GDK_LINE_ON_OFF_DASH ;
 		case GR_Graphics::LINE_DOUBLE_DASH :
+			{
+				gint8 dash_list[2] = { 4*iWidth, 4*iWidth };
+				gdk_gc_set_dashes(pGC, 0, dash_list, 2);
+			}
 			return GDK_LINE_DOUBLE_DASH ;
 		case GR_Graphics::LINE_DOTTED:
 			{
-				gint8 dash_list[2] = { 1, 2 }; // MARCM: I chose a pattern of 1 pixel on, 2 pixels off for dotted lines
+				/* strange but 1/3 ratio looks dotted */
+				gint8 dash_list[2] = { iWidth, 3*iWidth };
 				gdk_gc_set_dashes(pGC, 0, dash_list, 2);
 			}
 			return GDK_LINE_ON_OFF_DASH;
@@ -797,12 +810,13 @@ void GR_UnixGraphics::setLineProperties ( double inWidth,
 										  GR_Graphics::CapStyle inCapStyle,
 										  GR_Graphics::LineStyle inLineStyle )
 {
-	gdk_gc_set_line_attributes ( m_pGC, static_cast<gint>(tduD(inWidth)),
-								 mapLineStyle ( m_pGC, inLineStyle ),
+	gint iWidth = static_cast<gint>(tduD(inWidth));
+	gdk_gc_set_line_attributes ( m_pGC, iWidth,
+								 mapLineStyle ( m_pGC, inLineStyle, iWidth ),
 								 mapCapStyle ( inCapStyle ),
 								 mapJoinStyle ( inJoinStyle ) ) ;
-	gdk_gc_set_line_attributes ( m_pXORGC, static_cast<gint>(tduD(inWidth)),
-								 mapLineStyle ( m_pGC, inLineStyle ),
+	gdk_gc_set_line_attributes ( m_pXORGC, iWidth,
+								 mapLineStyle ( m_pXORGC, inLineStyle, iWidth ), /* this was m_pGC before */
 								 mapCapStyle ( inCapStyle ),
 								 mapJoinStyle ( inJoinStyle ) ) ;
 }
@@ -844,7 +858,9 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 	UT_sint32 idy = _tduY(yoff);
 	UT_sint32 idx = _tduX(xoff);
 
-	UT_sint32 iAscent = m_pXftFontL->ascent * getResolution() / s_getDeviceResolution();
+	//	UT_sint32 iAscent = m_pXftFontL->ascent * getResolution() / s_getDeviceResolution();
+	UT_sint32 iAscent = getFontAscent();
+
 	if (!pCharWidths)
 	{
 		if(!isSymbol() && !isDingbat())
@@ -1223,7 +1239,7 @@ UT_uint32 GR_UnixGraphics::getFontAscent(GR_Font * fnt)
 	// FIXME we should really be getting stuff fromt he font in layout units,
 	// FIXME but we're not smart enough to do that yet
     // we call s_getDeviceResolution() to avoid zoom
-	return static_cast<UT_uint32>(hndl->getUnixFont()->getAscender(hndl->getSize()) * getResolution() / s_getDeviceResolution() + 0.5);
+	return static_cast<UT_uint32>(hndl->getUnixFont()->getAscender(hndl->getSize()) * getResolution() / s_getDeviceResolution() ); // +0.5);
 }
 
 UT_uint32 GR_UnixGraphics::getFontAscent()
@@ -1241,7 +1257,7 @@ UT_uint32 GR_UnixGraphics::getFontDescent(GR_Font * fnt)
 	XAP_UnixFont* pFont = hndl->getUnixFont();
 	// FIXME we should really be getting stuff fromt he font in layout units,
 	// FIXME but we're not smart enough to do that yet
-	return static_cast<UT_uint32>(pFont->getDescender(hndl->getSize()) * getResolution() / getDeviceResolution() + 0.5);
+	return static_cast<UT_uint32>(pFont->getDescender(hndl->getSize()) * getResolution() / getDeviceResolution() );  // +0.5);
 }
 
 UT_uint32 GR_UnixGraphics::getFontDescent()
@@ -1679,8 +1695,16 @@ void GR_UnixGraphics::setCursor(GR_Graphics::Cursor c)
 	case GR_CURSOR_DOWNARROW:
 		cursor_number = GDK_SB_DOWN_ARROW;
 		break;
-	}
 
+	case GR_CURSOR_DRAGTEXT:
+		cursor_number = GDK_TARGET;
+		break;
+
+	case GR_CURSOR_COPYTEXT:
+		cursor_number = GDK_DRAPED_BOX;
+		break;
+	}
+	xxx_UT_DEBUGMSG(("cursor set to %d  gdk %d \n",c,cursor_number));
 	GdkCursor * cursor = gdk_cursor_new(cursor_number);
 	gdk_window_set_cursor(m_pWin, cursor);
 	gdk_cursor_unref(cursor);
@@ -1800,10 +1824,10 @@ GR_Image * GR_UnixGraphics::genImageFromRectangle(const UT_Rect &rec)
 	UT_sint32 idy = _tduY(rec.top);
 	UT_sint32 idw = _tduR(rec.width);
 	UT_sint32 idh = _tduR(rec.height);
-
+	GdkColormap* cmp = gdk_colormap_get_system();
 	GdkPixbuf * pix = gdk_pixbuf_get_from_drawable(NULL,
 												   m_pWin,
-												   NULL,
+												   cmp,
 												   idx, idy, 0, 0,
 												   idw, idh);
 	
