@@ -62,16 +62,6 @@
 
 /*****************************************************************/
 
-const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
-										 const char* pszFontStyle,
-										 const char* pszFontVariant,
-										 const char* pszFontWeight,
-										 const char* pszFontStretch,
-										 const char* pszFontSize)
-{
-	return pszFontFamily;
-}
-
 // A small helper class
 class private_FontReverter
 {
@@ -897,14 +887,14 @@ void GR_Win32Graphics::fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 	RECT r;
 	r.left = (UT_sint32)((double)_tduX(x) * m_fXYRatio);
 	r.top = _tduY(y);
-	r.right = (UT_sint32)(_tduX(x+w) * m_fXYRatio);
-	r.bottom = _tduY(y+h);
+	r.right = _tduX(x + w);
+	r.bottom =(UT_sint32)((double) _tduY(y + h) * m_fXYRatio);
+	w=(UT_sint32)((double)_tduR(w) * m_fXYRatio);
+	h=_tduR(h);
 
 	COLORREF clr = RGB(c.m_red, c.m_grn, c.m_blu);
 
 	#ifdef GR_GRAPHICS_DEBUG
-	w=(UT_sint32)((double)_tduR(w) * m_fXYRatio);
-	h=_tduR(h);
 	UT_DEBUGMSG(("GR_Win32Graphics::fillRect %x %u %u %u %u\n",  clr, r.left, r.top, w, h));	
 	#endif
 		
@@ -943,6 +933,10 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 		pDevMode->dmFields = DM_ORIENTATION;
 		pDevMode->dmOrientation = (bPortrait) ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
 		GlobalUnlock(m_hDevMode);
+
+		// call DocumentProperties() on the DEVMODE to ensure changes propagate down into
+		// the private part
+		fixDevMode(m_hDevMode);
 		
 		pDevMode = (DEVMODE*) GlobalLock(m_hDevMode);
 		ResetDC(m_hdc, pDevMode);
@@ -1045,21 +1039,7 @@ void GR_Win32Graphics::clearArea(UT_sint32 x, UT_sint32 y, UT_sint32 width, UT_s
 	r.right = r.left + width;
 	r.bottom = r.top + height;
 
-#if 1
-	// for the sake of consistency we should be using the same method as the fillRect()
-	// does; however, we already have brush created, so FillRect is probably quicker here
-	// (someone should do some profiling on this one day)
 	FillRect(m_hdc, &r, m_hClearBrush);
-#else
-	// this is the method used by fillRect(); if we were to use it, it we should
-	// cache lb.lbColor from inside setBrush() to avoid calling GetObject() all the time
-	LOGBRUSH lb;
-	GetObject(m_hClearBrush, sizeof(LOGBRUSH), &lb);
-	
-	const COLORREF cr = ::SetBkColor(m_hdc,  lb.lbColor);
-	::ExtTextOut(m_hdc, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
-	::SetBkColor(m_hdc, cr);
-#endif
 }
 
 void GR_Win32Graphics::invertRect(const UT_Rect* pRect)
@@ -1328,6 +1308,7 @@ void GR_Win32Graphics::init3dColors(void)
 void GR_Win32Graphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h)
 {
 	UT_ASSERT(c < COUNT_3D_COLORS);
+	HBRUSH hBrush = CreateSolidBrush(m_3dColors[c]); 
 
 	#ifdef GR_GRAPHICS_DEBUG
 	UT_DEBUGMSG(("GR_Win32Graphics::fillRect GR_Color3D  %x %u %u %u %u\n",  c, x, y, w, h));	
@@ -1336,17 +1317,11 @@ void GR_Win32Graphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint3
 	RECT r;
 	r.left = (UT_sint32)((double)_tduX(x) * m_fXYRatio);
 	r.top = _tduY(y);
-	r.right = (UT_sint32)((double)_tduX(x+w) * m_fXYRatio);
-	r.bottom = _tduY(y+h);
+	r.right = (UT_sint32)((double)_tduX(x + w) * m_fXYRatio);
+	r.bottom = _tduY(y + h);
 
-	// This might look wierd (and I think it is), but it's MUCH faster.
-	// CreateSolidBrush is dog slow.
-	HDC hdc = m_hdc;
-	COLORREF clr = RGB(GetRValue(m_3dColors[c]), GetGValue(m_3dColors[c]), GetBValue(m_3dColors[c]));
-	
-	const COLORREF cr = ::SetBkColor(hdc,  clr);
-	::ExtTextOut(hdc, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
-	::SetBkColor(hdc, cr);
+	FillRect(m_hdc, &r, hBrush);
+	DeleteObject(hBrush);
 	setExposePending(false);
 }
 
@@ -1680,6 +1655,20 @@ UT_sint32 GR_Win32Font::measureUnremappedCharForCache(UT_UCSChar cChar) const
 #endif
 }
 
+//
+// UT_Rect of glyph in Logical units.
+// rec.left = bearing Left (distance from origin to start)
+// rec.width = width of the glyph
+// rec.top = distance from the origin to the top of the glyph
+// rec.height = total height of the glyph
+
+bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec) const
+{
+  // FIXME: Write the code for this!
+  UT_ASSERT(0);
+  return false;
+}
+
 /*!
 	Implements a GR_CharWidths.
 	Override if you which to instanciate a subclass.
@@ -2272,7 +2261,7 @@ bool GR_Win32Graphics::fixDevMode(HGLOBAL hDevMode)
 	}
 	
 	// now get the printer driver to merge the data in the public section into its private part
-	dwRet = DocumentProperties(NULL,hPrinter, (char*)& pDM->dmDeviceName, pDM, pDM, DM_IN_BUFFER | DM_OUT_BUFFER);
+	dwRet = DocumentProperties(NULL,hPrinter, (char*)& pDM->dmDeviceName, pDM, pDM, DM_OUT_BUFFER | DM_IN_BUFFER);
 
 	// free what needs be ...
 	ClosePrinter(hPrinter);
