@@ -23,6 +23,7 @@
 #include "xap_Prefs.h"
 #include "xap_Frame.h"
 #include "xav_View.h"
+#include "ut_Win32OS.h"
 
 extern "C"  UT_uint16    wvLangToLIDConverter(const char * lang);
 extern "C"  const char * wvLIDToLangConverter(UT_uint16);
@@ -1279,21 +1280,29 @@ void GR_Win32USPGraphics::renderChars(GR_RenderInfo & ri)
 	// font; if we are drawing on screen and using printer metrics to do the layout, the
 	// ascent of the screen font can be smaller/greater and so we need to adjust the yoff
 	// accordingly
-	UT_sint32 iAscentScreen = pFont->getScreenAscent();
+	UT_sint32 yoff;
 
-	if(!iAscentScreen)
+	if(m_bPrint)
 	{
-		TEXTMETRIC tm = { 0 };
-		GetTextMetrics(m_hdc, &tm);
-		iAscentScreen = (UT_sint32)((double)tm.tmAscent*(double)getResolution()*100.0/
-									((double)getDeviceResolution()*(double)getZoomPercentage()));
-		
-		pFont->setScreenAscent(iAscentScreen);
+		yoff = _tduY(RI.m_yoff);
 	}
+	else
+	{
+		UT_sint32 iAscentScreen = pFont->getScreenAscent();
 
-	UT_sint32 iAscentPrint = pFont->getTextMetric().tmAscent * getResolution() / getDeviceResolution();
-	
-	UT_sint32 yoff = _tduY(RI.m_yoff + iAscentPrint - iAscentScreen);
+		if(!iAscentScreen)
+		{
+			TEXTMETRIC tm = { 0 };
+			GetTextMetrics(m_hdc, &tm);
+			iAscentScreen = (UT_sint32)((double)tm.tmAscent*(double)getResolution()*100.0/
+										((double)getDeviceResolution()*(double)getZoomPercentage()));
+		
+			pFont->setScreenAscent(iAscentScreen);
+		}
+
+		UT_sint32 iAscentPrint = pFont->getTextMetric().tmAscent * getResolution() / getDeviceResolution();
+		yoff = _tduY(RI.m_yoff + iAscentPrint - iAscentScreen);
+	}
 	
 	int * pJustify = RI.m_pJustify && RI.m_bRejustify ? RI.s_pJustify + iGlyphOffset : NULL;
 	
@@ -1334,7 +1343,6 @@ void GR_Win32USPGraphics::renderChars(GR_RenderInfo & ri)
 
 void GR_Win32USPGraphics::setPrintDC(HDC dc)
 {
-#if 1
 	// only do this for screen graphics
 	if(!m_bPrint && dc != m_printHDC)
 	{
@@ -1379,12 +1387,11 @@ void GR_Win32USPGraphics::setPrintDC(HDC dc)
 					GR_Graphics * pG = pView->getGraphics();
 
 					if(pG == this)
-						pView->remeasureChars();
+						pView->rebuildLayout();
 				}
 			}
 		}
 	}
-#endif
 }
 
 
@@ -1409,6 +1416,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	UT_uint32 iPoints = (UT_uint32)pFont->getPointSize();
 #endif
 
+	bool bFontSetUpOnDC = false;
+	
 	// the script cache is always containing data for 100% zoom, we scale widths manually later
 	// but we need to refresh it if the printer changed
 	//if(iZoom != RI.m_iZoom)
@@ -1425,9 +1434,18 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 		{
 			// we also need to remeasure the font metrics as well
 			// and scale it down for the screen
+			_setupFontOnDC(pFont, false);
+			bFontSetUpOnDC = true;
+			
 			TEXTMETRIC tm = { 0 };
 			GetTextMetrics(getPrintDC(), &tm);
 
+#if 0 //def DEBUG
+			HDC printHDC = UT_GetDefaultPrinterDC();
+			TEXTMETRIC tm2 = { 0 };
+			GetTextMetrics(printHDC, &tm2);
+			DeleteDC(printHDC);
+#endif
 			pFont->setHeight(MulDiv(tm.tmHeight, getDeviceResolution(), m_nPrintLogPixelsY));
 			pFont->setAscent(MulDiv(tm.tmAscent, getDeviceResolution(), m_nPrintLogPixelsY));
 			pFont->setDescent(MulDiv(tm.tmDescent, getDeviceResolution(), m_nPrintLogPixelsY));
@@ -1442,7 +1460,12 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		// need to make sure that the HDC has the correct font set
 		// we do not scale the font by zoom
-		_setupFontOnDC(pFont, false);
+		if(!bFontSetUpOnDC)
+		{
+			_setupFontOnDC(pFont, false);
+			bFontSetUpOnDC = true;
+		}
+		
 		hdc = m_printHDC ? m_printHDC : m_hdc;
 
 		// we remember the hdc for which we measured so we can remeasure when hdc changes
@@ -1450,7 +1473,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	}
 	else
 	{
-		RI.m_hdc = pFont->getPrintDC();
+		hdc = pFont->getPrintDC() ? pFont->getPrintDC() : m_hdc;
+		RI.m_hdc = hdc;
 	}
 	
 
@@ -1468,6 +1492,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		UT_ASSERT_HARMLESS( hdc == 0 );
 		_setupFontOnDC(pFont, false);
+		bFontSetUpOnDC = true;
+		
 		hdc = m_printHDC ? m_printHDC : m_hdc;
 
 		// we remember the hdc for which we measured so we can remeasure when hdc changes
@@ -1476,9 +1502,53 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 		hRes = fScriptPlace(hdc, pFont->getScriptCache(), RI.m_pIndices,
 							RI.m_iIndicesCount, RI.m_pVisAttr,
 							& pItem->m_si.a, RI.m_pAdvances, RI.m_pGoffsets, & RI.m_ABC);
-	}
-	
 
+		UT_ASSERT_HARMLESS( !hRes );
+	}
+
+	const UT_uint32 iAdvSize = 80;
+	int iAdvances[iAdvSize];
+	GOFFSET stGoffsets[iAdvSize];
+	UT_uint32 iCount = iAdvSize;
+	ABC stABC;
+
+	int * pAdvances = &iAdvances[0];
+	GOFFSET * pGoffsets = &stGoffsets[0];
+
+	bool bAdjustAdvances = false;
+	
+#if 1
+	if(!m_bPrint && getPrintDC())
+	{
+		// This is a trick to avoid unseemly gaps between letters on screen due to
+		// discrepancies between font metrics of screen and printer. We measure the font
+		// also on screen and distribute the difference to the left and right of each
+		// character by adjusting each character's m_pGoffsets.du. This improves things,
+		// but does not completely remove the problem; whether it is worth the extra
+		// is to be seen. Tomas, Apr 9, 2005
+		_setupFontOnDC(pFont, true);
+		
+		if(iCount <= RI.m_iIndicesCount)
+		{
+			iCount = RI.m_iIndicesCount + 1;
+			pAdvances = new int[iCount];
+			pGoffsets = new GOFFSET[iCount];
+		}
+		
+		
+		SCRIPT_CACHE sc = NULL;
+		HRESULT hRes = fScriptPlace(m_hdc, &sc, RI.m_pIndices, RI.m_iIndicesCount, RI.m_pVisAttr,
+									& pItem->m_si.a, pAdvances, pGoffsets, & stABC);
+
+		UT_ASSERT_HARMLESS( !hRes );
+		if(!hRes)
+			bAdjustAdvances = true;
+
+		if(sc)
+			fScriptFreeCache(&sc);			
+	}
+#endif
+	
 	pItem->m_si.a.eScript = eScript;
 
 	// remember the zoom at which we calculated this ...
@@ -1497,25 +1567,36 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	}
 
 	// now convert the whole lot to layout units
+	double dWidth = 0;
+	double dDeviceWidth = 0;
+	double dPrevDeviceWidth = 0;
+	double dAdjustment = 0;
+	
 	if(getPrintDC())
 	{
 		double fXYRatio = m_fXYRatio * m_fXYRatioPrint;
 
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
-			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
-			RI.m_pAdvances[i] = (UT_sint32)((double)RI.m_pAdvances[i]*(double)getResolution()
-											/((double)m_nPrintLogPixelsY*fXYRatio) + 0.5);
+			dWidth += (double)RI.m_pAdvances[i];
+			dDeviceWidth = dWidth*(double)getResolution() / ((double)m_nPrintLogPixelsY*fXYRatio);
 
+			if(bAdjustAdvances)
+			{
+				dAdjustment = (((double)RI.m_pAdvances[i]*(double)getResolution()
+								/((double)m_nPrintLogPixelsY*fXYRatio))
+							   - (double)pAdvances[i]*(double)getResolution()*100.
+							   /((double)getDeviceResolution()*(double)iZoom * m_fXYRatio))/2.;
+			}
+			RI.m_pAdvances[i]   = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
 			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()
-											/((double)m_nPrintLogPixelsY*fXYRatio) + 0.5);
+											/((double)m_nPrintLogPixelsY*fXYRatio) + 2*dAdjustment + 0.5);
 			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()
 											/((double)m_nPrintLogPixelsY) + 0.5);
+
+			dPrevDeviceWidth = dDeviceWidth;
 		}
 
-		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
-		//RI.m_ABC.abcB = tlu(RI.m_ABC.abcB);
-		//RI.m_ABC.abcC = tlu(RI.m_ABC.abcC);
 		RI.m_ABC.abcA = (UT_sint32)((double) RI.m_ABC.abcA * (double)getResolution()
 									/((double)m_nPrintLogPixelsY*fXYRatio) + 0.5);
 		RI.m_ABC.abcB = (UT_sint32)((double) RI.m_ABC.abcB * (double)getResolution()
@@ -1527,18 +1608,19 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
-			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
-			RI.m_pAdvances[i] = (UT_sint32)((double)RI.m_pAdvances[i]*(double)getResolution()/
-											((double)getDeviceResolution()*m_fXYRatio) + 0.5);
+			dWidth += (double)RI.m_pAdvances[i];
+			dDeviceWidth = dWidth*(double)getResolution()/
+									   ((double)getDeviceResolution()*m_fXYRatio);
+			
+			RI.m_pAdvances[i] = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
+			dPrevDeviceWidth = dDeviceWidth;
+
 			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()/
 											((double)getDeviceResolution()*m_fXYRatio) + 0.5);
 			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()/
 											((double)getDeviceResolution()) + 0.5);
 		}
 
-		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
-		//RI.m_ABC.abcB = tlu(RI.m_ABC.abcB);
-		//RI.m_ABC.abcC = tlu(RI.m_ABC.abcC);
 		RI.m_ABC.abcA = (UT_sint32)((double) RI.m_ABC.abcA * (double)getResolution() /
 									((double)getDeviceResolution()*m_fXYRatio) + 0.5);
 		RI.m_ABC.abcB = (UT_sint32)((double) RI.m_ABC.abcB * (double)getResolution() /
@@ -1550,18 +1632,22 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	{
 		for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 		{
-			// RI.m_pAdvances[i] = tlu(RI.m_pAdvances[i]);
-			RI.m_pAdvances[i] = (UT_sint32)((double)RI.m_pAdvances[i]*(double)getResolution()/
-											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio) + 0.5);
+			dWidth += (double)RI.m_pAdvances[i];
+			dDeviceWidth = dWidth*(double)getResolution()/
+									   ((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio);
+
+			RI.m_pAdvances[i]   = (UT_sint32)(dDeviceWidth - dPrevDeviceWidth + 0.5);
+
 			RI.m_pGoffsets[i].du = (UT_sint32)((double)RI.m_pGoffsets[i].du*(double)getResolution()/
-											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio) + 0.5);
+											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio)
+											   + 0.5);
+			
 			RI.m_pGoffsets[i].dv = (UT_sint32)((double)RI.m_pGoffsets[i].dv*(double)getResolution()/
 											((double)getDeviceResolution()*(double)GR_WIN32_USP_FONT_SCALING) + 0.5);
+			dPrevDeviceWidth = dDeviceWidth;
+
 		}
 
-		//RI.m_ABC.abcA = tlu(RI.m_ABC.abcA);
-		//RI.m_ABC.abcB = tlu(RI.m_ABC.abcB);
-		//RI.m_ABC.abcC = tlu(RI.m_ABC.abcC);
 		RI.m_ABC.abcA = (UT_sint32)((double) RI.m_ABC.abcA * (double)getResolution() /
 									((double)getDeviceResolution() * (double)GR_WIN32_USP_FONT_SCALING*m_fXYRatio) + 0.5);
 		RI.m_ABC.abcB = (UT_sint32)((double) RI.m_ABC.abcB * (double)getResolution() /
@@ -1571,8 +1657,13 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	}
 	
 	RI.m_bRejustify = true;
-	
 
+	if(iCount != iAdvSize)
+	{
+		delete[] pAdvances;
+		delete[] pGoffsets;
+	}
+	
 	if(hRes)
 	{
 		UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
@@ -2194,6 +2285,37 @@ void GR_Win32USPGraphics::drawChars(const UT_UCSChar* pChars,
 		delete [] pwChars;
 }
 
+void GR_Win32USPGraphics::setZoomPercentage(UT_uint32 iZoom)
+{
+	if(getZoomPercentage() != iZoom)
+	{
+		
+		GR_Graphics::setZoomPercentage(iZoom);
+
+		// now force remeasuring of character widths in any views that are associated with
+		// this graphics
+		XAP_App * pApp = XAP_App::getApp();
+		UT_return_if_fail( pApp );
+
+		UT_uint32 iFrameCount = pApp->getFrameCount();
+		for(UT_uint32 i = 0; i < iFrameCount; i++)
+		{
+			XAP_Frame * pFrame = pApp->getFrame(i);
+			if(!pFrame)
+				continue;
+
+			AV_View * pView = pFrame->getCurrentView();
+			if(pView)
+			{
+				GR_Graphics * pG = pView->getGraphics();
+
+				if(pG == this)
+					pView->remeasureCharsWithoutRebuild();
+			}
+		}
+	}
+}
+
 
 //////////////////////////////////////////////////////////////////////////////
 //
@@ -2410,7 +2532,6 @@ void GR_Win32USPFont::_clearAnyCachedInfo()
 	}
 }
 
-
 GR_Win32USPFont::~GR_Win32USPFont()
 {
 	GR_Win32USPGraphics::fScriptFreeCache(&m_sc);
