@@ -24,6 +24,7 @@
 
 #include "ut_assert.h"
 #include "ut_string.h"
+#include "ut_locale.h"
 #include "ut_Win32Locale.h"
 
 // Codes from ISO 3166
@@ -56,8 +57,8 @@ bool UT_getISO639Language(char * szLanguage)
 
 	UT_ASSERT(szLanguage);
 
-	GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SABBREVLANGNAME,szWinLang,4);
-	GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SISO639LANGNAME,szISOLang,3);
+	GetLocaleInfoA(LOCALE_USER_DEFAULT,LOCALE_SABBREVLANGNAME,szWinLang,4); //!TODO Using ANSI function
+	GetLocaleInfoA(LOCALE_USER_DEFAULT,LOCALE_SISO639LANGNAME,szISOLang,3); //!TODO Using ANSI function
 
 	// TODO Even Windows 2000 returns ISO 639-1 instead of ISO 639-2
 	// TODO Make sure we return correct ISO 639-2 for at least the locales Abi supports.
@@ -114,7 +115,7 @@ bool UT_getISO3166Country(char *szCountry)
 
 	UT_ASSERT(szCountry);
 
-	if (GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SISO3166CTRYNAME,szCountry,3))
+	if (GetLocaleInfoA(LOCALE_USER_DEFAULT,LOCALE_SISO3166CTRYNAME,szCountry,3)) //!TODO Using ANSI function
 	{
 		// Fix Serbia
 		if (!UT_stricmp(szCountry, "SP"))
@@ -124,7 +125,7 @@ bool UT_getISO3166Country(char *szCountry)
 
 		bSuccess = true;
 	}
-	else if (GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SABBREVCTRYNAME,szTmp,4))
+	else if (GetLocaleInfoA(LOCALE_USER_DEFAULT,LOCALE_SABBREVCTRYNAME,szTmp,4)) //!TODO Using ANSI function
 	{
 		// Convert to ISO 3166
 		char *psz;
@@ -147,37 +148,3 @@ bool UT_getISO3166Country(char *szCountry)
 
 	return bSuccess;
 }
-
-/*	
-	We build the default Abiword user's locale in the form ISO3166-ISO639 pair
-	(e.g. es-MX) from Windows locale information. For example, If the user's selected 
-	country and language are Argentina and Spanish we will build AR-ES. Unfortunally, 
-	it's likely that we do not have a string set for this variant of Spanish (there are
-	more than 19). What we do is we provide a fallback locale for this situation.
-	26/01/2003 Jordi 
-		
-	See bug 4174 for additional details
-*/
-const char* UT_getWin32FallBackStringSetLocale(const char* pLocale)
-{	
-	char	szLanguage[3];	
-	strncpy (szLanguage, pLocale,2);
-	szLanguage[2]='\0';
-		
-	if (stricmp(szLanguage,"es")==0)
-		return "es-ES";
-	
-	if (stricmp(szLanguage,"de")==0)
-		return "de-DE";
-	
-	if (stricmp(szLanguage,"en")==0)
-		return "en-US";		 	
-	
-	if (stricmp(szLanguage,"fr")==0)
-		return "fr-FR";		 
-	
-	if (stricmp(szLanguage,"nl")==0) 	
-		return "nl-NL";
-	
-	return NULL;
-}
