@@ -45,8 +45,7 @@ public:
 
 protected:
 
-	bool					_run_gtk_main(XAP_Frame * pFrame, void * pFSvoid,
-										  bool bCheckWritePermission,
+	bool					_run_gtk_main(XAP_Frame * pFrame,
 										  GtkWidget * filetypes_pulldown);
 	void 					_notifyError_OKOnly(XAP_Frame * pFrame,
 												XAP_String_Id sid);
@@ -56,10 +55,10 @@ protected:
 	bool 				_askOverwrite_YesNo(XAP_Frame * pFrame,
 												const char * fileName);
 
-	GtkFileSelection * m_FS;
+	GtkFileChooser * m_FC;
 	GtkWidget * m_preview;
 private:
-	bool				m_bExport;
+	bool				m_bSave;
 protected:
 	XAP_Frame *			m_pFrame;
 	char * 					m_szFinalPathnameCandidate;
