@@ -37,6 +37,10 @@
 #include "xap_UnixFontManager.h"
 #include "gr_UnixGraphics.h"
 
+#ifndef FT_ENCODING_ADOBE_CUSTOM
+#define FT_ENCODING_ADOBE_CUSTOM ft_encoding_adobe_custom
+#endif
+
 float fontPoints2float(UT_uint32 iSize, FT_Face pFace,
 				     UT_uint32 iFontPoints)
 {
@@ -102,14 +106,18 @@ UT_sint32 XAP_UnixFontHandle::measureUnremappedCharForCache(UT_UCSChar cChar) co
 	UT_sint32 width;
 	XftFaceLocker locker(m_font->getLayoutXftFont(GR_CharWidthsCache::CACHE_FONT_SIZE));
 	FT_Face pFace = locker.getFace();
-
+	if(m_font->isDingbat())
+	{
+		FT_Select_Charmap(pFace,FT_ENCODING_ADOBE_CUSTOM);
+	}
 	FT_UInt glyph_index = FT_Get_Char_Index(pFace, cChar);
 	FT_Error error =
 		FT_Load_Glyph(pFace, glyph_index,
 					FT_LOAD_LINEAR_DESIGN |
 					FT_LOAD_IGNORE_TRANSFORM |
 					FT_LOAD_NO_BITMAP | FT_LOAD_NO_SCALE);
-	if (error) {
+	if (error != 0) 
+	{
 		return 0;
 	}
 
@@ -137,13 +145,43 @@ bool XAP_UnixFontHandle::doesGlyphExist(UT_UCS4Char g)
 	return m_font->doesGlyphExist(g);
 }
 
+//
+// UT_Rect of glyph in Logical units.
+// rec.left = bearing Left (distance from origin to start)
+// rec.width = widht of the glyph
+// rec.top = distance from the origin to the top of the glyph
+// rec.height = total height of the glyph
+//
+bool  XAP_UnixFontHandle::glyphBox(UT_UCS4Char g, UT_Rect & rec) const
+{
+	XftFaceLocker locker(m_font->getLayoutXftFont(GR_CharWidthsCache::CACHE_FONT_SIZE));
+	FT_Face pFace = locker.getFace();
+
+	UT_UCS4Char cChar = g;
+	if(m_font->isSymbol()) cChar = static_cast<UT_UCS4Char>(adobeToUnicode(g));
+	FT_UInt glyph_index = FT_Get_Char_Index(pFace, cChar);
+	FT_Error error =
+		FT_Load_Glyph(pFace, glyph_index,
+					FT_LOAD_LINEAR_DESIGN |
+					FT_LOAD_IGNORE_TRANSFORM |
+					FT_LOAD_NO_BITMAP | FT_LOAD_NO_SCALE);
+	if (error) {
+		return false;
+	}
+	rec.left = pFace->glyph->metrics.horiBearingX;
+	rec.width = pFace->glyph->metrics.width;
+	rec.top = pFace->glyph->metrics.horiBearingY;
+	rec.height = pFace->glyph->metrics.height;
+	xxx_UT_DEBUGMSG((" left %d width %d top %d height %d \n",rec.left,rec.width,rec.top,rec.height));
+	return true;
+}
 /*******************************************************************/
 
 XAP_UnixFont::XAP_UnixFont(XAP_UnixFontManager * pFM)
 	: m_fontKey(NULL), 
 	  m_name(NULL), 
 	  m_style(STYLE_LAST),
-	  m_xlfd(NULL), 
+	  m_xlfd(""), 
 	  m_uniWidths(NULL),
 	  m_fontfile(NULL), 
 	  m_metricfile(NULL), 
@@ -167,7 +205,7 @@ XAP_UnixFont::XAP_UnixFont(const XAP_UnixFont & copy)
 
 	m_name = NULL;
 	m_style = STYLE_LAST;
-	m_xlfd = NULL;
+	m_xlfd = "";
 
 	m_fontfile = NULL;
 	m_metricfile = NULL;
@@ -189,23 +227,23 @@ XAP_UnixFont::XAP_UnixFont(const XAP_UnixFont & copy)
 	m_pFontManager = copy.m_pFontManager;
 
 	/* copy the fonts in our cache */
-	const UT_Vector & copyAllocFonts = copy.m_allocFonts;
+	const UT_GenericVector<allocFont *> & copyAllocFonts = copy.m_allocFonts;
 	for (UT_uint32 i = 0; i < m_allocFonts.getItemCount(); ++i)
 	  {
-		  allocFont *p = static_cast<allocFont *>(copyAllocFonts.getNthItem(i));
+		  allocFont *p = copyAllocFonts.getNthItem(i);
 		  insertFontInCache(p->pixelSize, XftFontCopy(GDK_DISPLAY(), p->xftFont));
+		  UT_ASSERT(m_allocFonts.getItemCount() < 100000);
+
 	  }
 }
 
 XAP_UnixFont::~XAP_UnixFont(void)
 {
-	xxx_UT_DEBUGMSG(("~XAP_UnixFont:: HELP! I'm getting deleted! Make sure this is at program startup or shutdown!!\n"));
+	xxx_UT_DEBUGMSG(("~XAP_UnixFont:: HELP! %x is  getting deleted! Make sure this is at program startup or shutdown!! \n",this));
 	FREEP(m_name);
 	FREEP(m_fontfile);
 	FREEP(m_metricfile);
-	FREEP(m_xlfd);
 	FREEP(m_fontKey);
-
 	//      UT_VECTOR_PURGEALL(allocFont *, m_allocFonts);
 	for (UT_uint32 i = 0; i < m_allocFonts.getItemCount(); ++i)
 	  {
@@ -222,6 +260,18 @@ XAP_UnixFont::~XAP_UnixFont(void)
 	_deleteEncodingTable();
 }
 
+
+bool XAP_UnixFont::isDingbat(void) const
+{
+	return m_bIsDingbat;
+}
+
+
+bool XAP_UnixFont::isSymbol(void) const
+{
+	return m_bIsSymbol;
+}
+
 bool XAP_UnixFont::doesGlyphExist(UT_UCS4Char g)
 {
 	// possibly need to call:
@@ -231,11 +281,24 @@ bool XAP_UnixFont::doesGlyphExist(UT_UCS4Char g)
 //
 	if(m_bIsSymbol)
 	{
-		g = static_cast<UT_UCS4Char>(adobeToUnicode(g));
+		UT_UCS4Char ig = g;
+		ig = static_cast<UT_UCS4Char>(adobeToUnicode(g));
+		if((ig != g) && (ig != 0))
+		{
+			return true;
+		}
+		g = ig;
+		xxx_UT_DEBUGMSG(("Symbol Glyph remapped to %x \n",g));
 	}
-	if(m_bIsDingbat)
+	if(isDingbat())
 	{
-		g = static_cast<UT_UCS4Char>(adobeDingbatsToUnicode(g));
+		UT_UCS4Char ig = static_cast<UT_UCS4Char>(adobeDingbatsToUnicode(g));
+		if((ig != g) && (ig != 0))
+		{
+			UT_DEBUGMSG(("Dingbat Glyph remapped to %x \n",g));
+			return true;
+		}
+		g = ig;
 	}
 	UT_return_val_if_fail (m_pXftFont, false);
 	return XftCharExists(GDK_DISPLAY(), m_pXftFont, (FcChar32)g);
@@ -262,6 +325,9 @@ bool XAP_UnixFont::openFileAs(const char *fontfile, const char *metricfile, cons
 	}
 	if(strstr(szLCFontName,"dingbat") != NULL)
 		m_bIsDingbat = true;
+	if(strstr(szLCFontName,"dingbat") != NULL)
+		m_bIsDingbat = true;
+
 	FREEP(szLCFontName);
 
 	// save to memebers
@@ -343,13 +409,15 @@ const char *XAP_UnixFont::getMetricfile(void) const
 
 void XAP_UnixFont::setXLFD(const char *xlfd)
 {
-	FREEP(m_xlfd);
-	UT_cloneString(m_xlfd, xlfd);
+	m_xlfd = xlfd;
+//	UT_ASSERT(m_xlfd.size() < 100);
+//	UT_DEBUGMSG(("m_xlfd set to %s \n",m_xlfd.c_str()));
+
 }
 
 const char *XAP_UnixFont::getXLFD(void) const
 {
-	return m_xlfd;
+	return m_xlfd.c_str();
 }
 
 
@@ -719,7 +787,7 @@ XftFont *XAP_UnixFont::getFontFromCache(UT_uint32 pixelsize, bool /*bIsLayout*/,
 
 	while (l < count)
 	  {
-		  entry = static_cast<allocFont *>(m_allocFonts.getNthItem(l));
+		  entry = m_allocFonts.getNthItem(l);
 		  if (entry && entry->pixelSize == pixelsize)
 			  return entry->xftFont;
 		  l++;
@@ -735,7 +803,8 @@ void XAP_UnixFont::insertFontInCache(UT_uint32 pixelsize, XftFont * pXftFont) co
 	entry->xftFont = pXftFont;
 
 	m_pXftFont = pXftFont;
-	m_allocFonts.push_back(static_cast<void *>(entry));
+	m_allocFonts.push_back(entry);
+	UT_ASSERT(m_allocFonts.getItemCount() < 100000);
 }
 
 /*! Loads pixelsize into cache - both layout & device versions. */
@@ -743,7 +812,7 @@ void XAP_UnixFont::fetchXftFont(UT_uint32 pixelsize) const
 {
 	XftFont * pXftFont;
 
-	FcPattern *fp = XftNameParse(m_xlfd);
+	FcPattern *fp = XftNameParse(m_xlfd.c_str());
 	FcResult result;
 	FcPattern *layout_fp = XftFontMatch(GDK_DISPLAY(),
 										DefaultScreen(GDK_DISPLAY()), fp,
