@@ -85,12 +85,16 @@ BOOL XAP_Win32Dialog_Image::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 	localizeControlText(XAP_RID_DIALOG_IMAGE_LBL_WRAPPING,	XAP_STRING_ID_DLG_Image_TextWrapping);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_LBL_PLACEMENT,	XAP_STRING_ID_DLG_Image_Placement);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_INLINE,	XAP_STRING_ID_DLG_Image_InLine);
+	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_FLOAT, XAP_STRING_ID_DLG_Image_WrappedNone);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_RIGHT,	XAP_STRING_ID_DLG_Image_WrappedRight);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_LEFT,	XAP_STRING_ID_DLG_Image_WrappedLeft);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_BOTHSIDES,XAP_STRING_ID_DLG_Image_WrappedBoth);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_PARAGRAPH,XAP_STRING_ID_DLG_Image_PlaceParagraph);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_COLUMN,	XAP_STRING_ID_DLG_Image_PlaceColumn);
 	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_PAGE,	XAP_STRING_ID_DLG_Image_PlacePage);
+	localizeControlText(XAP_RID_DIALOG_IMAGE_LBL_TYPE, XAP_STRING_ID_DLG_Image_WrapType);
+	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_SQUARE, XAP_STRING_ID_DLG_Image_SquareWrap);
+	localizeControlText(XAP_RID_DIALOG_IMAGE_RADIO_TIGHT, XAP_STRING_ID_DLG_Image_TightWrap);
 
 	// Initialize controls
 	setControlText( XAP_RID_DIALOG_IMAGE_EBX_HEIGHT, getHeightString() );
@@ -105,6 +109,11 @@ BOOL XAP_Win32Dialog_Image::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
         CheckRadioButton(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_INLINE, XAP_RID_DIALOG_IMAGE_RADIO_BOTHSIDES,
 		XAP_RID_DIALOG_IMAGE_RADIO_INLINE);
 	}
+	else if(getWrapping() == WRAP_NONE)
+	{
+        CheckRadioButton(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_INLINE, XAP_RID_DIALOG_IMAGE_RADIO_BOTHSIDES,
+		XAP_RID_DIALOG_IMAGE_RADIO_FLOAT);
+	}
 	else if(getWrapping() == WRAP_TEXTRIGHT)
 	{
         CheckRadioButton(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_INLINE, XAP_RID_DIALOG_IMAGE_RADIO_BOTHSIDES,
@@ -138,6 +147,18 @@ BOOL XAP_Win32Dialog_Image::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 		XAP_RID_DIALOG_IMAGE_RADIO_PAGE);
 	}
 
+	//Initialize type radio buttons
+	if(isTightWrap())
+	{
+        CheckRadioButton(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_SQUARE, XAP_RID_DIALOG_IMAGE_RADIO_TIGHT,
+		XAP_RID_DIALOG_IMAGE_RADIO_TIGHT);
+	}
+	else
+	{
+        CheckRadioButton(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_SQUARE, XAP_RID_DIALOG_IMAGE_RADIO_TIGHT,
+		XAP_RID_DIALOG_IMAGE_RADIO_SQUARE);
+	}
+
 	wrappingChanged();
 	return TRUE;
 }
@@ -161,6 +182,10 @@ BOOL XAP_Win32Dialog_Image::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		{
 			setWrapping(WRAP_INLINE);
 		}
+		else if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_FLOAT))
+		{
+			setWrapping(WRAP_NONE);
+		}
 		else if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_RIGHT))
 		{
 			setWrapping(WRAP_TEXTRIGHT);
@@ -174,18 +199,31 @@ BOOL XAP_Win32Dialog_Image::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 			setWrapping(WRAP_TEXTBOTH);
 		}
 
-		// change placement
-		if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_PARAGRAPH))
+		if(!IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_INLINE))
 		{
-			setPositionTo(POSITION_TO_PARAGRAPH);
-		}
-		else if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_COLUMN))
-		{
-			setPositionTo(POSITION_TO_COLUMN);
-		}
-		else if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_PAGE))
-		{
-			setPositionTo(POSITION_TO_PAGE);
+			// change placement if the inline option isn't selected
+			if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_PARAGRAPH))
+			{
+				setPositionTo(POSITION_TO_PARAGRAPH);
+			}
+			else if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_COLUMN))
+			{
+				setPositionTo(POSITION_TO_COLUMN);
+			}
+			else if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_PAGE))
+			{
+				setPositionTo(POSITION_TO_PAGE);
+			}
+
+			//change type if the inline option isn't selected
+			if(IsDlgButtonChecked(hWnd, XAP_RID_DIALOG_IMAGE_RADIO_SQUARE))
+			{
+				setTightWrap(false);
+			}
+			else
+			{
+				setTightWrap(true);
+			}
 		}
 
 		char buf[BUFSIZE];
@@ -203,11 +241,6 @@ BOOL XAP_Win32Dialog_Image::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		{
 			char buf[BUFSIZE];
 			GetDlgItemText( hWnd, wId, buf, BUFSIZE );
-
-			//let the user manually change dimensions too
-			if(!strstr(buf,UT_dimensionName(getPreferedUnits())))
-				strcat(buf,UT_dimensionName(getPreferedUnits()));
-
 			setHeight( buf );
 			setControlText( XAP_RID_DIALOG_IMAGE_EBX_HEIGHT, getHeightString() );
 			setControlText( XAP_RID_DIALOG_IMAGE_EBX_WIDTH, getWidthString() );
@@ -219,11 +252,6 @@ BOOL XAP_Win32Dialog_Image::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		{
 			char buf[BUFSIZE];
 			GetDlgItemText( hWnd, wId, buf, BUFSIZE );
-
-			//let the user manually change dimensions too
-			if(!strstr(buf,UT_dimensionName(getPreferedUnits())))
-				strcat(buf,UT_dimensionName(getPreferedUnits()));
-
 			setWidth( buf );
 			setControlText( XAP_RID_DIALOG_IMAGE_EBX_HEIGHT, getHeightString() );
 			setControlText( XAP_RID_DIALOG_IMAGE_EBX_WIDTH, getWidthString() );
@@ -234,8 +262,8 @@ BOOL XAP_Win32Dialog_Image::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		setPreserveAspect( isChecked(wId)!=0 );
 		return 1;
 
-	case XAP_RID_DIALOG_IMAGE_RADIO_INLINE:	case XAP_RID_DIALOG_IMAGE_RADIO_RIGHT:
-	 case XAP_RID_DIALOG_IMAGE_RADIO_LEFT: case XAP_RID_DIALOG_IMAGE_RADIO_BOTHSIDES:
+	case XAP_RID_DIALOG_IMAGE_RADIO_INLINE: case XAP_RID_DIALOG_IMAGE_RADIO_FLOAT:
+	case XAP_RID_DIALOG_IMAGE_RADIO_RIGHT: case XAP_RID_DIALOG_IMAGE_RADIO_LEFT: case XAP_RID_DIALOG_IMAGE_RADIO_BOTHSIDES:
 		wrappingChanged();
 		return 1;
 
@@ -287,11 +315,15 @@ void XAP_Win32Dialog_Image::wrappingChanged(void)
 		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_PARAGRAPH,FALSE);
 		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_COLUMN,FALSE);
 		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_PAGE,FALSE);
+		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_SQUARE,FALSE);
+		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_TIGHT,FALSE);
 	}
 	else
 	{
 		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_PARAGRAPH, TRUE);
 		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_COLUMN,TRUE);
 		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_PAGE,TRUE);
+		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_SQUARE,TRUE);
+		enableControl(XAP_RID_DIALOG_IMAGE_RADIO_TIGHT,TRUE);
 	}
 }
