@@ -29,6 +29,7 @@
 #endif
 
 #include "ut_debugmsg.h"
+#include "ut_Win32OS.h"
 #include "xap_ViewListener.h"
 #include "ev_EditMethod.h"
 #include "xav_View.h"
@@ -161,8 +162,7 @@ bool XAP_Win32FrameImpl::_updateTitle(void)
 	UT_String sTmp = pFrame->getTitle(INT_MAX);
 	sTmp += " - ";
 	sTmp += XAP_App::getApp()->getApplicationTitleForTitleBar();
-	
-	SetWindowText(m_hwndFrame, (AP_Win32App::s_fromUTF8ToWinLocale(sTmp.c_str())).c_str());
+	UT_SetWindowText(m_hwndFrame, sTmp.c_str());
 
 	return true;
 }
@@ -222,11 +222,10 @@ void XAP_Win32FrameImpl::_createTopLevelWindow(void)
 
 	XAP_Win32App *pWin32App = static_cast<XAP_Win32App *>(XAP_App::getApp());
 
-	m_hwndFrame = CreateWindow(pWin32App->getApplicationName(),
-							   pWin32App->getApplicationTitleForTitleBar(),
-							   WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
-							   iPosX, iPosY, iWidth, iHeight,
-							   NULL, NULL, pWin32App->getInstance(), NULL);
+	m_hwndFrame = UT_CreateWindowEx(0L, pWin32App->getApplicationName(), pWin32App->getApplicationTitleForTitleBar(),
+									WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
+									iPosX, iPosY, iWidth, iHeight,
+									NULL, NULL, pWin32App->getInstance(), NULL);
 	UT_ASSERT(m_hwndFrame);
 
 
@@ -532,23 +531,10 @@ UT_RGBColor XAP_Win32FrameImpl::getColorSelForeground () const
 #define MAXAPPNAME 256
 bool XAP_Win32FrameImpl::_RegisterClass(XAP_Win32App * app)
 {
-	WNDCLASSEX wndclass;
-
-	// register class for the frame window
-	wndclass.cbSize			= sizeof(wndclass);
-	wndclass.style			= CS_DBLCLKS;
-	wndclass.lpfnWndProc		= XAP_Win32FrameImpl::_FrameWndProc;
-	wndclass.cbClsExtra		= 0;
-	wndclass.cbWndExtra		= 0;
-	wndclass.hInstance		= app->getInstance();
-	wndclass.hIcon			= app->getIcon();
-	wndclass.hCursor			= LoadCursor(NULL, IDC_ARROW);
-	wndclass.hbrBackground		= (HBRUSH)(COLOR_BTNFACE+1);
-	wndclass.lpszMenuName		= NULL;
-	wndclass.lpszClassName		= app->getApplicationName();;
-	wndclass.hIconSm			= app->getSmallIcon();
-
-	ATOM a = RegisterClassEx(&wndclass);
+	ATOM a = UT_RegisterClassEx(CS_DBLCLKS, XAP_Win32FrameImpl::_FrameWndProc, app->getInstance(),
+								app->getIcon(), LoadCursor(NULL,IDC_ARROW), (HBRUSH)(COLOR_BTNFACE+1), app->getSmallIcon(),
+								NULL, app->getApplicationName());
+	
 	UT_return_val_if_fail(a, false);
 
 	return true;
@@ -563,11 +549,11 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	if (!f)
 	{
-		return DefWindowProc(hwnd,iMsg,wParam,lParam);
+		return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 	}
 	
 	XAP_Win32FrameImpl * fimpl = static_cast<XAP_Win32FrameImpl *>(f->getFrameImpl());
-	UT_return_val_if_fail(fimpl, DefWindowProc(hwnd,iMsg,wParam,lParam));
+	UT_return_val_if_fail(fimpl, UT_DefWindowProc(hwnd,iMsg,wParam,lParam));
 
 	AV_View * pView = NULL;
 
@@ -647,7 +633,7 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 					return 0;
 			}
 		}
-		return DefWindowProc(hwnd,iMsg,wParam,lParam);
+		return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 
 	case WM_INITMENU:
 		if (fimpl->m_pWin32Popup)
@@ -657,7 +643,7 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 		}
 		else if (fimpl->m_pWin32Menu->onInitMenu(f,pView,hwnd,(HMENU)wParam))
 			return 0;
-		return DefWindowProc(hwnd,iMsg,wParam,lParam);
+		return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 		
 	case WM_MEASUREITEM: 
 	{	
@@ -942,7 +928,7 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 					IEGraphicFileType iegft = IEGFT_Unknown;					
 					IE_ImpGraphic *pIEG;
 					FG_Graphic* pFG;
-					UT_Error errorCode = UT_ERROR;
+					UT_Error errorCode;
 
 					/*
 						The user may be dropping any kind of file
@@ -954,16 +940,26 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 					if(errorCode == UT_OK)
 					{						
 						errorCode = pIEG->importGraphic(szFileName, &pFG);
-
-						DELETEP(pIEG);
-						if(errorCode == UT_OK && pFG)
+						if(errorCode != UT_OK || !pFG)
 						{
-						  errorCode = pView->cmdInsertGraphic(pFG);
-						  DELETEP(pFG);
+							s_CouldNotLoadFileMessage(f, szFileName, errorCode);							
+							DELETEP(pIEG);
+							return 0;
 						}
-					  }
 
-					if (errorCode != UT_OK)
+						DELETEP(pIEG);					
+
+						errorCode = pView->cmdInsertGraphic(pFG);
+						if (errorCode != UT_OK)
+						{
+							s_CouldNotLoadFileMessage(f, szFileName, errorCode);							
+							DELETEP(pFG);
+							return 0;
+						}
+						
+						DELETEP(pFG);
+					  }
+					else
 					{	
 						// Check if the current document is empty.
 						if (f->isDirty() || f->getFilename() ||
@@ -972,9 +968,8 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 							pNewFrame = pApp->newFrame();
 							if (pNewFrame == NULL)
 							{
-							  errorCode = UT_ERROR;
-							  f->setStatusMessage("Could not open another window");
-							  return 0;
+								f->setStatusMessage("Could not open another window");
+								return 0;
 							}
 						}
 						else
@@ -982,22 +977,18 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 							pNewFrame = f;
 						}
 
-						errorCode = pNewFrame->loadDocument(szFileName, IEFT_Unknown);
-						if (errorCode != UT_OK)
+						UT_Error error = pNewFrame->loadDocument(szFileName, IEFT_Unknown);
+						if (error != UT_OK)
 						{
 							if (f != pNewFrame)
 								pNewFrame->close();
+							s_CouldNotLoadFileMessage(f, szFileName, error);
 						}
 						else
 						{
 							pNewFrame->show();
 						}
 					  }
-
-					if (errorCode != UT_OK)
-					  {
-					    s_CouldNotLoadFileMessage(f, szFileName, errorCode);
-					  }
 				}
 				else
 				{
@@ -1012,5 +1003,5 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	} /* switch (iMsg) */
 
-	return DefWindowProc(hwnd, iMsg, wParam, lParam);
+	return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 }
