@@ -54,6 +54,7 @@
 #include "gr_CharWidthsCache.h"
 #include "gr_ContextGlyph.h"
 #include "xav_Listener.h"
+#include "gr_EmbedManager.h"
 
 UT_Map * abi_ut_map_instance = 0;
 
@@ -130,6 +131,8 @@ XAP_App::~XAP_App()
 	// run thru and destroy all frames on our window list.
 	UT_VECTOR_PURGEALL(XAP_Frame *, m_vecFrames);
 
+	UT_VECTOR_PURGEALL(GR_EmbedManager  *, m_vecEmbedManagers);
+
 	FREEP(m_szAbiSuiteLibDir);
 	DELETEP(m_pEMC);
 	DELETEP(m_pBindingSet);
@@ -206,6 +209,73 @@ EV_Toolbar_ActionSet *XAP_App::getToolbarActionSet()
 	return m_pToolbarActionSet;
 }
 
+
+/*!
+ * Register an embeddable plugin with XAP_App
+ */
+UT_uint32 XAP_App::registerEmbeddable(GR_EmbedManager * pEmbed)
+{
+     UT_sint32 i=0;
+     bool bFound = false;
+     GR_EmbedManager * pCur = NULL;
+     for(i=0; !bFound && (i< static_cast<UT_sint32>(m_vecEmbedManagers.getItemCount())); i++)
+     {
+       pCur =  m_vecEmbedManagers.getNthItem(i);
+       if(UT_strcmp(pCur->getObjectType(),pEmbed->getObjectType()) == 0)
+       {
+	 bFound = true;
+       }
+     }
+     if(!bFound)
+     {
+       m_vecEmbedManagers.addItem(pEmbed);
+       return  m_vecEmbedManagers.getItemCount() - 1;
+     }
+     return 0xFFFFFFF;
+}
+
+
+/*!
+ * UnRegister an embeddable plugin with XAP_App. The plugin itself is 
+ * responsible for actually deleting the object.
+ */
+bool XAP_App::unRegisterEmbeddable(UT_uint32 uid)
+{
+  if(uid < m_vecEmbedManagers.getItemCount())
+    {
+      m_vecEmbedManagers.setNthItem(uid,NULL,NULL);
+      return true;
+    }
+  return false;
+}
+
+/*!
+ * Return a copy of the requested embedable plugin or a default manager.
+ * The calling routine must delete this.
+ */
+GR_EmbedManager * XAP_App:: getEmbeddableManager(GR_Graphics * pG, const char * szObjectType)
+{
+     UT_sint32 i=0;
+     bool bFound = false;
+     GR_EmbedManager * pCur = NULL;
+     for(i=0; !bFound && (i< static_cast<UT_sint32>(m_vecEmbedManagers.getItemCount())); i++)
+     {
+       pCur =  m_vecEmbedManagers.getNthItem(i);
+       UT_DEBUGMSG(("Look at Manager for Object type %s requested %s strcmp %d UT_strcmp %d \n",pCur->getObjectType(),szObjectType,UT_strcmp(pCur->getObjectType(),szObjectType),strcmp(pCur->getObjectType(),szObjectType) ));
+
+       if(UT_strcmp(pCur->getObjectType(),szObjectType) == 0)
+       {
+	 bFound = true;
+       }
+     }
+     if(bFound)
+     {
+       UT_DEBUGMSG(("Found a plugin of type %s \n",pCur->getObjectType()));
+       return pCur->create(pG);
+     }
+     return new GR_EmbedManager(pG);
+}
+ 
 bool XAP_App::initialize(const char * szKeyBindingsKey, const char * szKeyBindingsDefaultValue)
 {
 	// create application-wide resources that
@@ -379,7 +449,7 @@ bool XAP_App::removeListener(AV_ListenerId listenerId)
 /*!
  * Send notifications to all the registered plugins
  */
-bool XAP_App::notifyListeners(AV_View * pView, const AV_ChangeMask hint)
+bool XAP_App::notifyListeners(AV_View * pView, const AV_ChangeMask hint, void * pPrivateData)
 {
 	/*
 		App-specific logic calls this virtual method when relevant portions of 
@@ -420,10 +490,15 @@ bool XAP_App::notifyListeners(AV_View * pView, const AV_ChangeMask hint)
 	for (lid=0; lid<lidCount; lid++)
 	{
 		AV_Listener * pListener = static_cast<AV_Listener *>(m_vecPluginListeners.getNthItem(lid));
-		if(pListener)
+		if(pListener->getType()!= AV_LISTENER_PLUGIN_EXTRA )
 		{
 				pListener->notify(pView,hint);
 		}
+		else
+		{
+		  AV_ListenerExtra * pExtra = static_cast<AV_ListenerExtra *>(pListener);
+		  pExtra->notify(pView,hint,pPrivateData);
+		}
 	}
 
 	return true;
@@ -793,7 +868,10 @@ const char * XAP_App::getAbiSuiteLibDir() const
 
 bool XAP_App::findAbiSuiteLibFile(UT_String & path, const char * filename, const char * subdir)
 {
-	if (!filename) return false;
+	if (!filename)
+	{ 
+		return false;
+	}
 
 #if defined(WIN32)
 	const char * sep = "\\";
@@ -802,8 +880,8 @@ bool XAP_App::findAbiSuiteLibFile(UT_String & path, const char * filename, const
 #endif
 	bool bFound = false;
 
-	const char * dir = 0;
-	if (dir = getUserPrivateDirectory())
+	const char * dir = getUserPrivateDirectory();
+	if (dir)
 	{
 		path = dir;
 		if (subdir)
@@ -832,7 +910,10 @@ bool XAP_App::findAbiSuiteLibFile(UT_String & path, const char * filename, const
 
 bool XAP_App::findAbiSuiteAppFile(UT_String & path, const char * filename, const char * subdir) 
 {
-	if (!filename) return false;
+	if (!filename) 
+	{
+		return false;
+	}
 
 #if defined(WIN32)
 	const char * sep = "\\";
@@ -841,8 +922,8 @@ bool XAP_App::findAbiSuiteAppFile(UT_String & path, const char * filename, const
 #endif
 	bool bFound = false;
 
-	const char * dir = 0;
-	if (dir = getAbiSuiteAppDir())
+	const char * dir = getAbiSuiteAppDir();
+	if (dir)
 	{
 		path = dir;
 		if (subdir)
@@ -925,7 +1006,7 @@ void XAP_App::rememberFocussedFrame( void * pJustFocussedFrame)
 	notifyModelessDlgsOfActiveFrame(m_lastFocussedFrame);
 }
 
-UT_sint32 XAP_App::safefindFrame( XAP_Frame * f)
+UT_sint32 XAP_App::safefindFrame( XAP_Frame * f) const
 {
 	long ff = reinterpret_cast<long>(f);
 	UT_sint32 num_frames = m_vecFrames.getItemCount();
@@ -944,7 +1025,7 @@ void XAP_App::clearLastFocussedFrame()
 	m_lastFocussedFrame = static_cast<XAP_Frame *>(NULL);
 }
 
-XAP_Frame* XAP_App::getLastFocussedFrame() 
+XAP_Frame* XAP_App::getLastFocussedFrame() const
 {
 		if(m_lastFocussedFrame == static_cast<XAP_Frame *>(NULL))
 			return static_cast<XAP_Frame *>(NULL);
@@ -954,7 +1035,7 @@ XAP_Frame* XAP_App::getLastFocussedFrame()
 	return static_cast<XAP_Frame *>(NULL);
 }
 
-XAP_Frame * XAP_App::findValidFrame()
+XAP_Frame * XAP_App::findValidFrame() const
 {
 	XAP_Frame * validFrame =  getFrame(0);
 	return validFrame;
@@ -1312,6 +1393,34 @@ void XAP_App::setDefaultGraphicsId(UT_uint32 i)
 	}
 }
 
+/*!
+    Find the nearest matching font based on the provided parameters
+
+    TODO: we should use a more sophisticated description of fonts to allow for better matching
+    
+	NB: This is a convenience function; if you need to test for a number of fonts, you
+	    should use the code from this function to avoid recreating the graphics class. 
+*/
+const char* XAP_App::findNearestFont(const char* pszFontFamily,
+									 const char* pszFontStyle,
+									 const char* pszFontVariant,
+									 const char* pszFontWeight,
+									 const char* pszFontStretch,
+									 const char* pszFontSize)
+{
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail( pApp, pszFontFamily );
+	
+	GR_Graphics * pG = pApp->newDefaultScreenGraphics();
+	UT_return_val_if_fail( pG, pszFontFamily );
+
+	const char * pf = pG->findNearestFont(pszFontFamily, pszFontStyle, pszFontVariant,
+										   pszFontWeight, pszFontStretch, pszFontSize);
+
+	delete pG;
+	return pf;
+}
+
 #ifdef DEBUG
 /*!
     This function contains some of the most fundamental assumptions
