@@ -2,6 +2,7 @@
 
 /* AbiSource Application Framework
  * Copyright (C) 1998,1999 AbiSource, Inc.
+ * Copyright (C) 2004 Hubert Figuiere
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,6 +33,10 @@
 #endif
 #include "ut_vector.h"
 #include "ut_hash.h"
+#include "ut_Language.h"
+#include "ut_string_class.h"
+#include "xap_AppImpl.h"
+#include "xav_Listener.h"
 
 #define NUM_MODELESSID 39
 
@@ -42,6 +47,7 @@ class XAP_Toolbar_ControlFactory;
 class XAP_Frame;
 class EV_EditMethodContainer;
 class EV_EditBindingMap;
+class EV_EditEventMapper;
 class EV_Menu_ActionSet;
 class EV_Toolbar_ActionSet;
 class XAP_BindingSet;
@@ -50,10 +56,19 @@ class XAP_StringSet;
 class XAP_Dictionary;
 class PD_DocumentRange;
 class AV_View;
+class AD_Document;
 class XAP_EncodingManager;
 class UT_String;
 class XAP_Menu_Factory;
 class XAP_Toolbar_Factory;
+class UT_UUIDGenerator;
+class GR_GraphicsFactory;
+class GR_Graphics;
+class GR_AllocInfo;
+class XAP_InputModes;
+class AV_Listener;
+class GR_EmbedManager;
+
 
 /*****************************************************************
 ******************************************************************
@@ -87,23 +102,29 @@ public:
 	XAP_App(XAP_Args * pArgs, const char * szAppName);
 	virtual ~XAP_App();
 
+	XAP_AppImpl* getImpl()
+			{ return m_pImpl; }
+
 	virtual const char * getDefaultEncoding () const = 0 ;
 
-	virtual bool					initialize();
+	virtual bool					initialize(const char * szKeyBindingsKey, const char * szKeyBindingsDefaultValue);
 	virtual bool					rememberFrame(XAP_Frame* pFrame, XAP_Frame* pCloneOf = 0);
 	virtual bool					forgetFrame(XAP_Frame * pFrame);
 	virtual bool					forgetClones(XAP_Frame * pFrame);
-	virtual bool					getClones(UT_Vector *pvClonesCopy, XAP_Frame * pFrame);
+	virtual bool					getClones(UT_GenericVector<XAP_Frame*> *pvClonesCopy, XAP_Frame * pFrame);
 	virtual XAP_Frame * 			newFrame() = 0;
 	virtual void					reallyExit() = 0;
 
 	bool							updateClones(XAP_Frame * pFrame);
 
+	virtual void					notifyFrameCountChange (); // default is empty method
 	UT_uint32						getFrameCount() const;
 	XAP_Frame * 					getFrame(UT_uint32 ndx) const;
 	UT_sint32						findFrame(XAP_Frame * pFrame);
 	UT_sint32						findFrame(const char * szFilename);
 
+	void                            enumerateFrames(UT_Vector & v);
+	void                            enumerateDocuments(UT_Vector & v, const AD_Document * pExclude);
 	const char *					getApplicationTitleForTitleBar() const;
 	const char *					getApplicationName() const;
 	
@@ -120,7 +141,7 @@ public:
 	// only used in ispell builds because aspell doesn't suck...
 	bool				addWordToDict(const UT_UCSChar * pWord, UT_uint32 len);
 	bool				isWordInDict(const UT_UCSChar * pWord, UT_uint32 len) const;
-	void                            suggestWord(UT_Vector * pVecSuggestions, const UT_UCSChar * pWord, UT_uint32 lenWord);
+	void                            suggestWord(UT_GenericVector<UT_UCSChar*> * pVecSuggestions, const UT_UCSChar * pWord, UT_uint32 lenWord);
     XAP_Prefs *						getPrefs() const;
 	bool							getPrefsValue(const XML_Char * szKey, const XML_Char ** pszValue) const;
 	bool							getPrefsValue(const UT_String &stKey, UT_String &stValue) const;
@@ -135,15 +156,17 @@ public:
 	virtual const char *					getUserPrivateDirectory() = 0;
 	virtual const char *					getAbiSuiteLibDir() const;
 	virtual const char *					getAbiSuiteAppDir() const = 0;
+	virtual bool							findAbiSuiteLibFile(UT_String & path, const char * filename, const char * subdir = 0);
+	virtual bool							findAbiSuiteAppFile(UT_String & path, const char * filename, const char * subdir = 0); // doesn't check user-dir
 	virtual void							copyToClipboard(PD_DocumentRange * pDocRange, bool bUseClipboard = true) = 0;
 	virtual void							pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipboard, bool bHonorFormatting = true) = 0;
 	virtual bool							canPasteFromClipboard() = 0;
 	virtual void							cacheCurrentSelection(AV_View *) = 0;
 	void									rememberFocussedFrame(void * pJustFocussedFrame);
-	XAP_Frame *								getLastFocussedFrame();
-	XAP_Frame *								findValidFrame();
+	XAP_Frame *								getLastFocussedFrame() const;
+	XAP_Frame *								findValidFrame() const;
 	bool									safeCompare(XAP_Frame * lff, XAP_Frame * f);
-	UT_sint32								safefindFrame(XAP_Frame * f);
+	UT_sint32								safefindFrame(XAP_Frame * f) const;
 	void									clearLastFocussedFrame();
 	void									clearIdTable();
 	bool                                    setDebugBool(void)
@@ -190,8 +213,51 @@ public:
 	bool                            isBonoboRunning(void) const { return m_bBonoboRunning;}
 	virtual void							getDefaultGeometry(UT_uint32& width, UT_uint32& height, UT_uint32& flags){};
 
+	const UT_LangRecord *                   getKbdLanguage() const {return m_pKbdLang;}
+	void                                    setKbdLanguage(const char * pszLang);
+
+	UT_UUIDGenerator *                      getUUIDGenerator() const {return m_pUUIDGenerator;}
+
+	bool 									openURL(const char * url) 
+							{ return m_pImpl->openURL(url); }
+	bool 									openHelpURL(const char * url) 
+							{ return m_pImpl->openHelpURL(url); }
+	UT_String 								localizeHelpUrl (const char * pathBeforeLang, 
+												const char * pathAfterLang, const char * remoteURLbase)
+							{ return m_pImpl->localizeHelpUrl(pathBeforeLang, pathAfterLang, remoteURLbase); }
+
+	GR_GraphicsFactory *       getGraphicsFactory() const {return m_pGraphicsFactory;}
+	void                       setDefaultGraphicsId(UT_uint32 i);
+	                           /* primary graphics allocator */
+	GR_Graphics *              newGraphics(GR_AllocInfo &ai) const;
+	                           /*secondary graphics allocator; use only in special cases */
+	GR_Graphics *              newGraphics(UT_uint32 iClassId, GR_AllocInfo &ai) const;
+	virtual GR_Graphics *      newDefaultScreenGraphics() const {UT_ASSERT_HARMLESS( UT_NOT_IMPLEMENTED ); return NULL;}
+	
+	virtual UT_sint32			setInputMode(const char * szName);
+	const char *				getInputMode() const;
+	EV_EditEventMapper *		getEditEventMapper() const;
+	bool			            addListener(AV_Listener * pListener, AV_ListenerId * pListenerId);
+	bool			            removeListener(AV_ListenerId listenerId);
+	virtual bool	            notifyListeners(AV_View * pView, const AV_ChangeMask hint,void * pPrivateData = NULL);
+
+    UT_uint32                   registerEmbeddable(GR_EmbedManager * pEmbed);
+    bool                        unRegisterEmbeddable(UT_uint32 uid);
+	GR_EmbedManager *           getEmbeddableManager(GR_Graphics * pG, const char * szObjectType);
+
+	static const char*          findNearestFont(const char* pszFontFamily,
+												const char* pszFontStyle,
+												const char* pszFontVariant,
+												const char* pszFontWeight,
+												const char* pszFontStretch,
+												const char* pszFontSize);
+
 protected:
 	void									_setAbiSuiteLibDir(const char * sz);
+	virtual const char *                    _getKbdLanguage() {return NULL;}
+	void                                    _setUUIDGenerator(UT_UUIDGenerator * pG)
+	                                                                 {m_pUUIDGenerator = pG;}
+
 	XAP_Args *								m_pArgs;
 	const char *							m_szAppName;
 	const char *							m_szAbiSuiteLibDir;
@@ -203,8 +269,8 @@ protected:
 	XAP_Dictionary *						m_pDict;
 	XAP_Prefs *								m_prefs;			/* populated in AP_<platform>App::initialize() */
 
-	UT_Vector								m_vecFrames;
-	UT_StringPtrMap							m_hashClones;
+	UT_GenericVector<XAP_Frame*>			m_vecFrames;
+	UT_GenericStringMap<UT_GenericVector<XAP_Frame*>*>	m_hashClones;
 	XAP_Frame *								m_lastFocussedFrame;
 	XAP_Menu_Factory *                      m_pMenuFactory;
 	XAP_Toolbar_Factory *                   m_pToolbarFactory;
@@ -221,12 +287,23 @@ protected:
 	bool                                    m_bDebugBool;
 	bool                                    m_bBonoboRunning;
 	bool                                    m_bEnableSmoothScrolling;
+
 private:
+	const UT_LangRecord *                   m_pKbdLang;
+	UT_UUIDGenerator *                      m_pUUIDGenerator;
+
+	GR_GraphicsFactory *                    m_pGraphicsFactory;
+	UT_uint32                               m_iDefaultGraphicsId;
+	
+	XAP_InputModes *						m_pInputModes;
+	UT_GenericVector <GR_EmbedManager *>    m_vecEmbedManagers;	
 	XAP_App(const XAP_App&);				// should not even be called. Just to avoid a warning.
 	void operator=(const XAP_App&);
 #ifdef DEBUG
 	void _fundamentalAsserts() const;
 #endif
+	XAP_AppImpl* m_pImpl;
+    UT_GenericVector<AV_Listener *>         m_vecPluginListeners;
 };
 
 #endif /* XAP_APP_H */
