@@ -17,7 +17,6 @@
  * 02111-1307, USA.
  */
 
-
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -27,6 +26,7 @@
 #include "ut_types.h"
 #include "ut_string.h"
 #include "gr_Graphics.h"
+#include "gr_Painter.h"
 
 #include "xap_Draw_Symbol.h"
 
@@ -45,10 +45,8 @@ XAP_Draw_Symbol::XAP_Draw_Symbol(GR_Graphics * gc)
 {
 }
 
-
 XAP_Draw_Symbol::~XAP_Draw_Symbol(void)
 {
-  	DELETEP(m_pFont);
 }
 
 void XAP_Draw_Symbol::setWindowSize( UT_uint32 width, UT_uint32 height)
@@ -69,7 +67,6 @@ void XAP_Draw_Symbol::setAreaSize( UT_uint32 width, UT_uint32 height)
 	setFontStringarea();
 }
 
-
 void XAP_Draw_Symbol::setFontString( void )
 {
 	setFontToGC(m_gc, m_drawWidth / 32, 14);	
@@ -84,11 +81,9 @@ void XAP_Draw_Symbol::setFontToGC(GR_Graphics *p_gc, UT_uint32 MaxWidthAllowable
 
 	int SizeOK = false;
 
-#ifndef WITH_PANGO	
 	UT_UCSChar p_buffer[224];
 	for(int i = 0; i < 224; i++)
 		p_buffer[i] = i + 32;
-#endif
 
 	char temp[10];
 	while (!SizeOK)
@@ -135,12 +130,13 @@ void XAP_Draw_Symbol::setFontStringarea(void)
 	setFontToGC(m_areagc, m_drawareaWidth, 32);
 }
 
-
 void XAP_Draw_Symbol::draw(void)
 {
 	UT_ASSERT(m_gc);
 	UT_uint32 wwidth, wheight, yoff, xoff, x, y;
 	size_t i;
+
+	GR_Painter painter(m_gc);
 	
 	wwidth = m_drawWidth;
 	wheight = m_drawHeight;
@@ -148,7 +144,7 @@ void XAP_Draw_Symbol::draw(void)
 	UT_uint32 tmph = wheight / 7;
 	yoff = wheight / 14;
 	xoff = wwidth / 64;
-	m_gc->clearArea(0, 0, wwidth, wheight);
+	painter.clearArea(0, 0, wwidth, wheight);
 	int pos = 0;
 	
 	for (i = 0; i < m_vCharSet.size(); i += 2)
@@ -161,13 +157,7 @@ void XAP_Draw_Symbol::draw(void)
 			unsigned short w = m_gc->measureUnRemappedChar(j);
 			UT_uint32 x = (pos % 32) * tmpw + (tmpw - w) / 2;
 			UT_uint32 y = pos / 32 * tmph;
-#ifndef WITH_PANGO
-			m_gc->drawChars(&j, 0, 1, x, y);
-#else
-			PangoGlyphString * pGlyph = m_gc->getPangoGlyphString(c);
-			m_gc->drawPangoGlyphString(pGlyph, x, y);
-			pango_glyph_string_free(pGlyph);
-#endif			
+			painter.drawChars(&j, 0, 1, x, y);
 			++pos;
 		}
 	}
@@ -175,17 +165,18 @@ void XAP_Draw_Symbol::draw(void)
 	y = 0;
 	for(i = 0; i <= 6; i++)
 	{
-		m_gc->drawLine(0, y, wwidth - m_areagc->tlu(1), y);
+		painter.drawLine(0, y, wwidth - m_areagc->tlu(1), y);
 		y += tmph;
 	}
 
 	x = 0;
 	for(i = 0; i <= 31; i++)
 	{
-		m_gc->drawLine(x, 0, x, wheight - m_areagc->tlu(1));
+		painter.drawLine(x, 0, x, wheight - m_areagc->tlu(1));
 		x += tmpw;
 	}
 }
+
 UT_UCSChar XAP_Draw_Symbol::calcSymbolFromCoords(UT_uint32 ix, UT_uint32 iy)
 {
 	UT_uint32 index,count;
@@ -220,7 +211,6 @@ UT_UCSChar XAP_Draw_Symbol::calcSymbol(UT_uint32 x, UT_uint32 y)
 	return calcSymbolFromCoords(ix, iy);
 }
 
-
 void XAP_Draw_Symbol::calculatePosition(UT_UCSChar c, UT_uint32 &x, UT_uint32 &y)
 {
 	UT_uint32 index = 0;
@@ -262,6 +252,9 @@ void XAP_Draw_Symbol::drawarea(UT_UCSChar c, UT_UCSChar p)
 	UT_uint32 wwidth,wheight,x,y,cx,cy,px,py,swidth,sheight;
 	UT_uint32 cx1,cy1,px1,py1;
 
+	GR_Painter areaPainter(m_areagc);
+	GR_Painter painter(m_gc);
+
 	wwidth = m_drawareaWidth;
 	wheight = m_drawareaHeight;
 
@@ -269,27 +262,14 @@ void XAP_Draw_Symbol::drawarea(UT_UCSChar c, UT_UCSChar p)
 	// Note: That's bogus.  measureString will give us the horizontal advance of "c",
 	// but we need the bounding box of "c" instead.  To get it, we should use with FreeType face->bbox,
 	// in windows we should use the (FIXME: find the right name, it was something as getCharABC(...)
-	// NOTE: The Pango version has the same problem
-#ifndef WITH_PANGO 	
 	unsigned short w1 = m_areagc->measureUnRemappedChar(c);
-#else	
-	PangoGlyphString* pGlyph = m_areagc->getPangoGlyphString(c);
-	PangoRectangle ink_rect;
-	unsigned short w1;
-
-	pango_glyph_string_extents(pGlyph, m_pFont, &ink_rect, NULL);
-	w1 = ink_rect.width;
-#endif	
 
 	x = (m_drawareaWidth - w1) / 2;
 	y = (m_drawareaHeight - m_areagc->getFontHeight()) / 2;
 
-	m_areagc->clearArea(0, 0, wwidth, wheight);
-#ifndef WITH_PANGO	
-	m_areagc->drawChars(&c, 0, 1, x, y);
-#else
-	m_areagc->drawPangoGlyphString(pGlyph, x, y);
-#endif
+	areaPainter.clearArea(0, 0, wwidth, wheight);
+	areaPainter.drawChars(&c, 0, 1, x, y);
+
 	//
 	// Calculate the cordinates of the current and previous symbol
 	// along with the widths of the appropriate boxes.
@@ -317,25 +297,13 @@ void XAP_Draw_Symbol::drawarea(UT_UCSChar c, UT_UCSChar p)
 	py1 = py + tmph;
 
 	// Redraw the Previous Character in black on White
-	m_gc->clearArea(px + m_areagc->tlu(1), py + m_areagc->tlu(1), tmpw - m_areagc->tlu(1), tmph - m_areagc->tlu(1));
-#ifndef WITH_PANGO	
-	m_gc->drawChars(&p, 0, 1, px + (tmpw - wp) / 2, py);
-#else
-	PangoGlyphString * pGlyph2 = m_gc->getPangoGlyphString(p);
-	m_gc->drawPangoGlyphString(pGlyph2, px + m_areagc->tlu(2), py);
-#endif
+	painter.clearArea(px + m_areagc->tlu(1), py + m_areagc->tlu(1), tmpw - m_areagc->tlu(1), tmph - m_areagc->tlu(1));
+	painter.drawChars(&p, 0, 1, px + (tmpw - wp) / 2, py);
 
 	// Redraw the Current Character in black on Blue
 	UT_RGBColor colour(128, 128, 192);
-	m_gc->fillRect(colour, cx + m_areagc->tlu(1), cy + m_areagc->tlu(1), tmpw - m_areagc->tlu(1), tmph - m_areagc->tlu(1));
-#ifndef WITH_PANGO
-	m_gc->drawChars(&c, 0, 1, cx + (tmpw - wc) / 2, cy);
-#else
-	m_gc->drawPangoGlyphString(pGlyph, cx + m_areagc->tlu(2), cy);
-
-	pango_glyph_string_free(pGlyph);
-	pango_glyph_string_free(pGlyph2);
-#endif	
+	painter.fillRect(colour, cx + m_areagc->tlu(1), cy + m_areagc->tlu(1), tmpw - m_areagc->tlu(1), tmph - m_areagc->tlu(1));
+	painter.drawChars(&c, 0, 1, cx + (tmpw - wc) / 2, cy);
 }
 
 void XAP_Draw_Symbol::onLeftButtonDown(UT_sint32 x, UT_sint32 y)
