@@ -22,6 +22,7 @@
 #include "fp_ContainerObject.h"
 #include "fp_TableContainer.h"
 #include "fp_FootnoteContainer.h"
+#include "fp_TOCContainer.h"
 #include "fl_BlockLayout.h"
 #include "fp_Line.h"
 #include "fp_Column.h"
@@ -32,7 +33,10 @@
 
 fb_ColumnBreaker::fb_ColumnBreaker() :
 	m_pStartPage(NULL),
-	m_bStartFromStart(true)
+	m_bStartFromStart(true),
+	m_bReBreak(false),
+	m_pDocSec(NULL),
+	m_pCurrentBlock(NULL)
 {
 }
 
@@ -86,7 +90,8 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 	fl_ContainerLayout* pFirstLayout = NULL;
 	fp_Container* pOuterContainer = NULL;
 	fp_Column* pCurColumn = NULL;
-	UT_DEBUGMSG(("Doing ColumnBreak for section %x at page %x \n",pSL,m_pStartPage));
+	fp_Column* pPrevColumn = NULL;
+	xxx_UT_DEBUGMSG(("Doing ColumnBreak for section %x at page %x \n",pSL,m_pStartPage));
 //	UT_ASSERT(pSL->needsSectionBreak());
 	pSL->setNeedsSectionBreak(false,m_pStartPage);
 	pFirstLayout = pSL->getFirstLayout();
@@ -113,15 +118,36 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 		{
 			pFirstLayout = static_cast<fl_ContainerLayout *>(pOuterContainer->getSectionLayout());
 		}
-		else if(pOuterContainer)
+		else if(pOuterContainer &&pOuterContainer->getContainerType() == FP_CONTAINER_TABLE )
 		{
 			UT_ASSERT(pOuterContainer->getContainerType() == FP_CONTAINER_TABLE);
 			pFirstLayout = static_cast<fl_ContainerLayout *>(pOuterContainer->getSectionLayout());
 		}
+		else if(pOuterContainer)
+		{
+			UT_ASSERT(pOuterContainer->getContainerType() == FP_CONTAINER_TOC);
+			pFirstLayout = static_cast<fl_ContainerLayout *>(pOuterContainer->getSectionLayout());
+		}
 	}
-
+	fp_Page * pPrevPage = pCurColumn->getPage();
 	while (pCurColumn)
 	{
+		if(pCurColumn->getPage() != pPrevPage)
+		{
+			if(pPrevPage)
+			{
+//
+// check and update wrapped text around positioned objects
+//
+				fp_Container * pNextContainer = pPrevPage->updatePageForWrapping(pCurColumn);
+				UT_DEBUGMSG(("Returned container updatePage %x \n",pNextContainer));
+				if(pNextContainer != NULL)
+				{
+					pOuterContainer = pNextContainer;
+				}
+			}
+		}
+		pPrevPage = pCurColumn->getPage();
 		fp_Container* pFirstContainerToKeep = pOuterContainer;
 		xxx_UT_DEBUGMSG(("SEVIOR: first to keep 1 %x \n",pFirstContainerToKeep));
 		fp_Container* pLastContainerToKeep = NULL;
@@ -176,19 +202,27 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 			{
 //
 // skip this! We've already taken it's height into account.
+//
+				pCurContainer = _getNext(pCurContainer);
+				continue;
+			}
+			if(pCurContainer->getContainerType() == FP_CONTAINER_FRAME)
+			{
+//
+// skip this! It's height is ignored in the layout
 //
 				pCurContainer = _getNext(pCurContainer);
 				continue;
 			}
 			UT_sint32 iContainerHeight = 0;
 			UT_sint32 iContainerMarginAfter = pCurContainer->getMarginAfter();
-			fp_TableContainer * pVTab = NULL;
-			if(pCurContainer->getContainerType() == FP_CONTAINER_TABLE)
+			fp_Container * pVTab = NULL;
+			if(pCurContainer->isVBreakable())
 			{
-				pVTab = static_cast<fp_TableContainer *>(pCurContainer);
-				UT_sint32 iOldBreak = pVTab->getLastWantedVBreak(); 
-				UT_DEBUGMSG(("pVTab height is %d \n",pVTab->getHeight()));
-				if( (iOldBreak > 0) && !pVTab->isThisBroken())
+				pVTab = static_cast<fp_Container *>(pCurContainer);
+				UT_sint32 iOldBreak = _getLastWantedVBreak(pVTab); 
+				xxx_UT_DEBUGMSG(("pVTab height is %d \n",pVTab->getHeight()));
+				if( (iOldBreak > 0) && !_isThisBroken(pVTab))
 				{
 					UT_sint32 iAvail = iMaxColHeight - iWorkingColHeight - iContainerMarginAfter;
 					UT_sint32 iBreakAt = pVTab->wantVBreakAt(iAvail-1);
@@ -197,12 +231,23 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 //
 // Rebreak it
 //
-						pVTab->deleteBrokenTables();
+						if(pVTab->getContainerType() == FP_CONTAINER_TABLE)
+						{
+							static_cast<fp_TableContainer *>(pVTab)->deleteBrokenTables(true,true);
+						}
+						else
+						{
+							static_cast<fp_TOCContainer *>(pVTab)->deleteBrokenTOCs(true);
+						}
 						pVTab->VBreakAt(0);
-						pVTab->setLastWantedVBreak(iBreakAt);
+						_setLastWantedVBreak(pVTab,iBreakAt);
 					}
 				}
 				iContainerHeight = pVTab->getHeight();
+				if(pVTab->getContainerType() == FP_CONTAINER_TOC)
+				{
+					iContainerHeight = static_cast<fp_TOCContainer *>(pVTab)->getHeight();
+				}
 			}
 			else
 			{
@@ -213,6 +258,14 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				pCurContainer->getContainerType() == FP_CONTAINER_LINE)
 			{
 				fp_Line* pCurLine = static_cast<fp_Line *>(pCurContainer);
+				if(pCurLine->isSameYAsPrevious())
+				{
+//
+// No vertical changes to skip this
+//
+					pCurContainer = _getNext(pCurContainer);
+					continue;
+				}
 				// Excellent.  If we have a footnote, we can start deducting
 				// from the working height if the footnote container is not on
 				// this page.
@@ -221,14 +274,14 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					// Ok.  Now, deduct the proper amount from iMaxColHeight.
 
 					// OK get a vector of the footnote containers in this line.
-					UT_Vector vecFootnotes;
+					UT_GenericVector<fp_FootnoteContainer*> vecFootnotes;
 					pCurLine->getFootnoteContainers(&vecFootnotes);
 					fp_Page *pCurPage = pCurColumn->getPage();
 					// Now loop through all these and add them to the height.
 					UT_sint32 i =0;
 					for(i=0; i< static_cast<UT_sint32>(vecFootnotes.getItemCount());i++)
 					{
-						fp_FootnoteContainer * pFC = static_cast<fp_FootnoteContainer *>(vecFootnotes.getNthItem(i));
+						fp_FootnoteContainer * pFC = vecFootnotes.getNthItem(i);
 						if(pFC && ((pFC->getPage() == NULL) || (pFC->getPage() != pCurPage)))
 						{
 							iFootnoteHeight += pFC->getHeight();
@@ -238,6 +291,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					iWorkingColHeight += iFootnoteHeight;
 				}
 			}
+
 			if ((iWorkingColHeight + iTotalContainerSpace) > iMaxColHeight)
 			{
 				pOffendingContainer = pCurContainer;
@@ -253,10 +307,10 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				*/
 
 				xxx_UT_DEBUGMSG(("SEVIOR: iWorkingColHeight %d iTotalContainerSpace %d iMaxColHeight %d pCurContainer %x height %d \n",iWorkingColHeight,iTotalContainerSpace,iMaxColHeight,   pCurContainer,  iContainerHeight));
-				if(pOffendingContainer->getContainerType() == FP_CONTAINER_TABLE)
+				if(pOffendingContainer->isVBreakable())
 				{
-					xxx_UT_DEBUGMSG(("fb_ColumnBreak 1 Broken Table num %d \n",(static_cast<fp_TableContainer *>(pOffendingContainer))->getBrokenNumber()));
-					if (_breakTable(pOffendingContainer,
+					xxx_UT_DEBUGMSG(("fb_ColumnBreak 1 Broken Container num %d \n",(static_cast<fp_TableContainer *>(pOffendingContainer))->getBrokenNumber()));
+					if (_breakCON(pOffendingContainer,
 									pLastContainerToKeep,
 									iMaxColHeight, iWorkingColHeight, 
 									iContainerMarginAfter))
@@ -264,7 +318,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 						pPrevWorking = pCurContainer;
 						pCurContainer = pOffendingContainer;
 					}
-					else
+					else if(pOffendingContainer->getContainerType() == FP_CONTAINER_TABLE)
 					{
 //
 // Can't break the table so bump it.
@@ -275,6 +329,18 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 						xxx_UT_DEBUGMSG(("Can't break table. pCurContainer %x pTabOffend %x pLastContainerToKeep %x \n",pCurContainer,pTabOffend,pLastContainerToKeep));
 						break;
 					}
+					else
+					{
+//
+// Can't break the TOC so bump it.
+//
+						pCurContainer = pOffendingContainer;
+						UT_ASSERT(pCurContainer->getContainerType() == FP_CONTAINER_TOC);
+						fp_TOCContainer * pTOCOffend = static_cast<fp_TOCContainer *>(pOffendingContainer);
+						pLastContainerToKeep = pTOCOffend->getPrevContainerInSection();
+					    UT_DEBUGMSG(("Can't break TOC. pCurContainer %x pTabOffend %x pLastContainerToKeep %x \n",pCurContainer,pTOCOffend,pLastContainerToKeep));
+						break;
+					}
 				}
 				if (pOffendingContainer == pFirstContainerToKeep)
 				{
@@ -290,7 +356,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				{
 					UT_uint32 iWidows = 0;
 					UT_uint32 iOrphans = 0;
-					bool bIsTable = false;
+					bool bIsTableOrTOC = false;
 					fl_BlockLayout* pBlock = NULL;
 					fl_ContainerLayout * pConLayout = NULL;
 					if(pOffendingContainer->getContainerType() == FP_CONTAINER_LINE)
@@ -314,16 +380,16 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 						UT_ASSERT(pLastContainerToKeep);
 //
 // All this deals with widows and orphans. We don't need this for
-// tables
+// tables or TOCs
 //
 						break;
 					}
 					else
 					{
-						bIsTable = true;
+						bIsTableOrTOC = true;
 						pConLayout = static_cast<fl_ContainerLayout *>(pOffendingContainer->getSectionLayout());
 					}
-					if(bIsTable)
+					if(bIsTableOrTOC)
 					{
 
 						pLastContainerToKeep = pOffendingContainer->getPrevContainerInSection();
@@ -429,7 +495,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 							//
 							for (fp_TableContainer * pTab = 
 								 static_cast<fp_TableContainer *>(pLastContainerToKeep);
-								 pTab; 
+								 pTab;
 								 pTab=static_cast<fp_TableContainer *>(pTab->getNext()))
 							{
 								if(pTab == pFirstContainerToKeep)
@@ -442,6 +508,36 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 								
 							}
 						}
+//
+// If this happens to be a TOC better get the LAST of the broken TOCs
+//
+						if(pLastContainerToKeep && pLastContainerToKeep->getContainerType() == 
+						                    FP_CONTAINER_TOC && 
+						   pFirstContainerToKeep->getContainerType() == 
+						                    FP_CONTAINER_TOC)
+						{
+							//
+							// Can get into trouble if the pFirstContainer
+							// points to a broken table after
+							// this. Check for this.
+							// Not sure why this code is meant to work at all
+#if 0
+							for (fp_TOCContainer * pTOC = 
+								 static_cast<fp_TOCContainer *>(pLastContainerToKeep);
+								 pTOC;
+								 pTOC=static_cast<fp_TOCContainer *>(pTOC->getNext()))
+							{
+								if(pTOC == pFirstContainerToKeep)
+								{
+									xxx_UT_DEBUGMSG(("SEVIOR: FirstContainer l325 %x \n",pFirstContainerToKeep)); 
+									pFirstContainerToKeep = 
+										pLastContainerToKeep;
+									break;
+								}
+								
+							}
+#endif
+						}
 						xxx_UT_DEBUGMSG(("SEVIOR: Set lasttokeep 5 %x \n",pLastContainerToKeep));
 					}
 					else if (
@@ -560,14 +656,43 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 						pCurColumn->addContainer(pCurContainer);
 					}
 				}
-				UT_ASSERT(pCurContainer->getContainer());
-				if(pCurContainer->getContainer()->findCon(pCurContainer) >= 0)
+//				UT_ASSERT(pCurContainer->getContainer());
+				if(pCurContainer->getContainer() && pCurContainer->getContainer()->findCon(pCurContainer) >= 0)
 				{
-					static_cast<fp_VerticalContainer *>(pCurContainer->getContainer())->removeContainer(pCurContainer);
+					fp_VerticalContainer *pVert = static_cast<fp_VerticalContainer *>(pCurContainer->getContainer());
+#if 0
+					if(pCurContainer->getContainer() != pCurColumn)
+					{
+						pCurColumn->addContainer(pCurContainer);
+					}
+#endif
+					pVert->removeContainer(pCurContainer,true);
 				}
 				if(pCurContainer->getContainer() != pCurColumn)
 				{
-					pCurColumn->addContainer(pCurContainer);
+					if(pCurContainer->getContainerType() != FP_CONTAINER_FRAME)
+					{
+						pCurColumn->addContainer(pCurContainer);
+					}
+					else
+					{
+						fp_Page * pPage = pCurContainer->getPage();
+						fp_Page * pNextPage = pCurColumn->getPage();
+						if(pNextPage && pPage && (pNextPage != pPage))
+						{
+							FL_DocLayout * pDL = m_pDocSec->getDocLayout();
+							fp_FrameContainer * pFC = reinterpret_cast<fp_FrameContainer *>(pCurContainer);
+							if((pDL->findPage(pPage) >= 0) && (pDL->findPage(pNextPage) >= 0))
+							{
+								if((pPage->findFrameContainer(pFC) >=0) &&
+								   (pNextPage->findFrameContainer(pFC) < 0))
+								{
+									pPage->removeFrameContainer(pFC);
+									pNextPage->insertFrameContainer(pFC);
+								}				
+							}			
+						}
+					}
 				}
 			}
 //
@@ -580,7 +705,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				if(pCurLine->containsFootnoteReference())
 				{
 					// OK get a vector of the footnote containers in this line.
-					UT_Vector vecFootnotes;
+					UT_GenericVector<fp_FootnoteContainer*> vecFootnotes;
 					pCurLine->getFootnoteContainers(&vecFootnotes);
 				
 					// Now loop through all these and check they're on this
@@ -590,7 +715,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					UT_sint32 i =0;
 					for(i=0; i< static_cast<UT_sint32>(vecFootnotes.getItemCount());i++)
 					{
-						fp_FootnoteContainer * pFC = static_cast<fp_FootnoteContainer *>(vecFootnotes.getNthItem(i));
+						fp_FootnoteContainer * pFC = vecFootnotes.getNthItem(i);
 						if(pFC != NULL)
 						{
 							fp_Page * myPage = pFC->getPage();
@@ -626,7 +751,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					if(pCurTable->containsFootnoteReference())
 					{
 						// OK get a vector of the footnote containers in this line.
-						UT_Vector vecFootnotes;
+						UT_GenericVector<fp_FootnoteContainer*> vecFootnotes;
 						pCurTable->getFootnoteContainers(&vecFootnotes);
 						
 					// Now loop through all these and check they're on this
@@ -637,7 +762,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 						for(i=0; i< static_cast<UT_sint32>(vecFootnotes.getItemCount());i++)
 						{
 							xxx_UT_DEBUGMSG(("Found reference %d in broken table %x \n",i,pCurTable));
-							fp_FootnoteContainer * pFC = static_cast<fp_FootnoteContainer *>(vecFootnotes.getNthItem(i));
+							fp_FootnoteContainer * pFC = vecFootnotes.getNthItem(i);
 							fp_Page * myPage = pFC->getPage();
 							xxx_UT_DEBUGMSG(("Footnote %x is on Page %x \n",pFC,myPage));
 							if(myPage != pCurPage)
@@ -660,6 +785,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 //
 // Code to fix order in the column
 //
+#if 1
 			if((pCurColumn->findCon(pCurContainer) >= 0) && (pCurColumn->findCon(pCurContainer)  != conPos))
 			{
 				xxx_UT_DEBUGMSG(("fb_ColumnBreaker:Container out of order. Should be at %d is at %d \n",conPos,pCurColumn->findCon(pCurContainer)));
@@ -670,6 +796,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				xxx_UT_DEBUGMSG(("fb_ColumnBreak: Insert at %d now num cons \n",conPos,pCurColumn->countCons()));
 //				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 			}
+#endif
 			conPos++;
 			if (pCurContainer == pLastContainerToKeep)
 				break;
@@ -681,7 +808,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					UT_DEBUGMSG((" CurContainer %x type %d \n",pCurContainer,pCurContainer->getContainerType()));
 					UT_DEBUGMSG((" FirstContainer to keep %x Last container to keep %x \n",pTab,pLastContainerToKeep));
 					UT_DEBUGMSG(("Try to recover.... \n"));
-//					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 					pLastContainerToKeep = NULL;
 					break;
 				}
@@ -726,7 +853,6 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				   static_cast<fp_Line *>(pLastContainerToKeep)
 				        ->containsForcedPageBreak() &&
 				   (pNextColumn->getPage() == pPrevPage));
-
 			if(pCurColumn != NULL && (pCurColumn == pNextColumn))
 			{
 				pCurColumn->layout();
@@ -734,21 +860,38 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 			// Bump content down the columns
 			while (pCurColumn != NULL && pCurColumn != pNextColumn)
 			{
+				xxx_UT_DEBUGMSG(("Start of bump loop pNextColumn %x \n",pNextColumn));
+				bool isTOCTABLE = ((pOuterContainer->getContainerType() == FP_CONTAINER_TABLE) || (pOuterContainer->getContainerType() == FP_CONTAINER_TABLE));
 				pCurColumn->bumpContainers(pLastContainerToKeep);
+
 				pCurColumn->layout();
 //
-// Layout might delete a broken table. Check for this.
+// Layout might delete a broken table or TOC. Check for this.
 //
 				fp_Container * pCon = pCurColumn->getLastContainer();
+				bool bDoneTabTOC = false;
 				if(pCon && pCon->getContainerType() == FP_CONTAINER_TABLE)
 				{
+				        bDoneTabTOC = true;
 					pOuterContainer = _getNext(pCon);
 				}
+				if(pCon && pCon->getContainerType() == FP_CONTAINER_TOC)
+				{
+					xxx_UT_DEBUGMSG(("Last Con was TOC \n"));
+				        bDoneTabTOC = true;
+					pOuterContainer = _getNext(pCon);
+				}
+				if(!bDoneTabTOC && isTOCTABLE)
+				{
+				        pOuterContainer = _getNext(pCon);
+				}
 //				pCurColumn->validate();
+				pPrevColumn = pCurColumn;
 				pCurColumn = static_cast<fp_Column *>(pCurColumn->getNext());
 					// This is only relevant for the initial column. All
 					// other columns should flush their entire content.
 				pLastContainerToKeep = NULL;
+				xxx_UT_DEBUGMSG(("Last of bump loop pCurColumn %x \n",pCurColumn));
 			}
 
 		}
@@ -758,24 +901,35 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 					  (pCurColumn->getLastContainer() == pLastContainerToKeep));
 
 			bool bTableTest = false;
+			bool bTOCTest = false;
 			if(pOuterContainer)
 				bTableTest = (pOuterContainer->getContainerType() == 
 							  FP_CONTAINER_TABLE);
+			if(pOuterContainer)
+				bTOCTest = (pOuterContainer->getContainerType() == 
+							  FP_CONTAINER_TOC);
 
 			pCurColumn->layout();
 
-			// pCurColumn->layout() might delete a broken table; fixup.
+			// pCurColumn->layout() might delete a broken table or TOC; fixup.
 			if(bTableTest &&
 			   pOuterContainer != pCurColumn->getLastContainer())
 			{
 				pOuterContainer =  _getNext(pCurColumn->getLastContainer());
 				bTableTest = true;
 			}
-//			pCurColumn->validate();
-			pCurColumn = static_cast<fp_Column *>(pCurColumn->getNext());
-			if(pCurColumn == NULL && bTableTest && pOuterContainer != NULL)
+			if(bTOCTest &&
+			   pOuterContainer != pCurColumn->getLastContainer())
 			{
-				if(pOuterContainer->getContainerType() == FP_CONTAINER_TABLE)
+				pOuterContainer =  _getNext(pCurColumn->getLastContainer());
+				bTOCTest = true;
+			}
+//			pCurColumn->validate();
+			pPrevColumn = pCurColumn;
+			pCurColumn = static_cast<fp_Column *>(pCurColumn->getNext());
+			if(pCurColumn == NULL && (bTableTest || bTOCTest) && pOuterContainer != NULL)
+			{
+				if(pOuterContainer->isVBreakable())
 				{
 					pCurColumn = static_cast<fp_Column *>(pSL->getNewContainer(NULL));
 				}
@@ -787,7 +941,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 			else if(pCurColumn == NULL && pLastContainerToKeep && _getNext(pLastContainerToKeep))
 			{
 				fp_Container * pCon = _getNext(pLastContainerToKeep);
-				if(pCon->getContainerType() == FP_CONTAINER_TABLE)
+				if(pCon->isVBreakable())
 				{
 					pCurColumn =  static_cast<fp_Column *>(pSL->getNewContainer(NULL));
 				}
@@ -800,7 +954,7 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 		if(pCurColumn == NULL && pLastContainerToKeep && _getNext(pLastContainerToKeep))
 		{
 			fp_Container * pCon = _getNext(pLastContainerToKeep);
-			if(pCon->getContainerType() == FP_CONTAINER_TABLE)
+			if(pCon->isVBreakable())
 			{
 				pCurColumn =  static_cast<fp_Column *>(pSL->getNewContainer(NULL));
 			}
@@ -809,9 +963,23 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 				pCurColumn =  static_cast<fp_Column *>(pSL->getNewContainer(pCon));
 			}
 		}
+//
+// check for wrapped objects on last page
+//
 		if(pCurColumn == NULL)
 		{
 			UT_ASSERT(pOuterContainer == NULL);
+			fp_Page * pPage = pPrevColumn->getPage();
+			fp_Container * pNextContainer = pPage->updatePageForWrapping(pCurColumn);
+			UT_DEBUGMSG(("Returned container updatePage %x \n",pNextContainer));
+			if(pNextContainer == NULL)
+			{
+				pCurColumn = NULL;
+			}
+			else
+			{
+				pOuterContainer = pNextContainer;
+			}
 		}
 //
 // Loop back for next pCurContainer. Finish if pCurColumn == NULL.
@@ -839,6 +1007,64 @@ UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
 	return 0; // TODO return code
 }
 
+
+bool fb_ColumnBreaker::_isThisBroken(fp_Container * pCon)
+{
+	if(pCon->getContainerType() == FP_CONTAINER_TABLE)
+	{
+		return static_cast<fp_TableContainer *>(pCon)->isThisBroken();
+	}
+	else
+	{
+		return static_cast<fp_TOCContainer *>(pCon)->isThisBroken();
+	}
+}
+
+void fb_ColumnBreaker::_setLastWantedVBreak(fp_Container * pCon, UT_sint32 iBreakAt)
+{
+	if(pCon->getContainerType() == FP_CONTAINER_TABLE)
+	{
+		static_cast<fp_TableContainer *>(pCon)->setLastWantedVBreak(iBreakAt);
+	}
+	else
+	{
+		static_cast<fp_TOCContainer *>(pCon)->setLastWantedVBreak(iBreakAt);
+	}
+}
+
+
+UT_sint32 fb_ColumnBreaker::_getLastWantedVBreak(fp_Container * pCon)
+{
+	if(pCon->getContainerType() == FP_CONTAINER_TABLE)
+	{
+		return static_cast<fp_TableContainer *>(pCon)->getLastWantedVBreak();
+	}
+	fp_TOCContainer * pTOC = static_cast<fp_TOCContainer *>(pCon);
+	UT_sint32 i = pTOC->getLastWantedVBreak();
+	return i;
+}
+
+/*!
+ * Breaks the given breakabke container, if appropriate.  
+ * \return true iff the table was broken.
+ */
+bool fb_ColumnBreaker::_breakCON(fp_Container*& pOffendingContainer,
+								   fp_Container*& pLastContainerToKeep,
+								   int iMaxColHeight, 
+								   int iWorkingColHeight,
+								   int iContainerMarginAfter)
+{
+	if(pOffendingContainer->getContainerType() == FP_CONTAINER_TABLE)
+	{
+		return _breakTable(pOffendingContainer,pLastContainerToKeep,
+						   iMaxColHeight,iWorkingColHeight,
+						   iContainerMarginAfter);
+	}
+	return _breakTOC(pOffendingContainer,pLastContainerToKeep,
+						   iMaxColHeight,iWorkingColHeight,
+						   iContainerMarginAfter);
+}
+
 /*!
  * Breaks the given table, if appropriate.  
  * \return true iff the table was broken.
@@ -851,7 +1077,7 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 {
 	bool bDoTableBreak;
 
-    UT_DEBUGMSG(("breakTable:!!!!!!!!!!!! Offending container is table %x \n",pOffendingContainer));
+    xxx_UT_DEBUGMSG(("breakTable:!!!!!!!!!!!! Offending Table is %x \n",pOffendingContainer));
 	fp_TableContainer * pTab = static_cast<fp_TableContainer *>(pOffendingContainer);
 	if(!pTab->isThisBroken())
 	{
@@ -859,7 +1085,7 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 // This is the first of this table set. Clear the old broken tables
 // and rebreak.
 //
-		pTab->deleteBrokenTables();
+		pTab->deleteBrokenTables(true,true);
 		bDoTableBreak = true;
 		xxx_UT_DEBUGMSG(("SEVIOR: Need Table Break 1 \n"));
 		xxx_UT_DEBUGMSG(("firstbroke %x lastbroke %x \n",pTab->getFirstBrokenTable(),pTab->getLastBrokenTable()));
@@ -876,7 +1102,7 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 	UT_sint32 iAvail = iMaxColHeight - iWorkingColHeight - iContainerMarginAfter;
 	UT_sint32 iBreakAt = pTab->wantVBreakAt(iAvail-1);
 	pTab->setLastWantedVBreak(iBreakAt);
-	UT_DEBUGMSG(("breakTable column: iAvail %d actual break at %d \n",iAvail,iBreakAt));
+	xxx_UT_DEBUGMSG(("breakTable column: iAvail %d actual break at %d \n",iAvail,iBreakAt));
 //
 // Look to see if the table can be broken. If iBreakAt < 0 we have to bump 
 // the whole table into the next column.
@@ -909,7 +1135,7 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 			{
 				xxx_UT_DEBUGMSG(("SEVIOR: Container of next %d \n",pNext->getContainerType()));
 			}
-			pTab->deleteBrokenTables();
+			pTab->deleteBrokenTables(true,true);
 			pTab->VBreakAt(0);
 		}
 //
@@ -937,9 +1163,9 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 // table in the column. This then becomes the offending container and will
 // be bumped into the next column. Pretty cool eh ? :-)
 //
-			pOffendingContainer = static_cast<fp_Container *>(pBroke->VBreakAt(iBreakAt));
+			fp_TableContainer * pNewTab = static_cast<fp_TableContainer *>(pBroke->VBreakAt(iBreakAt));
+			pOffendingContainer = static_cast<fp_Container *>(pNewTab);
 		    xxx_UT_DEBUGMSG(("SEVIOR: Created broken table %x \n",pOffendingContainer));
-			fp_TableContainer * pNewTab = static_cast<fp_TableContainer *>(pOffendingContainer);
 			UT_ASSERT(pBroke->getHeight() > 0);
 			UT_ASSERT(pNewTab->getHeight() > 0);
 			pLastContainerToKeep = static_cast<fp_Container *>(pTab);
@@ -950,6 +1176,120 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 	return false;
 }
 
+
+
+/*!
+ * Breaks the given TOC, if appropriate.  
+ * \return true iff the TOC was broken.
+ */
+bool fb_ColumnBreaker::_breakTOC(fp_Container*& pOffendingContainer,
+								   fp_Container*& pLastContainerToKeep,
+								   int iMaxColHeight, 
+								   int iWorkingColHeight,
+								   int iContainerMarginAfter)
+{
+	bool bDoTOCBreak;
+
+    UT_DEBUGMSG(("breakTOC:!!!!!!!!!!!! Offending TOC is %x \n",pOffendingContainer));
+	fp_TOCContainer * pTOC = static_cast<fp_TOCContainer *>(pOffendingContainer);
+	if(!pTOC->isThisBroken())
+	{
+//
+// This is the first of this table set. Clear the old broken tables
+// and rebreak.
+//
+		pTOC->deleteBrokenTOCs(true);
+		bDoTOCBreak = true;
+		xxx_UT_DEBUGMSG(("SEVIOR: Need TOC Break 1 \n"));
+		xxx_UT_DEBUGMSG(("firstbroke %x lastbroke %x \n",pTOC->getFirstBrokenTOC(),pTOC->getLastBrokenTOC()));
+	}
+	else
+	{
+		bDoTOCBreak = true;
+	}
+//
+// If we don't break the table, the heights of the broken table's
+// will be adjusted at the setY() in the layout stage.
+//
+	fp_TOCContainer * pBroke = NULL;
+	UT_sint32 iAvail = iMaxColHeight - iWorkingColHeight - iContainerMarginAfter;
+	UT_sint32 iBreakAt = pTOC->wantVBreakAt(iAvail-1);
+	pTOC->setLastWantedVBreak(iBreakAt);
+	UT_DEBUGMSG(("breakTOC column: iAvail %d actual break at %d \n",iAvail,iBreakAt));
+//
+// Look to see if the table can be broken. If iBreakAt < 0 we have to bump 
+// the whole table into the next column.
+//
+	if(iBreakAt < 1)
+	{
+		xxx_UT_DEBUGMSG(("breakTOC Col: Can't break this table %d \n",iBreakAt));
+		return false;
+	}
+
+	UT_ASSERT(iBreakAt <= (iAvail-1));
+
+	if(bDoTOCBreak && (iBreakAt + iWorkingColHeight <= iMaxColHeight))
+	{
+//
+// OK we can break this table and keep some of it in this column. The
+// rest goes into the next column.
+//
+// Look to see if this table is broken.
+//
+		if(!pTOC->isThisBroken())
+		{
+//
+// Break it at 0 first.
+//
+			xxx_UT_DEBUGMSG(("SEVIOR: Breaking MAster iBreakAt %d yloc = %d \n",iBreakAt,pTOC->getY()));
+			fp_Container * pNext = static_cast<fp_Container *>(pTOC->getNext());
+			xxx_UT_DEBUGMSG(("SEVIOR: getNext %x \n",pNext));
+			if(pNext)
+			{
+				xxx_UT_DEBUGMSG(("SEVIOR: Container of next %d \n",pNext->getContainerType()));
+			}
+			pTOC->deleteBrokenTOCs(true);
+			pTOC->VBreakAt(0);
+		}
+//
+// Now get a broken table and break it again.
+//
+		if(!pTOC->isThisBroken())
+		{
+			pBroke = pTOC->getFirstBrokenTOC();
+		}
+		else
+		{
+			pBroke = pTOC;
+		}
+//
+// Look to see if we have to move the table out of this container.
+//
+		if(iBreakAt < 30)
+		{
+			pOffendingContainer = static_cast<fp_Container *>(pTOC);
+		}
+		else
+		{
+//
+// When we break the table, the bit broken will be placed after the current
+// table in the column. This then becomes the offending container and will
+// be bumped into the next column. Pretty cool eh ? :-)
+//
+			pOffendingContainer = static_cast<fp_Container *>(pBroke->VBreakAt(iBreakAt));
+		    xxx_UT_DEBUGMSG(("SEVIOR: Created broken table %x \n",pOffendingContainer));
+			fp_TOCContainer * pNewTOC = static_cast<fp_TOCContainer *>(pOffendingContainer);
+			UT_ASSERT(pBroke->getHeight() > 0);
+			UT_ASSERT(pNewTOC->getHeight() > 0);
+			pLastContainerToKeep = static_cast<fp_Container *>(pTOC);
+			xxx_UT_DEBUGMSG(("SEVIOR: Set lasttokeep 1 %x \n",pLastContainerToKeep));
+		}
+		return true;
+	}
+
+	return false;
+}
+
 fp_Container * fb_ColumnBreaker::_getNext(fp_Container * pCon)
 {
 	fp_Container * pNext = NULL;
@@ -982,6 +1322,13 @@ fp_Container * fb_ColumnBreaker::_getNext(fp_Container * pCon)
 		}
 #endif
 	}
+#if DEBUG
+		if(pNext)
+		{
+			UT_ASSERT(pNext->getContainerType() != FP_CONTAINER_FRAME);
+		}
+#endif
+
 	return pNext;
 }
 	
