@@ -47,6 +47,8 @@
 #define fl_BLOCK_STRUX_OFFSET	1
 
 class fl_Squiggles;
+class fl_SpellSquiggles;
+class fl_GrammarSquiggles;
 class FL_DocLayout;
 class fl_SectionLayout;
 class fl_ContainerLayout;
@@ -102,6 +104,8 @@ void buildTabStops(GR_Graphics * pG, const char* pszTabStops, UT_GenericVector<f
 class ABI_EXPORT fl_BlockLayout : public fl_ContainerLayout
 {
 	friend class fl_Squiggles;
+	friend class fl_SpellSquiggles;
+	friend class fl_GrammarSquiggles;
 	friend class fl_DocListener;
 	friend class fl_TOCLayout;
 	friend class fb_LineBreaker;
@@ -129,10 +133,15 @@ public:
 	fp_Line *           getNextWrappedLine(UT_sint32 iX,
 											  UT_sint32 iHeight,
 										   fp_Page * pPage);
+	void                getLeftRightForWrapping(UT_sint32 iX, 
+												UT_sint32 iHeight,
+												UT_sint32 & iMinLeft,
+												UT_sint32 & iMinRight,
+												UT_sint32 & iMinWidth);
 	virtual bool		recalculateFields(UT_uint32 iUpdateCount);
 
 	virtual void		redrawUpdate();
-	virtual void        updateLayout(void) {}
+	virtual void        updateLayout(bool bDoAll) {}
 	virtual fp_Container * getNewContainer(fp_Container * pCon = NULL);
 	FV_View *		getView(void) const { UT_return_val_if_fail( m_pLayout, NULL ); return m_pLayout->getView(); }
 
@@ -142,6 +151,7 @@ public:
 	UT_sint32       getLength(void);
 	bool            isEmbeddedType(void);
 	bool            isNotTOCable(void);
+	bool            isLastRunInBlock(fp_Run * pRun);
 	void            updateOffsets(PT_DocPosition posEmbedded, UT_uint32 iEmebbedSize, UT_sint32 iSuggestedDiff);
 	void            updateEnclosingBlockIfNeeded(void);
 	UT_sint32       getEmbeddedOffset(UT_sint32 startOffset, fl_ContainerLayout *& pEmbedCL);
@@ -192,7 +202,9 @@ public:
 	fl_BlockLayout * getPreviousListOfSameMargin(void);
 	inline fl_BlockLayout * getParentItem(void);
 
-	void findSquigglesForRun(fp_Run* pRun);
+	void findSpellSquigglesForRun(fp_Run* pRun);
+	void drawGrammarSquiggles(void);
+	void findGrammarSquigglesForRun(fp_Run* pRun);
 	UT_uint32 canSlurp(fp_Line* pLine) const;
 
 	PT_DocPosition getPosition(bool bActualBlockPos=false) const;
@@ -228,7 +240,9 @@ public:
 	inline UT_BidiCharType getDominantDirection(void) const { return m_iDomDirection; }
 	void setDominantDirection(UT_BidiCharType iDirection);
 
-	inline fl_Squiggles* getSquiggles(void) const { return m_pSquiggles; }
+	inline fl_SpellSquiggles* getSpellSquiggles(void) const { return m_pSpellSquiggles; }
+	inline fl_GrammarSquiggles* getGrammarSquiggles(void) const { return  m_pGrammarSquiggles; }
+
 
 	bool isHdrFtr(void);
 	void setHdrFtr(void) { m_bIsHdrFtr = true;}
@@ -303,7 +317,7 @@ public:
 	virtual bool			isCollapsed(void) const
 		{return m_bIsCollapsed;}
 	void					coalesceRuns(void);
-	virtual void			setNeedsReformat(UT_uint32 offset = 0);
+	virtual void			setNeedsReformat(fl_ContainerLayout * pCL, UT_uint32 offset = 0);
 	inline bool 		    needsReformat(void) const
 		{ return (m_iNeedsReformat >= 0); }
 	virtual void			setNeedsRedraw(void);
@@ -311,6 +325,9 @@ public:
 		{ return m_bNeedsRedraw; }
 	virtual void			markAllRunsDirty(void);
 	UT_sint32               findLineInBlock(fp_Line * pLine);
+	
+	bool                    isWordDelimiter(UT_UCS4Char c, UT_UCS4Char next, UT_UCS4Char prev, UT_uint32 iBlockPos);
+	bool                    isSentenceSeparator(UT_UCS4Char c, UT_uint32 iBlockPos);
 	bool					checkWord(fl_PartOfBlock* pPOB);
 	void					recheckIgnoredWords();
 	void                    setStyleInTOC(bool b)
@@ -346,11 +363,15 @@ public:
 	void					__dump(FILE * fp) const;
 #endif
 
+private:
+	virtual bool            _canContainPoint() const;
+	
 protected:
 
 	void					_recalcPendingWord(UT_uint32 iOffset, UT_sint32 chg);
 	bool					_doCheckWord(fl_PartOfBlock* pPOB,
 										 const UT_UCSChar* pBlockText,
+										 UT_sint32 iLength,
 										 bool bAddSquiggle = true,
 										 bool bClearScreen = true);
 
@@ -374,6 +395,8 @@ protected:
 	bool					_doInsertFieldEndRun(PT_BlockOffset blockOffset);
 	bool					_doInsertBookmarkRun(PT_BlockOffset blockOffset);
 	bool					_doInsertHyperlinkRun(PT_BlockOffset blockOffset);
+	bool					_doInsertMathRun(PT_BlockOffset blockOffset,PT_AttrPropIndex indexAP,PL_ObjectHandle oh);
+	bool					_doInsertEmbedRun(PT_BlockOffset blockOffset,PT_AttrPropIndex indexAP,PL_ObjectHandle oh);
 //	bool					_deleteBookmarkRun(PT_BlockOffset blockOffset);
 	bool					_doInsertForcedColumnBreakRun(PT_BlockOffset blockOffset);
 	bool					_doInsertForcedPageBreakRun(PT_BlockOffset blockOffset);
@@ -389,7 +412,7 @@ protected:
 	bool					_deleteFmtMark(PT_BlockOffset blockOffset);
 
 	virtual void			_lookupProperties(const PP_AttrProp* pAP);
-	void					_removeLine(fp_Line*, bool bRemoveFromContainer=false);
+	void					_removeLine(fp_Line*, bool bRemoveFromContainer, bool bReCalc);
 	void                    _purgeLine(fp_Line*);
 	void					_removeAllEmptyLines(void);
 
@@ -441,7 +464,7 @@ protected:
 	bool                    m_bStartList;
 	bool                    m_bStopList;
     bool                    m_bListLabelCreated;
-	fl_Squiggles *          m_pSquiggles;
+	fl_SpellSquiggles *     m_pSpellSquiggles;
 	bool                    m_bListItem;
 	const XML_Char *		m_szStyle;
 	bool                    m_bIsCollapsed;
@@ -460,6 +483,7 @@ protected:
 	UT_sint32               m_iLinePosInContainer;
 	bool                    m_bForceSectionBreak;
 	bool                    m_bPrevListLabel;
+	fl_GrammarSquiggles *   m_pGrammarSquiggles;
 };
 
 /*
@@ -471,24 +495,31 @@ class ABI_EXPORT fl_PartOfBlock
 {
 public:
 	fl_PartOfBlock();
-	fl_PartOfBlock(UT_sint32 iOffset, UT_sint32 iLength,
+	fl_PartOfBlock(UT_sint32 iOffset, UT_sint32 iPTLength,
 				   bool bIsIgnored = false);
 
-	bool doesTouch(UT_sint32 iOffset, UT_sint32 iLength) const;
-
+	bool             doesTouch(UT_sint32 iOffset, UT_sint32 iLength) const;
 	inline UT_sint32 getOffset(void) const { return m_iOffset; }
-	inline UT_sint32 getLength(void) const { return m_iLength; }
+	inline UT_sint32 getPTLength(void)const{ return m_iPTLength; }
 	inline bool 	 getIsIgnored(void) const { return m_bIsIgnored; }
 
 	inline void 	 setOffset(UT_sint32 iOffset) { m_iOffset = iOffset; }
-	inline void 	 setLength(UT_sint32 iLength) { m_iLength = iLength; }
+	inline void 	 setPTLength(UT_sint32 iLength) { m_iPTLength = iLength; }
 	inline void 	 setIsIgnored(bool bIsIgnored) { m_bIsIgnored = bIsIgnored; }
-
+	void             setInvisible(void)
+	{m_bIsInvisible = true;}
+	bool             isInvisible(void)
+	{ return m_bIsInvisible;}
+	void             setGrammarMessage(UT_UTF8String & sMsg);
+	void             getGrammarMessage(UT_UTF8String & sMsg);
+	
 private:
 	UT_sint32	m_iOffset;
-	UT_sint32	m_iLength;
+	UT_sint32   m_iPTLength;
 
 	bool		m_bIsIgnored;
+	bool        m_bIsInvisible;
+	UT_UTF8String  m_sGrammarMessage;
 };
 
 class ABI_EXPORT fl_TabStop
@@ -554,7 +585,8 @@ public:
 
 	bool            nextWordForSpellChecking(const UT_UCSChar*& pWord,
 											 UT_sint32& iLength,
-											 UT_sint32& iBlockPos);
+											 UT_sint32& iBlockPos,
+											 UT_sint32& iPTLength);
 	void              updateBlock(void);
 	void              updateSentenceBoundaries(void);
 
