@@ -591,7 +591,11 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 				{
 					reason = (UT_uint32) FL_DocLayout::bgcrSpelling;
 				}
-				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,true);
+				if( m_pLayout->getAutoGrammarCheck())
+				{
+					reason = reason | (UT_uint32) FL_DocLayout::bgcrGrammar;
+				}
+				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,false);
 			}
 		}
 		else
@@ -612,7 +616,11 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 				{
 					reason = (UT_uint32) FL_DocLayout::bgcrSpelling;
 				}
-				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,true);
+				if( m_pLayout->getAutoGrammarCheck())
+				{
+					reason = reason | (UT_uint32) FL_DocLayout::bgcrGrammar;
+				}
+				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,false);
 			}
 		}
 
@@ -634,7 +642,6 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 		UT_ASSERT(m_pCurrentSL);
 		
 		// Append a new TableLayout to the SectionLayout or CellLayout
-		
 		fl_ContainerLayout * pCon = getTopContainerLayout();
 		fl_ContainerLayout*	pCL = NULL;
 		UT_DEBUGMSG(("!!!!Appending Table \n"));
@@ -1731,7 +1738,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
 			return pSL->bl_doclistener_insertBlock(NULL, pcrx,sdh,lid,pfnBindHandles);
 		}
-#if 0
+#if 1
 //
 // FIXME When I get brave I'll see if we can't make a table the first layout
 // in a section.
@@ -1740,7 +1747,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 		{
 			UT_DEBUGMSG(("Insert Table immediately after Section \n"));
 			fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
-			return pSL->bl_doclistener_insertTable(NULL, pcrx,sdh,lid,pfnBindHandles);
+			return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles);
 		}
 #endif
 		default:						// unknown strux.
@@ -2239,6 +2246,14 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   bool bResult = (pCLSL->bl_doclistener_insertFrame(pCL,FL_SECTION_FRAME, pcrx,sdh,lid,pfnBindHandles) != 0);
 			   return bResult;
 		   }
+		case PTX_SectionTable:	  // we are inserting a Table as the first layout into a frame. This is valid
+		  {
+			   // The immediately prior strux is a Frame.  So, this
+			   // will become the first table of the Frame.
+		           fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
+			   return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles);
+
+		  }
 		default:
 		   {
 			   UT_DEBUGMSG(("Illegal strux type after frame %d \n",pcrx->getStruxType()));
