@@ -97,7 +97,8 @@ fl_FrameLayout::fl_FrameLayout(FL_DocLayout* pLayout,
 	  m_iXPage(0),
 	  m_iYPage(0),
 	  m_iBoundingSpace(0),
-	  m_iFrameWrapMode(FL_FRAME_ABOVE_TEXT)
+	  m_iFrameWrapMode(FL_FRAME_ABOVE_TEXT),
+	  m_bIsTightWrap(false)
 {
 	UT_ASSERT(m_pDocSL->getContainerType() == FL_CONTAINER_DOCSECTION);
 }
@@ -153,6 +154,11 @@ void 	fl_FrameLayout::setContainerProperties(void)
 	pFrame->setRightStyle(m_lineRight );
 	pFrame->setXpad(m_iXpad);
 	pFrame->setYpad(m_iYpad);
+	pFrame->setTightWrapping(m_bIsTightWrap);
+	if(FL_FRAME_BELOW_TEXT ==  m_iFrameWrapMode)
+        {
+	        pFrame->setAbove(false);
+	}
 //
 // Now do the image for this frame.
 //
@@ -390,12 +396,14 @@ void fl_FrameLayout::markAllRunsDirty(void)
 	}
 }
 
-void fl_FrameLayout::updateLayout(void)
+void fl_FrameLayout::updateLayout(bool bDoAll)
 {
+        xxx_UT_DEBUGMSG(("UpdsateLayout in in framelayout \n"));
 	if(needsReformat())
 	{
 		format();
 	}
+	m_vecFormatLayout.clear();
 	fl_ContainerLayout*	pBL = getFirstLayout();
 	while (pBL)
 	{
@@ -450,30 +458,12 @@ bool fl_FrameLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 	collapse();
 //	UT_ASSERT(pcrx->getStruxType()== PTX_SectionFrame);
 //
+	fl_ContainerLayout * pCL = getPrev();
+	myContainingLayout()->remove(this);
 	UT_DEBUGMSG(("Unlinking frame Layout %x \n",this));
-	fl_ContainerLayout * pPrev = getPrev();
-	fl_ContainerLayout * pNext = getNext();
-
-	if(pPrev != NULL)
-	{
-		pPrev->setNext(pNext);
-	}
-	else
-	{
-		myContainingLayout()->setFirstLayout(pNext);
-	}
-	if(pNext != NULL)
-	{
-		pNext->setPrev(pPrev);
-	}
-	else
-	{
-		myContainingLayout()->setLastLayout(pPrev);
-	}
 //
 // Remove from the list of frames in the previous block
 //
-	fl_ContainerLayout * pCL = getPrev();
 	while(pCL && pCL->getContainerType() != FL_CONTAINER_BLOCK)
 	{
 		pCL = pCL->getPrev();
@@ -587,7 +577,7 @@ void fl_FrameLayout::_createFrameContainer(void)
 
 /*!
   Create a new Frame container.
-  \params If pPrevFrame is non-null place the new cell after this in the linked
+  \param If pPrevFrame is non-null place the new cell after this in the linked
           list, otherwise just append it to the end.
   \return The newly created Frame container
 */
@@ -631,7 +621,7 @@ void fl_FrameLayout::format(void)
 		getNewContainer();
 	}
 	fl_ContainerLayout*	pBL = getFirstLayout();
-	
+	bool bOnPage = false;
 	while (pBL)
 	{
 		pBL->format();
@@ -702,10 +692,25 @@ void fl_FrameLayout::format(void)
 			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 			return;
 		}
-		pBL->setFramesOnPage(NULL);
+		if(!pBL->isCollapsed())
+		{
+		  bOnPage = pBL->setFramesOnPage(NULL);
+		  if(!bOnPage)
+		  {
+		    setNeedsReformat(this);
+		  }
+		}
+	}
+	m_bNeedsFormat = bOnPage;
+	m_bNeedsReformat = bOnPage;
+	if(!bOnPage)
+	{
+	  setNeedsReformat(this);
+	}
+	if(!bOnPage)
+	{
+	  return;
 	}
-	m_bNeedsFormat = false;
-	m_bNeedsReformat = false;
 	fl_DocSectionLayout * pDSL = getDocSectionLayout();
 	fp_FrameContainer * pFC = static_cast<fp_FrameContainer *>(getFirstContainer());
 	if(pFC)
@@ -714,6 +719,11 @@ void fl_FrameLayout::format(void)
 	}
 }
 
+void fl_FrameLayout::setNeedsReformat(fl_ContainerLayout * pCL,UT_uint32 offset)
+{
+        m_bNeedsReformat = true;
+        myContainingLayout()->setNeedsReformat(this);
+}
 /*!
     this function is only to be called by fl_ContainerLayout::lookupProperties()
     all other code must call lookupProperties() instead
@@ -742,6 +752,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	const XML_Char * pszBorderWidth = NULL;
 
 	const XML_Char * pszBoundingSpace = NULL;
+	const XML_Char * pszTightWrapped = NULL;
 // Frame Type
 
 	if(!pSectionAP || !pSectionAP->getProperty("frame-type",pszFrameType))
@@ -821,6 +832,21 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		m_iFrameWrapMode = FL_FRAME_ABOVE_TEXT;
 	}
+	//
+	// Wrap type
+	//
+	if(!pSectionAP || !pSectionAP->getProperty("tight-wrap",pszTightWrapped))
+	{
+		m_bIsTightWrap = false;
+	}
+	else if(strcmp(pszTightWrapped,"1") == 0)
+	{
+		m_bIsTightWrap = true;
+	}
+	else
+	{
+		m_bIsTightWrap = false;
+	}
 
 // Xpos
 
