@@ -34,7 +34,8 @@ typedef enum _SectionType
 	FL_SECTION_CELL,
 	FL_SECTION_FOOTNOTE,
 	FL_SECTION_MARGINNOTE,
-	FL_SECTION_FRAME
+	FL_SECTION_FRAME,
+	FL_SECTION_TOC
 } SectionType;
 
 
@@ -77,6 +78,7 @@ class fp_Run;
 class fp_Line;
 class fp_Container;
 class fp_HdrFtrContainer;
+class FG_Graphic;
 class PD_Document;
 class PP_AttrProp;
 class PX_ChangeRecord_FmtMark;
@@ -90,7 +92,8 @@ class PX_ChangeRecord_StruxChange;
 class fb_ColumnBreaker;
 class fp_EndnoteContainer;
 class fl_TableLayout;
-
+class fl_CellLayout;
+class UT_Worker;
 class ABI_EXPORT fl_SectionLayout : public fl_ContainerLayout
 {
 	friend class fl_DocListener;
@@ -102,22 +105,21 @@ public:
 	SectionType     	getType(void) const { return m_iType; }
 
 	virtual bool		recalculateFields(UT_uint32 iUpdateCount);
-
+	fl_BlockLayout *        getFirstBlock(void) const; 
 	virtual fp_Container*		getNewContainer(fp_Container * pFirstContainer = NULL) = 0;
-    virtual void                updateLayout(void) = 0;
 	virtual FL_DocLayout*		getDocLayout(void) const;
 	virtual void                markAllRunsDirty(void) =0;
 	virtual bool                isCollapsed(void) const
 		{return m_bIsCollapsed;}
-	virtual void                setNeedsReformat(UT_uint32 offset = 0);
+	virtual void                setNeedsReformat(fl_ContainerLayout * pCL, UT_uint32 offset = 0);
 	virtual void                setNeedsRedraw(void);
+	virtual void                removeFromUpdate(fl_ContainerLayout * pL);
 	virtual bool                needsReformat(void) const
 		{return m_bNeedsReformat;}
 	virtual bool                needsRedraw(void) const
 		{return m_bNeedsRedraw;}
 	virtual void                clearNeedsRedraw(void)
 		{m_bNeedsRedraw = false;}
-	virtual void        updateBackgroundColor(void);
 
 	virtual bool 	doclistener_changeStrux(const PX_ChangeRecord_StruxChange * pcrxc) = 0;
 	void                        checkAndAdjustCellSize(void);
@@ -150,6 +152,21 @@ public:
 											  void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
 																	  PL_ListenerId lid,
 																	  PL_StruxFmtHandle sfhNew));
+	virtual fl_SectionLayout * bl_doclistener_insertTable(SectionType iType,
+											  const PX_ChangeRecord_Strux * pcrx,
+											  PL_StruxDocHandle sdh,
+											  PL_ListenerId lid,
+											  void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
+																	  PL_ListenerId lid,
+																	  PL_StruxFmtHandle sfhNew));
+	virtual fl_SectionLayout * bl_doclistener_insertFrame(fl_ContainerLayout*,
+											  SectionType iType,
+											  const PX_ChangeRecord_Strux * pcrx,
+											  PL_StruxDocHandle sdh,
+											  PL_ListenerId lid,
+											  void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
+																	  PL_ListenerId lid,
+																	  PL_StruxFmtHandle sfhNew));
 
 	virtual bool bl_doclistener_insertObject(fl_ContainerLayout*, const PX_ChangeRecord_Object * pcro);
 	virtual bool bl_doclistener_deleteObject(fl_ContainerLayout*, const PX_ChangeRecord_Object * pcro);
@@ -159,12 +176,17 @@ public:
 	virtual bool bl_doclistener_deleteFmtMark(fl_ContainerLayout*, const PX_ChangeRecord_FmtMark * pcrfm);
 	virtual bool bl_doclistener_changeFmtMark(fl_ContainerLayout*, const PX_ChangeRecord_FmtMarkChange * pcrfmc);
 
+	virtual void         checkGraphicTick(GR_Graphics * pG);
+	virtual void         setImageWidth(UT_sint32 iWidth);
+	virtual void         setImageHeight(UT_sint32 iHeight);
+	GR_Image *           getBackgroundImage(void)
+	  {	return m_pImageImage;}
+	
 #ifdef FMT_TEST
 	virtual void		__dump(FILE * fp) const;
 #endif
 
 protected:
-	virtual void		_lookupProperties(void) = 0;
 
 	void				_purgeLayout();
 
@@ -174,6 +196,12 @@ protected:
 	bool                m_bIsCollapsed;
 	bool                m_bNeedsReformat;
 	bool                m_bNeedsRedraw;
+	FG_Graphic *        m_pGraphicImage;
+	GR_Image *          m_pImageImage;
+	UT_uint32           m_iGraphicTick;
+	UT_sint32           m_iDocImageWidth;
+	UT_sint32           m_iDocImageHeight;
+	UT_GenericVector<fl_ContainerLayout *> m_vecFormatLayout;
 };
 
 class ABI_EXPORT fl_DocSectionLayout : public fl_SectionLayout
@@ -188,7 +216,7 @@ public:
 	fl_DocSectionLayout* getPrevDocSection(void) const;
 
 	virtual void		format(void);
-	virtual void		updateLayout(void);
+	virtual void		updateLayout(bool bDoFull);
 	void                updateDocSection(void);
 	virtual void        collapse(void);
 	virtual fp_Container * getFirstContainer(void) const;
@@ -217,6 +245,8 @@ public:
 	UT_sint32                   getColumnGap(void) const;
 	UT_uint32			        getFootnoteLineThickness(void) const
 		{ return m_iFootnoteLineThickness;}
+	UT_uint32			        getFootnoteYoff(void) const
+		{ return m_iFootnoteYoff;}
 
 	void                completeBreakSection(void);
 	bool                arePageNumbersRestarted (void) const { return m_bRestart;}
@@ -225,7 +255,6 @@ public:
 	bool				getColumnLineBetween(void) const {return m_bColumnLineBetween;}
 	UT_uint32			getColumnOrder(void) const;
 	void                setPaperColor();
-	UT_RGBColor *       getPaperColor(void);
 	void				deleteEmptyColumns(void);
 	virtual bool 	    doclistener_changeStrux(const PX_ChangeRecord_StruxChange * pcrxc);
 	bool				doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx);
@@ -236,7 +265,6 @@ public:
 											  void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
 																	  PL_ListenerId lid,
 																	  PL_StruxFmtHandle sfhNew));
-
 	void				setHdrFtr(HdrFtrType iType, fl_HdrFtrSectionLayout* pHFSL);
 
 	fl_HdrFtrSectionLayout*         getHeader(void);
@@ -248,6 +276,9 @@ public:
 	fl_HdrFtrSectionLayout*         getHeaderLast(void);
 	fl_HdrFtrSectionLayout*         getFooterLast(void);
 
+	bool                            setHdrFtrHeightChange(bool bDoHdr, UT_sint32 newHeight);
+	static void         _HdrFtrChangeCallback(UT_Worker * pWorker);
+
 	void				addOwnedPage(fp_Page*);
 	void                            prependOwnedHeaderPage(fp_Page * p_Page);
 	void                            prependOwnedFooterPage(fp_Page * p_Page);
@@ -259,8 +290,9 @@ public:
     void                markForReformat(void) { m_bNeedsFormat = true;}
     bool                needsReFormat(void) const { return m_bNeedsFormat;}
 	bool                isThisPageValid(HdrFtrType hfType, fp_Page * pThisPage);
-    void                getVecOfHdrFtrs(UT_Vector * vecHdrFtr);
+    void                getVecOfHdrFtrs(UT_GenericVector<fl_HdrFtrSectionLayout *> * vecHdrFtr);
 	void                formatAllHdrFtr(void);
+	void                doMarginChangeOnly(void);
 	void                checkAndRemovePages(void);
 	void                addValidPages(void);
 	void                setNeedsSectionBreak(bool bSet, fp_Page * pPage );
@@ -269,9 +301,14 @@ public:
 	void                setLastEndnoteContainer(fp_EndnoteContainer * pECon);
 	fp_Container *      getFirstEndnoteContainer(void);
 	fp_Container *      getLastEndnoteContainer(void);
-	void                deleteBrokenTablesFromHere(fl_TableLayout * pTL);
+	void                deleteBrokenTablesFromHere(fl_ContainerLayout * pTL);
+	UT_sint32           getWidth(void);
+	UT_sint32           getActualColumnHeight(void);
+	UT_sint32           getActualColumnWidth(void);
+	bool                isCollapsing(void) const
+		{ return m_bDoingCollapse;}
 private:
-	virtual void		_lookupProperties(void);
+	virtual void		_lookupProperties(const PP_AttrProp* pAP);
 	fb_ColumnBreaker    m_ColumnBreaker;
 	/*
 	  TODO support special case header/footer for first page of section
@@ -308,6 +345,7 @@ private:
 	UT_sint32           m_iMaxSectionColumnHeight;
 	double              m_dMaxSectionColumnHeight;
 	UT_sint32           m_iFootnoteLineThickness;
+	UT_sint32           m_iFootnoteYoff;
 	
 	bool				m_bForceNewPage;
 
@@ -316,7 +354,6 @@ private:
 	//! Last column in the section
 	fp_Column*			m_pLastColumn;
 	fp_Page *           m_pFirstOwnedPage;
-	UT_RGBColor         m_clrPaper;
 
 	bool                m_bNeedsFormat;
 	bool                m_bNeedsRebuild;
@@ -324,8 +361,18 @@ private:
 	fp_EndnoteContainer * m_pFirstEndnoteContainer;
 	fp_EndnoteContainer * m_pLastEndnoteContainer;
 	bool                m_bDeleteingBrokenContainers;
+	UT_String           m_sPaperColor;
+	UT_String           m_sScreenColor;
+	UT_sint32           m_iNewHdrHeight;
+	UT_sint32           m_iNewFtrHeight;
+	UT_Worker *         m_pHdrFtrChangeTimer;
+	UT_String           m_sHdrFtrChangeProps;
+	bool                m_bDoingCollapse;
 };
 
+class _PageHdrFtrShadowPair;
+
+
 class ABI_EXPORT fl_HdrFtrSectionLayout : public fl_SectionLayout
 {
 	friend class fl_DocListener;
@@ -340,6 +387,7 @@ public:
 	void                        setHdrFtr(HdrFtrType iHFType) { 	m_iHFType = iHFType;}
 	virtual bool				recalculateFields(UT_uint32 iUpdateCount);
 	bool                        doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx);
+	void                        checkAndAdjustCellSize(fl_ContainerLayout * pCL);
 	void                        localFormat(void);
 	virtual void                markAllRunsDirty(void);
 	void                        checkAndRemovePages(void);
@@ -350,11 +398,10 @@ public:
 	virtual void				format(void);
 	virtual fl_SectionLayout *  getSectionLayout(void) const
 		{ return static_cast<fl_SectionLayout *>(m_pDocSL);}
-	virtual void				updateLayout(void);
+	virtual void				updateLayout(bool bDoFull);
 	void                        layout(void);
 	fl_ContainerLayout *        findMatchingContainer( fl_ContainerLayout * pBL);
 	virtual void				redrawUpdate(void);
-	void                        updateBackgroundColor(void);
 	virtual fp_Container*		getNewContainer(fp_Container * pFirstContainer = NULL);
 	virtual fp_Container*		getFirstContainer() const;
 	virtual fp_Container*		getLastContainer() const;
@@ -366,13 +413,23 @@ public:
 	void					deletePage(fp_Page*);
 	void                    clearScreen(void);
 	virtual void            collapse(void);
-
+	bool                    bl_doclistener_insertCell(fl_ContainerLayout* pCell,
+													  const PX_ChangeRecord_Strux * pcrx,
+													  PL_StruxDocHandle sdh,
+													  PL_ListenerId lid,
+													  fl_TableLayout * pTL);
+	bool                    bl_doclistener_insertEndTable(fl_ContainerLayout* pTab,
+													  const PX_ChangeRecord_Strux * pcrx,
+													  PL_StruxDocHandle sdh,
+													  PL_ListenerId lid);
 	virtual bool bl_doclistener_populateSpan(fl_ContainerLayout*, const PX_ChangeRecord_Span * pcrs, PT_BlockOffset blockOffset, UT_uint32 len);
 	virtual bool bl_doclistener_populateObject(fl_ContainerLayout*, PT_BlockOffset blockOffset, const PX_ChangeRecord_Object * pcro);
 	virtual bool bl_doclistener_insertSpan(fl_ContainerLayout*, const PX_ChangeRecord_Span * pcrs);
 	virtual bool bl_doclistener_deleteSpan(fl_ContainerLayout*, const PX_ChangeRecord_Span * pcrs);
 	virtual bool bl_doclistener_changeSpan(fl_ContainerLayout*, const PX_ChangeRecord_SpanChange * pcrsc);
 	virtual bool bl_doclistener_deleteStrux(fl_ContainerLayout*, const PX_ChangeRecord_Strux * pcrx);
+	bool bl_doclistener_deleteCellStrux(fl_ContainerLayout*, const PX_ChangeRecord_Strux * pcrx);
+	bool bl_doclistener_deleteTableStrux(fl_ContainerLayout*, const PX_ChangeRecord_Strux * pcrx);
 	virtual bool bl_doclistener_changeStrux(fl_ContainerLayout*, const PX_ChangeRecord_StruxChange * pcrxc);
 	virtual fl_SectionLayout * bl_doclistener_insertTable(fl_ContainerLayout*,
 											  SectionType iType,
@@ -382,6 +439,14 @@ public:
 											  void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
 																	  PL_ListenerId lid,
 																	  PL_StruxFmtHandle sfhNew));
+	virtual fl_SectionLayout * bl_doclistener_insertTable(SectionType iType,
+											  const PX_ChangeRecord_Strux * pcrx,
+											  PL_StruxDocHandle sdh,
+											  PL_ListenerId lid,
+											  void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
+																	  PL_ListenerId lid,
+																	  PL_StruxFmtHandle sfhNew));
+	bool         bl_doclistener_insertFirstBlock(fl_ContainerLayout* pCL, const PX_ChangeRecord_Strux * pcrx,PL_StruxDocHandle sdh,PL_ListenerId lid);
 	virtual bool bl_doclistener_insertBlock(fl_ContainerLayout*, const PX_ChangeRecord_Strux * pcrx,
 											PL_StruxDocHandle sdh,
 											PL_ListenerId lid,
@@ -404,12 +469,12 @@ public:
 
 private:
 	UT_sint32					_findShadow(fp_Page * pPage);
-	virtual void				_lookupProperties(void);
+	virtual void				_lookupProperties(const PP_AttrProp* pAP);
 	void                        _localCollapse(void);
 
 	fl_DocSectionLayout*		m_pDocSL;
 	HdrFtrType					m_iHFType;
-	UT_Vector					m_vecPages;
+	UT_GenericVector<_PageHdrFtrShadowPair*> m_vecPages;
 	fp_Container *              m_pHdrFtrContainer;
 };
 
@@ -426,7 +491,7 @@ virtual	fl_HdrFtrSectionLayout*	getHdrFtrSectionLayout(void) const { return m_pH
 	fl_ContainerLayout *		findMatchingContainer(fl_ContainerLayout * pBL);
 	fl_ContainerLayout *		findBlockAtPosition(PT_DocPosition pos);
 	virtual void				format(void);
-	virtual void				updateLayout(void);
+	virtual void				updateLayout(bool bDoFull);
 	virtual void				redrawUpdate(void);
 	fp_Page *                       getPage(void) { return m_pPage;}
 	virtual fp_Container*		getNewContainer(fp_Container *pFirstContainer = NULL);
@@ -442,7 +507,7 @@ virtual	fl_HdrFtrSectionLayout*	getHdrFtrSectionLayout(void) const { return m_pH
 
 
 private:
-	virtual void				_lookupProperties(void);
+	virtual void				_lookupProperties(const PP_AttrProp* pAP);
 	void						_createContainer(void);
 
 	fp_ShadowContainer*			m_pContainer;
@@ -482,6 +547,8 @@ private:
 	bool						m_bListening;
 	fl_ContainerLayout*			m_pCurrentBL;
 	fl_HdrFtrSectionLayout *    m_pHFSL;
+	fl_TableLayout *            m_pCurrentTL;
+	fl_CellLayout *             m_pCurrentCell;
 };
 
 #endif /* SECTIONLAYOUT_H */
