@@ -219,7 +219,7 @@ void fl_TableLayout::setTableContainerProperties(fp_TableContainer * pTab)
 /*!
   Create a new Table container and plug it into the linked list of Table 
   containers.
-  \params If pPrevTab is non-null place the new cell after this in the linked
+  \param If pPrevTab is non-null place the new cell after this in the linked
           list, otherwise just append it to the end.
   \return The newly created Table container
 */
@@ -251,9 +251,9 @@ void fl_TableLayout::insertTableContainer( fp_TableContainer * pNewTab)
 	fl_ContainerLayout * pPrevL = static_cast<fl_ContainerLayout *>(getPrev());
 	fp_Container * pPrevCon = NULL;
 	fp_Container * pUpCon = NULL;
-	if(pPrevL != NULL)
+	if(pPrevL != NULL )
 	{
-		while(pPrevL && ((pPrevL->getContainerType() == FL_CONTAINER_FOOTNOTE)
+		while(pPrevL && (pPrevL != pUPCL) && ((pPrevL->getContainerType() == FL_CONTAINER_FOOTNOTE)
           || (pPrevL->getContainerType() == FL_CONTAINER_ENDNOTE)||
 		  (pPrevL->getContainerType() == FL_CONTAINER_FRAME) ||
 		  (pPrevL->isHidden() == FP_HIDDEN_FOLDED) ||
@@ -283,6 +283,26 @@ void fl_TableLayout::insertTableContainer( fp_TableContainer * pNewTab)
 					pUpCon = pPrevCon->getContainer();
 				}
 			}
+			else if(pPrevL->getContainerType() == FL_CONTAINER_DOCSECTION)
+			{
+				pUpCon= static_cast<fl_DocSectionLayout *>(pPrevL)->getFirstContainer();
+				pPrevCon = NULL;
+			}
+			else if(pPrevL->getContainerType() == FL_CONTAINER_SHADOW)
+			{
+				pUpCon= static_cast<fl_HdrFtrShadow *>(pPrevL)->getFirstContainer();
+				pPrevCon = NULL;
+			}
+			else if(pPrevL->getContainerType() == FL_CONTAINER_HDRFTR)
+			{
+				pUpCon= static_cast<fl_HdrFtrSectionLayout *>(pPrevL)->getFirstContainer();
+				pPrevCon = NULL;
+			}
+			else if(pPrevL->getContainerType() == FL_CONTAINER_FRAME)
+			{
+				pUpCon= static_cast<fl_FrameLayout *>(pPrevL)->getFirstContainer();
+				pPrevCon = NULL;
+			}
 			else
 			{
 				pPrevCon = pPrevL->getLastContainer();
@@ -302,6 +322,17 @@ void fl_TableLayout::insertTableContainer( fp_TableContainer * pNewTab)
 		}
 		UT_ASSERT(pUpCon);
 	}
+	else if((pUPCL->getContainerType() == FL_CONTAINER_HDRFTR) || (pUPCL->getContainerType() == FL_CONTAINER_SHADOW) || (pUPCL->getContainerType() == FL_CONTAINER_FRAME))
+	{
+		pUpCon = pUPCL->getFirstContainer();
+		if(pUpCon == NULL)
+		{
+			pUpCon = pUPCL->getNewContainer(NULL);
+		}
+		UT_ASSERT(pUpCon);
+		pPrevL = pUPCL;
+		pPrevCon = NULL;
+	} 
 	else
 	{
 		pUpCon = pUPCL->getLastContainer();
@@ -320,21 +351,30 @@ void fl_TableLayout::insertTableContainer( fp_TableContainer * pNewTab)
 	}
 	else
 	{
-		UT_sint32 i = pUpCon->findCon(pPrevCon);
-		xxx_UT_DEBUGMSG(("SEVIOR!!!!!!!!!! New Table %x inserted into %x \n",pNewTab,pUpCon));
-		if(i >= 0 && (i+1) < static_cast<UT_sint32>(pUpCon->countCons()))
+		UT_sint32 i =0;
+		if(pPrevCon == NULL)
 		{
-			pUpCon->insertConAt(pNewTab,i+1);
-			pNewTab->setContainer(pUpCon);
-		}
-		else if( i >=0 &&  (i+ 1) == static_cast<UT_sint32>(pUpCon->countCons()))
-		{
-			pUpCon->addCon(pNewTab);
+			pUpCon->insertConAt(pNewTab,0);
 			pNewTab->setContainer(pUpCon);
 		}
 		else
 		{
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			i = pUpCon->findCon(pPrevCon);
+			xxx_UT_DEBUGMSG(("SEVIOR!!!!!!!!!! New Table %x inserted into %x \n",pNewTab,pUpCon));
+			if(i >= 0 && (i+1) < static_cast<UT_sint32>(pUpCon->countCons()))
+			{
+				pUpCon->insertConAt(pNewTab,i+1);
+				pNewTab->setContainer(pUpCon);
+			}
+			else if( i >=0 &&  (i+ 1) == static_cast<UT_sint32>(pUpCon->countCons()))
+			{
+				pUpCon->addCon(pNewTab);
+				pNewTab->setContainer(pUpCon);
+			}
+			else
+			{
+				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			}
 		}
 	}
 }
@@ -528,7 +568,6 @@ void fl_TableLayout::format(void)
 		m_iHeightChanged = 0;
 		m_pNewHeightCell = NULL;
 	}
-
 	if((!bSim && isDirty()) || bRebuild)
 	{
 		while (pCell)
@@ -628,8 +667,10 @@ void fl_TableLayout::format(void)
 	{
 		m_iHeightChanged = 0;
 		m_pNewHeightCell = NULL;
-		m_bNeedsReformat = false;
 		m_bIsDirty = false;
+		m_bNeedsReformat = false;
+		m_vecFormatLayout.clear();
+		xxx_UT_DEBUGMSG(("TableLayout format cleared %x \n"));
 	}
 }
 
@@ -650,22 +691,23 @@ void fl_TableLayout::markAllRunsDirty(void)
 	}
 }
 
-void fl_TableLayout::updateLayout(void)
+void fl_TableLayout::updateLayout(bool bDoAll)
 {			
-	xxx_UT_DEBUGMSG(("updateTableLayout  \n"));
+	xxx_UT_DEBUGMSG(("updateTableLayout  in TableLayout\n"));
 	if(getDocument()->isDontImmediateLayout())
 	{
-		xxx_UT_DEBUGMSG(("updateTableLayout not allowed updates  \n"));
+		UT_DEBUGMSG(("updateTableLayout not allowed updates  \n"));
 		return;
 	}
 	xxx_UT_DEBUGMSG(("updateTableLayout Doing updates  \n"));
 	fl_ContainerLayout*	pBL = getFirstLayout();
 	bool bNeedsFormat = false;
+	m_vecFormatLayout.clear();
 	while (pBL)
 	{
 		if (pBL->needsReformat())
 		{
-			pBL->updateLayout();
+			pBL->updateLayout(false);
 			bNeedsFormat = true;
 		}
 
@@ -863,8 +905,12 @@ bool fl_TableLayout::bl_doclistener_insertTable( const PX_ChangeRecord_Strux * p
 	m_pDoc->getBounds(true,pos1);
 
 	fl_SectionLayout* pSL = NULL;
-
-	pSL = static_cast<fl_SectionLayout *>(static_cast<fl_ContainerLayout *>(getSectionLayout())->insert(sdh,this,pcrx->getIndexAP(), FL_CONTAINER_TABLE));
+	fl_ContainerLayout * pMyCL = static_cast<fl_ContainerLayout *>(getSectionLayout());
+	if(pMyCL == NULL)
+	{
+		pMyCL = myContainingLayout();
+	}
+	pSL = static_cast<fl_SectionLayout *>(pMyCL->insert(sdh,this,pcrx->getIndexAP(), FL_CONTAINER_TABLE));
 
 		// Must call the bind function to complete the exchange of handles
 		// with the document (piece table) *** before *** anything tries
@@ -973,7 +1019,7 @@ bool fl_TableLayout::bl_doclistener_insertEndTable(fl_ContainerLayout*,
 	{
 		pView->setPoint(pView->getPoint() +  fl_BLOCK_STRUX_OFFSET);
 	}
-	setNeedsReformat(0);
+	setNeedsReformat(this,0);
 	m_bIsEndTableIn = true;
 	//
 	// Look to see if we're in a HfrFtr section
@@ -1491,7 +1537,7 @@ void fl_TableLayout::collapse(void)
 	}
 	setFirstContainer(NULL);
 	setLastContainer(NULL);
-	setNeedsReformat();
+	setNeedsReformat(this);
 }
 
 bool fl_TableLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
@@ -1507,27 +1553,7 @@ bool fl_TableLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 		pTab->decNumNestedTables();
 	}
 	xxx_UT_DEBUGMSG(("SEVIOR: !!!!!!!! Doing table delete strux!! \n"));
-	fl_ContainerLayout * pPrev = getPrev();
-	fl_ContainerLayout * pNext = getNext();
-	
 	collapse();
-
-	if(pPrev != NULL)
-	{
-		pPrev->setNext(pNext);
-	}
-	else
-	{
-		myContainingLayout()->setFirstLayout(pNext);
-	}
-	if(pNext != NULL)
-	{
-		pNext->setPrev(pPrev);
-	}
-	else
-	{
-		myContainingLayout()->setLastLayout(pPrev);
-	}
 	//
 	// Look to see if we're in a HdrFtr
 	//
@@ -1537,6 +1563,7 @@ bool fl_TableLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 		fl_HdrFtrSectionLayout * pHFSL = static_cast<fl_HdrFtrSectionLayout *>(pMyCL);
 		pHFSL->bl_doclistener_deleteTableStrux(this,pcrx);
 	}
+	myContainingLayout()->remove(this);
 
 	delete this;			// TODO whoa!  this construct is VERY dangerous.
 
@@ -1937,7 +1964,7 @@ fl_SectionLayout * fl_CellLayout::getSectionLayout(void) const
 /*!
   Create a new Cell containers and plug it into the linked list of Cell 
   containers.
-  \params If pPrevCell is non-null place the new cell after this in the linked
+  \param If pPrevCell is non-null place the new cell after this in the linked
           list, otherwise just append it to the end.
   \return The newly created Cell container
 */
@@ -1973,6 +2000,7 @@ void fl_CellLayout::format(void)
 	UT_sint32 iOldHeight = getFirstContainer()->getHeight();
 	fl_ContainerLayout * pPrevCL = myContainingLayout()->getPrev();
 	fp_Page * pPrevP = NULL;
+	m_vecFormatLayout.clear(); // Later we'll use this.
 	if(pPrevCL)
 	{
 		fp_Container * pPrevCon = pPrevCL->getFirstContainer();
@@ -1988,7 +2016,7 @@ void fl_CellLayout::format(void)
 		xxx_UT_DEBUGMSG(("Formatting Block in Cell %x \n",pBL));
 		if(iOldHeight <= 0)
 		{
-			pBL->setNeedsReformat(0);
+			pBL->setNeedsReformat(pBL,0);
 		}
 		pBL->format();
 		UT_sint32 count = 0;
@@ -2017,7 +2045,7 @@ void fl_CellLayout::format(void)
 			getDocSectionLayout()->setNeedsSectionBreak(true,pPrevP);
 		}
 	}
-	m_bNeedsReformat = false;
+	m_bNeedsReformat = (m_vecFormatLayout.getItemCount() > 0);
 	checkAndAdjustCellSize();
 	m_bDoingFormat = false;
 }
@@ -2059,11 +2087,12 @@ bool fl_CellLayout::isLayedOut(void) const
 	return true;
 }
 
-void fl_CellLayout::updateLayout(void)
+void fl_CellLayout::updateLayout(bool bDoAll)
 {
 	fl_ContainerLayout*	pBL = getFirstLayout();
 	bool bNeedsFormat = false;
-	xxx_UT_DEBUGMSG(("updateCellLayout \n"));
+    xxx_UT_DEBUGMSG(("updateCellLayout \n"));
+	m_vecFormatLayout.clear();
 	while (pBL)
 	{
 		if (pBL->needsReformat())
@@ -2524,7 +2553,7 @@ void fl_CellLayout::collapse(void)
 	}
 	setFirstContainer(NULL);
 	setLastContainer(NULL);
-	setNeedsReformat();
+	setNeedsReformat(this);
 }
 
 bool fl_CellLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
@@ -2532,29 +2561,7 @@ bool fl_CellLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 	UT_ASSERT(pcrx->getType()==PX_ChangeRecord::PXT_DeleteStrux);
 	UT_ASSERT(pcrx->getStruxType()== PTX_SectionCell);
 
-
-	fl_ContainerLayout * pPrev = getPrev();
-	fl_ContainerLayout * pNext = getNext();
-
 	collapse();
-//	fl_TableLayout * pTL = static_cast<fl_TableLayout *>(myContainingLayout());
-//	pTL->collapse();
-	if(pPrev != NULL)
-	{
-		pPrev->setNext(pNext);
-	}
-	else
-	{
-		myContainingLayout()->setFirstLayout(pNext);
-	}
-	if(pNext != NULL)
-	{
-		pNext->setPrev(pPrev);
-	}
-	else
-	{
-		myContainingLayout()->setLastLayout(pPrev);
-	}
 	//
 	// Look to see if we're in a HdrFtr
 	//
@@ -2568,6 +2575,7 @@ bool fl_CellLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 		fl_HdrFtrSectionLayout * pHFSL = static_cast<fl_HdrFtrSectionLayout *>(pMyCL);
 		pHFSL->bl_doclistener_deleteCellStrux(this,pcrx);
 	}
+	myContainingLayout()->remove(this);
 
 //	pTL->updateTable(); // may not need this. FIXME check if we do!
 	delete this;			// TODO whoa!  this construct is VERY dangerous.
