@@ -59,7 +59,7 @@ fp_Line     * fp_Line::s_pMapOwner = 0;
 #endif
 
 #define STATIC_BUFFER_INITIAL 150
-
+#define INITIAL_LINE_OFFSET -99999999
 UT_sint32 * fp_Line::s_pOldXs = NULL;
 UT_uint32   fp_Line::s_iOldXsSize = 0;
 UT_uint32	fp_Line::s_iClassInstanceCounter = 0;
@@ -76,7 +76,7 @@ fp_Line::fp_Line(fl_SectionLayout * pSectionLayout) :
 	m_iAscent(0),
 	m_iDescent(0),
 	m_iX(0),
-	m_iY(-2000000), // So setY(0) triggers a clearscreen and redraw!
+	m_iY(INITIAL_LINE_OFFSET), // So setY(0) triggers a clearscreen and redraw!
 		            // I do not like this at all; we have no business
 		            // of clearing at fictional coordinances
 	//m_bRedoLayout(true),
@@ -381,9 +381,12 @@ void fp_Line::setMaxWidth(UT_sint32 iMaxWidth)
 {
 	if(iMaxWidth < 60) // This is hardwired to disallow -ve or too small values
 	{
+	  UT_DEBUGMSG(("Max width set to... %d \n",iMaxWidth));
+	  UT_ASSERT(0);
 		iMaxWidth = 60;
 	}
 	m_iMaxWidth = iMaxWidth;
+	xxx_UT_DEBUGMSG(("Line %x MaxWidth %d \n",this,iMaxWidth));
 	m_iClearToPos = iMaxWidth;
 	m_iClearLeftOffset = getDescent();
 }
@@ -1075,7 +1078,7 @@ void fp_Line::clearScreen(void)
 			UT_ASSERT(m_iClearToPos + m_iClearLeftOffset < getPage()->getWidth());
 //			pRun->Fill(getGraphics(),xoffLine - m_iClearLeftOffset, yoffLine, m_iClearToPos + m_iClearLeftOffset+iExtra, height);
 			pRun->Fill(getGraphics(),xoffLine - m_iClearLeftOffset, yoffLine, getMaxWidth() + m_iClearLeftOffset +iExtra, height);
-			xxx_UT_DEBUGMSG(("Clear pLine %x xoffline %d width %d \n",this,xoffLine,getMaxWidth() + m_iClearLeftOffset +iExtra));
+
 //
 // Sevior: I added this for robustness.
 //
@@ -1233,7 +1236,7 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 		
 		fp_VerticalContainer * pVCon= (static_cast<fp_VerticalContainer *>(getContainer()));
 		pVCon->getScreenOffsets(this, xoffLine, yoffLine);
-		UT_sint32 diff = xoff - xoffLine;
+
 		UT_ASSERT(yoff == yoffLine);
 
 		fp_Line * pPrevLine = static_cast<fp_Line *>(getPrevContainerInSection());
@@ -1259,20 +1262,6 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 			xxx_UT_DEBUGMSG(("pl_Line _doClear no Page \n"));
 			return;
 		}
-		fl_DocSectionLayout * pSL =  getBlock()->getDocSectionLayout();
-		UT_sint32 iExtra = getGraphics()->tlu(2);
-		if(getContainer() && (getContainer()->getContainerType() != FP_CONTAINER_CELL) && (getContainer()->getContainerType() != FP_CONTAINER_FRAME))
-		  
-		{
-		    if(pSL->getNumColumns() >1)
-		    {
-		         iExtra = pSL->getColumnGap()/2;
-		    }
-		    else
-		    {
-		         iExtra = pSL->getRightMargin()/2;
-		    }
-		}
 //		UT_ASSERT((m_iClearToPos + leftClear - (xoff-xoffLine)) <= getPage()->getWidth());
 		xxx_UT_DEBUGMSG(("Clear from runindex to end height %d \n",getHeight()));
 		xxx_UT_DEBUGMSG(("Width of clear %d \n",m_iClearToPos + leftClear - xoff));
@@ -1284,7 +1273,7 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 			// to the end of the line
 			pRun->Fill(getGraphics(), xoff - leftClear,
 					   yoff,
-					   getMaxWidth() + leftClear + iExtra -diff,
+					   m_iClearToPos + leftClear - xoff,
 					   getHeight());
 		}
 		else
@@ -2550,6 +2539,7 @@ void fp_Line::setX(UT_sint32 iX, bool bDontClearIfNeeded)
 	{
 		clearScreen();
 	}
+	xxx_UT_DEBUGMSG(("SetX Line %x X %d \n",this,iX));
 	m_iX = iX;
 }
 
@@ -2946,6 +2936,10 @@ bool	fp_Line::containsForcedColumnBreak(void) const
 		{
 			return true;
 		}
+		if(pRun->getPrevRun() && (pRun->getPrevRun()->getType() == FPRUN_FORCEDCOLUMNBREAK))
+		{
+			return true;
+		}
 	}
 
 	return false;
@@ -2960,6 +2954,10 @@ bool fp_Line::containsForcedPageBreak(void) const
 		{
 			return true;
 		}
+		if(pRun->getPrevRun() && (pRun->getPrevRun()->getType() == FPRUN_FORCEDPAGEBREAK))
+		{
+			return true;
+		}
 	}
 	return false;
 }
