@@ -86,7 +86,9 @@ enum FP_RUN_TYPE
 	FPRUN_HYPERLINK					= 13,
 	FPRUN_DIRECTIONMARKER           = 14,
 	FPRUN_DUMMY                     = 15,
-	FPRUN__LAST__					= 16
+	FPRUN_MATH                      = 16,
+	FPRUN_EMBED                      = 17,
+	FPRUN__LAST__					= 18
 };
 
 // specifies how setX should handle screen clearing
@@ -780,7 +782,6 @@ protected:
 											  GR_Graphics * pG = NULL);
 
 	virtual void			_draw(dg_DrawArgs*);
-	virtual void			_drawResizeBox(UT_Rect box);
 	virtual void			_clearScreen(bool bFullLineHeightRect);
 	virtual bool			_letPointPass(void) const;
 
