@@ -115,11 +115,12 @@ public:
 	void		        drawBroken(dg_DrawArgs* pDa, fp_TableContainer * pTab);
 	virtual void		clearScreen(void);
 	void		        clearScreen(bool bNoRecursive);
-	void                drawLines(fp_TableContainer * pBroke);
+	void                getScreenPositions(fp_TableContainer * pBroke,GR_Graphics * pG,UT_sint32 & iLeft, UT_sint32 & iRight,UT_sint32 & iTop,UT_sint32 & iBot,UT_sint32 & col_y, fp_Column *& pCol, fp_ShadowContainer *& pShadow, bool & bDoClear );
+	void                drawLines(fp_TableContainer * pBroke,GR_Graphics * pG);
 	void                drawLinesAdjacent(void);
 	void                draw(fp_Line * pLine);
 	fp_TableContainer * getBrokenTable(fp_Container * pCon);
-	fp_Column *         getColumn(fp_Container *pCon);
+	fp_VerticalContainer * getColumn(fp_Container *pCon);
 	UT_sint32           tweakBrokenTable(fp_TableContainer * pBroke);
 	virtual void		draw(dg_DrawArgs*);
 	virtual void		draw(GR_Graphics*) {}
@@ -143,8 +144,10 @@ public:
 	UT_sint32           getSpannedHeight(void);
 	void                setLineMarkers(void);
 	bool                containsFootnoteReference(void);
-	bool                getFootnoteContainers(UT_Vector * pvecFoots);
-    UT_sint32           getLeftAttach(void) const
+	void                deleteBrokenTables(bool bClearFirst=true);
+	bool                getFootnoteContainers(UT_GenericVector<fp_FootnoteContainer*>* pvecFoots);
+	void                getLeftTopOffsets(UT_sint32 & xoff, UT_sint32 & yoff);
+   UT_sint32           getLeftAttach(void) const
 		{ return m_iLeftAttach;}
     UT_sint32           getRightAttach(void) const
 		{ return m_iRightAttach;}
@@ -180,7 +183,8 @@ public:
 		
 	PP_PropertyMap::Background getBackground () const;
 
-	void setBackground (const PP_PropertyMap::Background & style) { m_background = style; }
+	void setBackground (const PP_PropertyMap::Background & style);
+
 
 	PP_PropertyMap::Line getBottomStyle (const fl_TableLayout * table) const;
 	PP_PropertyMap::Line getLeftStyle   (const fl_TableLayout * table) const;
@@ -229,6 +233,8 @@ public:
 	    {  return m_bDirty;}
 	bool                doesIntersectClip(fp_TableContainer * pBroke, UT_Rect * rClip);
 	bool                isInNestedTable(void);
+	bool                containsNestedTables(void);
+	bool                isRepeated(void) const;
 #ifdef FMT_TEST
 	void				__dump(FILE * fp) const;
 #endif
@@ -237,8 +243,8 @@ private:
 		
 	void                _clear(fp_TableContainer * pBroke);
 	void				_drawLine(const PP_PropertyMap::Line & style,
-								  UT_sint32 left, UT_sint32 top, UT_sint32 right, UT_sint32 bot);
-	void				_getBrokenRect(fp_TableContainer * pBroke, fp_Page* &pPage, UT_Rect &bRec);
+								  UT_sint32 left, UT_sint32 top, UT_sint32 right, UT_sint32 bot,GR_Graphics * pG);
+	void				_getBrokenRect(fp_TableContainer * pBroke, fp_Page* &pPage, UT_Rect &bRec, GR_Graphics * pG);
 		
 //
 // These variables describe where the cell is attached to the table.
@@ -320,9 +326,9 @@ private:
 
 // Flag to see if this cell is drawn "selected"
 	bool		           m_bIsSelected;
-	
-	
 	bool                   m_bDirty;
+
+	bool                   m_bIsRepeated;
 };
 
 class ABI_EXPORT fp_TableContainer : public fp_VerticalContainer
@@ -336,21 +342,23 @@ public:
 	void                sizeAllocate(fp_Allocation * pAllocate);
     virtual void        mapXYToPosition(UT_sint32 x, UT_sint32 y, 
 										PT_DocPosition& pos,
-										bool& bBOL, bool& bEOL);
+										bool& bBOL, bool& bEOL, bool &isTOC);
 	virtual fp_Page *   getPage(void);
 	fp_Line *           getFirstLineInColumn(fp_Column * pCol);
 	fp_Line *           getLastLineInColumn(fp_Column * pCol);
 	void				layout(void);
 	virtual void        setY(UT_sint32 iY);
 	virtual UT_sint32   getHeight(void);
+	virtual void        setHeight(UT_sint32 iHeight);
 	virtual void        setContainer(fp_Container * pContainer);
 	virtual void		draw(dg_DrawArgs*);
 	virtual void		draw(GR_Graphics*) {}
- 	virtual UT_sint32   getMarginBefore(void) const;
+	virtual UT_sint32   getMarginBefore(void) const;
 	virtual UT_sint32   getMarginAfter(void) const;
+	fp_Column *         getBrokenColumn(void);
 	void                drawLines();
 	bool                containsFootnoteReference(void);
-	bool                getFootnoteContainers(UT_Vector * pvecFoots);
+	bool                getFootnoteContainers(UT_GenericVector<fp_FootnoteContainer*>* pvecFoots);
 	UT_sint32           tweakBrokenTable(fp_TableContainer * pBroke);
     virtual void        clearScreen(void);
 	virtual bool        isVBreakable(void);
@@ -358,6 +366,7 @@ public:
 	virtual UT_sint32   wantVBreakAt(UT_sint32);
 	virtual UT_sint32   wantHBreakAt(UT_sint32) {return 0;}
 	virtual fp_ContainerObject * VBreakAt(UT_sint32);
+	void                breakCellsAt(UT_sint32 vpos);
 	virtual fp_ContainerObject * HBreakAt(UT_sint32) {return NULL;}
 	UT_sint32           getBrokenNumber(void);
 	void                setToAllocation(void);
@@ -386,6 +395,7 @@ public:
 	void                queueResize(void);
 	UT_sint32           getYOfRow(UT_sint32 row);
 	fp_CellContainer *  getCellAtRowColumn(UT_sint32 row, UT_sint32 column);
+	fp_CellContainer *  getCellAtRowColumnLinear(UT_sint32 row, UT_sint32 column);
 	virtual fp_Container * getNextContainerInSection(void) const;
 	virtual fp_Container * getPrevContainerInSection(void) const;
 	fp_TableContainer * getMasterTable(void) const
@@ -412,8 +422,14 @@ public:
 	fp_TableContainer * getLastBrokenTable(void) const;
 	void                setFirstBrokenTable(fp_TableContainer * pBroke);
 	void                setLastBrokenTable(fp_TableContainer * pBroke);
-	void                deleteBrokenTables(bool bClearFirst=true);
+	void                deleteBrokenTables(bool bClearFirst, bool bRecurseUp = true);
 	void                adjustBrokenTables(void);
+	UT_sint32               getBrokenTop(void);
+	UT_sint32               getBrokenBot(void);
+	void                    setBrokenTop(UT_sint32 iTop) 
+		{ m_iBrokenTop = iTop;}
+	void                    setBrokenBot(UT_sint32 iBot) 
+		{ m_iBrokenBottom = iBot;}
 	UT_sint32           getNumRows(void) const;
 	UT_sint32           getNumCols(void) const;
 	UT_sint32           getRowHeight(UT_sint32 iRow, UT_sint32 iMeasHeight);
@@ -423,12 +439,7 @@ public:
 		{ return m_bRedrawLines;}
 	fp_TableRowColumn *     getNthCol(UT_sint32 i);
 	fp_TableRowColumn *     getNthRow(UT_sint32 i);
-	UT_sint32               getBrokenTop(void);
-	UT_sint32               getBrokenBot(void);
-	void                    setBrokenTop(UT_sint32 iTop) 
-		{ m_iBrokenTop = iTop;}
-	void                    setBrokenBot(UT_sint32 iBot) 
-		{ m_iBrokenBottom = iBot;}
+	bool                    containsNestedTables(void);
 	void setRowHeightType(FL_RowHeightType iType)
 		{
 	      m_iRowHeightType = iType;
@@ -468,8 +479,8 @@ private:
 	UT_sint32               m_iBorderWidth;
 	bool                    m_bIsHomogeneous;
 
-	UT_Vector               m_vecRows;
-	UT_Vector               m_vecColumns;
+	UT_GenericVector<fp_TableRowColumn *> m_vecRows;
+	UT_GenericVector<fp_TableRowColumn *> m_vecColumns;
 
 // Local size request and allocation.
 
@@ -514,6 +525,6 @@ private:
 	fp_CellContainer *  m_pFirstBrokenCell;
 	fp_CellContainer *  m_pLastBrokenCell;
 	bool                m_bRecursiveClear;
-};
+ };
 
 #endif /* TABLECONTAINER_H */
