@@ -103,6 +103,7 @@ GR_Graphics * FV_FrameEdit::getGraphics(void) const
 
 void FV_FrameEdit::setMode(FV_FrameEditMode iEditMode)
 {
+        UT_DEBUGMSG(("Frame Edit mode set to %d \n",iEditMode));
 	if(iEditMode == FV_FrameEdit_NOT_ACTIVE)
 	{
 		m_pFrameLayout = NULL;
@@ -890,9 +891,11 @@ void FV_FrameEdit::setDragType(UT_sint32 x, UT_sint32 y, bool bDrawFrame)
 void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 {
 	m_bFirstDragDone = false;
+	UT_DEBUGMSG(("Mouse Left Press \n"));
 	if(!isActive())
 	{
 		setDragType(x,y,true);
+		UT_DEBUGMSG(("Was not active now %d FrameLayout %x \n",getFrameEditMode(),getFrameLayout()));
 		return;
 	}
 //
@@ -901,11 +904,23 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 	if(FV_FrameEdit_EXISTING_SELECTED == m_iFrameEditMode )
 	{
 		setDragType(x,y,true);
+		UT_DEBUGMSG(("Was Existing Selected now %d \n",getFrameEditMode()));
 		if(FV_FrameEdit_DragNothing == m_iDraggingWhat)
 		{
 			m_bFirstDragDone = false;
 			m_iFrameEditMode = FV_FrameEdit_NOT_ACTIVE;
 			drawFrame(false);
+			if(m_pFrameContainer && m_pFrameLayout)
+			{
+			  if(m_pFrameLayout->getFrameType() > FL_FRAME_TEXTBOX_TYPE)
+			  {
+			    if(m_pFrameContainer->isTightWrapped())
+			    {
+			      //			      m_pFrameContainer->clearScreen();
+			      m_pView->updateScreen(false);
+			    }
+			  }
+			} 
 			m_pFrameLayout = NULL;
 			m_pFrameContainer = NULL;
 			DELETEP(m_pFrameImage);
@@ -1171,10 +1186,10 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 //
 // If we've just selected the frame, ignore this event.
 //
-	UT_DEBUGMSG(("Doing mouse release now! \n"));
+	UT_DEBUGMSG(("Doing mouse release now! Mode %d \n", getFrameEditMode()));
 	if(FV_FrameEdit_EXISTING_SELECTED == m_iFrameEditMode)
 	{
-		UT_DEBUGMSG(("Existing Frame selected now released button \n"));
+		UT_DEBUGMSG(("Existing Frame selected now released button isActive() %d \n",isActive()));
 		return;
 	}
 	if(m_pAutoScrollTimer != NULL)
@@ -1206,7 +1221,7 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		fl_BlockLayout * pCloseBL = NULL;
 		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,&pCloseBL);
 		pf_Frag_Strux * pfFrame = NULL;
-		const XML_Char * props[26] = {"frame-type","textbox",
+		const XML_Char * props[28] = {"frame-type","textbox",
 					      "wrap-mode","wrapped-both",
 					      "position-to","column-above-text",
 					      "xpos",sXpos.c_str(),
@@ -1218,6 +1233,7 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 					      "frame-page-xpos",sPageXpos.c_str(),
 					      "frame-page-ypos",sPageYpos.c_str(),
 					      "background-color", "ffffff",
+					      "tight-wrap","0",
 					      NULL,NULL};
 //
 // This should place the the frame strux immediately after the block containing
@@ -1394,6 +1410,18 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 			sVal = pszWrapMode;
 		}
 		UT_String_setProperty(sFrameProps,sProp,sVal);		
+// tight-wrap
+
+		sProp = "tight-wrap";
+		if(!pSectionAP || !pSectionAP->getProperty("tight-wrap",pszWrapMode))
+		{
+			sVal = "0";
+		}
+		else
+		{
+			sVal = pszWrapMode;
+		}
+		UT_String_setProperty(sFrameProps,sProp,sVal);		
 
 // Xpadding
 
@@ -1624,12 +1652,14 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		}
 		PT_DocPosition posStart = m_pFrameLayout->getPosition(true);
 		PT_DocPosition posEnd = posStart + m_pFrameLayout->getLength();
+		bool bHasContent = false;
 		if(isTextBox)
 		{
 			PD_DocumentRange dr_oldFrame;
-			dr_oldFrame.set(getDoc(),posStart+2,posEnd-1);
+			dr_oldFrame.set(getDoc(),posStart+1,posEnd-1);
 			UT_DEBUGMSG(("SEVIOR: Copy to local changing frame \n"));
-			m_pView->copyToLocal(posStart+2,posEnd-1);
+			m_pView->copyToLocal(posStart+1,posEnd-1);
+			bHasContent = true;
 		}
 // Delete the frame
 
@@ -1673,9 +1703,17 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		PT_DocPosition posFrame = pfFrame->getPos();
 		if(isTextBox)
 		{
-			getDoc()->insertStrux(posFrame+1,PTX_Block);
-			getDoc()->insertStrux(posFrame+2,PTX_EndFrame);
-			m_pView->insertParaBreakIfNeededAtPos(posFrame+3);
+			if(!bHasContent)
+			{
+			     getDoc()->insertStrux(posFrame+1,PTX_Block);
+			     getDoc()->insertStrux(posFrame+2,PTX_EndFrame);
+			     m_pView->insertParaBreakIfNeededAtPos(posFrame+3);
+			}
+			else
+			{
+			     getDoc()->insertStrux(posFrame+1,PTX_EndFrame);
+			     m_pView->insertParaBreakIfNeededAtPos(posFrame+2);
+			}
 		}
 		else
 		{
@@ -1689,7 +1727,14 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		if(isTextBox)
 		{
 			UT_DEBUGMSG(("SEVIOR: Pasting from clipboard Frame changed \n"));
-			m_pView->_pasteFromLocalTo(posFrame+2);
+			if(!bHasContent)
+			{
+			     m_pView->_pasteFromLocalTo(posFrame+2);
+			}
+			else
+			{
+			     m_pView->_pasteFromLocalTo(posFrame+1);
+			}
 		}
 // Finish up with the usual stuff
 		getDoc()->setDontImmediatelyLayout(false);
@@ -1775,12 +1820,16 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 /*!
  * This method deletes the current selected frame
  */
-void FV_FrameEdit::deleteFrame(void)
+void FV_FrameEdit::deleteFrame(fl_FrameLayout * pFL)
 {
 	if(m_pFrameLayout == NULL)
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		return;
+	        m_pFrameLayout = pFL;
+		if(m_pFrameLayout == NULL)
+		{
+		  UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		  return;
+		}
 	}
 	PP_AttrProp * p_AttrProp_Before = NULL;
 
@@ -1880,7 +1929,17 @@ void FV_FrameEdit::drawFrame(bool bWithHandles)
 		if(m_iDraggingWhat == FV_FrameEdit_DragWholeFrame)
 		{
 			GR_Painter painter (getGraphics());
-			m_pFrameImage = painter.genImageFromRectangle(m_recCurFrame);
+			if(m_pFrameLayout->getFrameType() == FL_FRAME_TEXTBOX_TYPE)
+			{
+			      m_pFrameImage = painter.genImageFromRectangle(m_recCurFrame);
+			}
+			else
+		        {
+			      UT_Rect rec = m_recCurFrame;
+			      rec.left = 0;
+			      rec.top = 0;
+			      m_pFrameImage = m_pFrameLayout->getBackgroundImage()->createImageSegment(getGraphics(),rec);
+			}
 		}
 	}
 	else
