@@ -31,7 +31,7 @@
 #include "ie_types.h"
 #include "ap_types.h"
 #include "fp_types.h"
-
+#include "fl_Squiggles.h"
 #include "ev_EditBits.h"
 
 
@@ -40,6 +40,7 @@
 #include "fv_FrameEdit.h"
 #include "fv_VisualDragText.h"
 #include "fv_Selection.h"
+#include "fv_InlineImage.h"
 
 #define AUTO_SCROLL_MSECS	100
 
@@ -107,6 +108,8 @@ struct FV_DocCount
 	UT_uint32 ch_sp;
 	UT_uint32 line;
 	UT_uint32 page;
+       // sometimes people want to have a word count without header/footers included
+	UT_uint32 words_no_hdrftr;
 };
 
 class ABI_EXPORT fv_PropCache
@@ -144,6 +147,7 @@ class ABI_EXPORT FV_View : public AV_View
 	friend class GR_Caret;
 	friend class FV_FrameEdit;
 	friend class FV_VisualDragText;
+	friend class FV_VisualInlineImage;
 	friend class FV_Selection;
 	friend class CellLine;
 
@@ -167,9 +171,14 @@ public:
 
 	virtual void	cmdHyperlinkJump(UT_sint32 xPos, UT_sint32 yPos);
 	void	        cmdHyperlinkJump(PT_DocPosition pos);
+	void			cmdHyperlinkCopyLocation(PT_DocPosition pos);
 
 	virtual void	draw(const UT_Rect* pRect=static_cast<UT_Rect*>(NULL));
-
+	virtual void 	drawSelectionBox(UT_Rect & box, bool drawHandles);
+private: 
+	inline void 	_drawResizeHandle(UT_Rect & box);
+	
+public:
 	const PP_AttrProp * getAttrPropForPoint();
 
 	virtual bool	notifyListeners(const AV_ChangeMask hint);
@@ -188,6 +197,14 @@ public:
 	UT_Error		cmdInsertHyperlink(const char* szName);
 	fp_Run *        getHyperLinkRun(PT_DocPosition pos);
 	UT_Error		cmdDeleteHyperlink();
+	bool                    cmdInsertMathML(const char * szFileName,
+						PT_DocPosition pos);
+	bool                    cmdInsertEmbed(UT_ByteBuf * pBuf,PT_DocPosition pos,const char * szMime,const char * szProps);
+	bool                    cmdUpdateEmbed(UT_ByteBuf * pBuf, const char * szMime, const char * szProps);
+
+	bool                    cmdInsertLatexMath(UT_UTF8String & sLatex,
+						   UT_UTF8String & sMath);
+
 	UT_Error		cmdInsertTOC(void);
 	UT_Error		cmdHyperlinkStatusBar(UT_sint32 xPos, UT_sint32 yPos);
 
@@ -205,6 +222,7 @@ public:
 	void            pasteFromLocalTo(PT_DocPosition pos);
 	void            _pasteFromLocalTo(PT_DocPosition pos);
 	void            copyToLocal(PT_DocPosition pos1, PT_DocPosition pos2);
+	void			copyTextToClipboard(const UT_UCS4String sIncoming, bool useClipboard=true);
 
 	virtual void	getTopRulerInfo(AP_TopRulerInfo * pInfo);
 	virtual void	getTopRulerInfo(PT_DocPosition pos, AP_TopRulerInfo * pInfo);
@@ -217,6 +235,7 @@ public:
 	virtual EV_EditMouseContext getInsertionPointContext(UT_sint32 * pxPos, UT_sint32 * pyPos);
 	virtual void        updateLayout(void);
 	virtual void        rebuildLayout(void);
+	virtual void        remeasureCharsWithoutRebuild();
 	virtual bool		isSelectionEmpty(void) const;
 	virtual void		cmdUnselectSelection(void);
 	void				getDocumentRangeOfCurrentSelection(PD_DocumentRange * pdr);
@@ -259,7 +278,7 @@ public:
 	bool	processPageNumber(HdrFtrType hfType, const XML_Char ** atts);
 
 	bool	isTextMisspelled()const ;
-	bool	isTabListBehindPoint(void);
+	bool	isTabListBehindPoint(UT_sint32 & iNumToDelete);
 	bool	isTabListAheadPoint(void);
 	void	processSelectedBlocks(FL_ListType listType);
 	void	getBlocksInSelection(UT_GenericVector<fl_BlockLayout*> * vBlock);
@@ -366,6 +385,19 @@ public:
 	void            dragVisualText(UT_sint32 x, UT_sint32 y);
 	void            pasteVisualText(UT_sint32 x, UT_sint32 y);
 	void            btn0VisualDrag(UT_sint32 x, UT_sint32 y);
+	FV_VisualDragText * getVisualText(void)
+	  { return &m_VisualDragText;}
+
+
+//---------
+//Visual Inline Image Drag stuff
+//
+	void            btn0InlineImage(UT_sint32 x, UT_sint32 y);
+	void            btn1InlineImage(UT_sint32 x, UT_sint32 y);
+	void            btn1CopyImage(UT_sint32 x, UT_sint32 y);
+	void            dragInlineImage(UT_sint32 x, UT_sint32 y);
+	void            releaseInlineImage(UT_sint32 x, UT_sint32 y);
+
 // -------
 // Frame stuff
 //
@@ -381,10 +413,13 @@ public:
 	fl_FrameLayout * getFrameLayout(PT_DocPosition pos);
 	fl_FrameLayout * getFrameLayout(void);
 	void            setFrameFormat(const XML_Char ** props);
+	void            setFrameFormat(const XML_Char ** attribs, const XML_Char ** props);
 	void            setFrameFormat(const XML_Char ** props,FG_Graphic * pFG, UT_String & dataID);
 	void            convertInLineToPositioned(PT_DocPosition pos, 
 											const XML_Char ** attribs);
 
+	bool            convertPositionedToInLine(fl_FrameLayout * pFrame);
+
 // ----------------------
 
 	bool			isPosSelected(PT_DocPosition pos) const;
@@ -586,6 +621,8 @@ public:
 	bool                cmdSelectColumn(PT_DocPosition posOfColumn);
 	bool                cmdAutoFitTable(void);
 	bool                cmdMergeCells(PT_DocPosition posSource, PT_DocPosition posDestination);
+	bool                cmdTableToText(PT_DocPosition posSource,UT_sint32 iSepType);
+
 	bool                _MergeCells( PT_DocPosition posDestination,PT_DocPosition posSource, bool bBefore);
 	bool                getCellParams(PT_DocPosition posCol, UT_sint32 *iLeft, 
 									  UT_sint32 *iRight,UT_sint32 *iTop, UT_sint32 *iBot);
@@ -614,7 +651,7 @@ public:
 	void				_generalUpdate(void);
 	
 	UT_RGBColor			getColorShowPara(void) const { return m_colorShowPara; }
-	UT_RGBColor			getColorSquiggle(void) const { return m_colorSquiggle; }
+	UT_RGBColor			getColorSquiggle(FL_SQUIGGLE_TYPE iSquiggleType) const;
 	UT_RGBColor			getColorMargin(void) const { return m_colorMargin; }
 	UT_RGBColor			getColorSelBackground(void);
 	UT_RGBColor			getColorSelForeground(void);
@@ -857,7 +894,8 @@ private:
 	
 	// default color values
 	UT_RGBColor			m_colorShowPara;
-	UT_RGBColor			m_colorSquiggle;
+	UT_RGBColor			m_colorSpellSquiggle;
+	UT_RGBColor			m_colorGrammarSquiggle;
 	UT_RGBColor			m_colorMargin;
 	UT_RGBColor			m_colorFieldOffset;
 	UT_RGBColor			m_colorImage;
@@ -892,6 +930,9 @@ private:
 	bool                m_bDontNotifyListeners;
 	UT_ByteBuf *        m_pLocalBuf;
 	UT_sint32           m_iGrabCell;
+	FV_VisualInlineImage  m_InlineImage;
+	bool                m_bInsertAtTablePending;
+	PT_DocPosition      m_iPosAtTable;
 };
 
 #endif /* FV_VIEW_H */
