@@ -51,7 +51,8 @@ FV_VisualDragText::FV_VisualDragText (FV_View * pView)
 	  m_pAutoScrollTimer(NULL),
 	  m_xLastMouse(1),
 	  m_yLastMouse(1),
-	  m_bDoingCopy(false)
+	  m_bDoingCopy(false),
+	  m_bNotDraggingImage(false)
 {
 	UT_ASSERT (pView);
 }
@@ -286,27 +287,30 @@ void FV_VisualDragText::mouseDrag(UT_sint32 x, UT_sint32 y)
 		expY.height = dy + 2*iext;
 	}
 
-	if(expX.width > 0)
+	if(!m_bNotDraggingImage && (expX.width > 0))
 	{
 		getGraphics()->setClipRect(&expX);
 		m_pView->updateScreen(false);
 	}
-	if(expY.height > 0)
+	if(!m_bNotDraggingImage && (expY.height > 0))
 	{
 		getGraphics()->setClipRect(&expY);
 		m_pView->updateScreen(false);
 	}
-	getGraphics()->setClipRect(NULL);
-	drawImage();
-	if(m_recOrigLeft.width > 0)
+	if(!m_bNotDraggingImage)
 	{
-		getGraphics()->setClipRect(&m_recOrigLeft);
-		m_pView->updateScreen(false);
-	}
-	if(m_recOrigRight.width > 0)
-	{
-		getGraphics()->setClipRect(&m_recOrigRight);
-		m_pView->updateScreen(false);
+	        getGraphics()->setClipRect(NULL);
+		drawImage();
+		if(m_recOrigLeft.width > 0)
+		{
+		     getGraphics()->setClipRect(&m_recOrigLeft);
+		     m_pView->updateScreen(false);
+		}
+		if(m_recOrigRight.width > 0)
+		{
+		     getGraphics()->setClipRect(&m_recOrigRight);
+		     m_pView->updateScreen(false);
+		}
 	}
 	m_iLastX = x;
 	m_iLastY = y;
@@ -378,6 +382,8 @@ void FV_VisualDragText::getImageFromSelection(UT_sint32 x, UT_sint32 y)
 	UT_sint32 xCaret2, yCaret2;
 	bool bDirection = false;
 	bool bEOL = false;
+	fp_Page * pPageLow = NULL;
+	fp_Page * pPageHigh = NULL;
 	if(m_pView->getSelectionMode() < 	FV_SelectionMode_Multiple)
 	{
 		if(m_pView->getSelectionAnchor() < m_pView->getPoint())
@@ -469,6 +475,48 @@ void FV_VisualDragText::getImageFromSelection(UT_sint32 x, UT_sint32 y)
 	fp_Run * pRunHigh = NULL;
 	m_pView->_findPositionCoords(posHigh, bEOL, xHigh, yHigh, xCaret2, yCaret2, heightCaret, bDirection, NULL, &pRunHigh);
 	fp_Line * pLineHigh = pRunHigh->getLine();
+	pPageLow = pLineLow->getPage();
+	pPageHigh = pLineHigh->getPage();
+	//
+	// Decide if the selection is fully on screen and all on the same page.
+	//
+	bool bOffscreen = false;
+	if(pPageLow != pPageHigh)
+	{
+	     bOffscreen = true;
+	}
+	if(!bOffscreen && ((yLow <0) || (yHigh >  m_pView->getWindowHeight())))
+	{
+	     bOffscreen = true;
+	}
+	if(!bOffscreen && ((xLow <0) || (xHigh <0) || (xLow >  m_pView->getWindowWidth()) ||(xLow >  m_pView->getWindowWidth())))
+	{
+	     bOffscreen = true;
+	}
+	if(bOffscreen)
+	{
+	     m_bNotDraggingImage = true;
+	     m_recCurFrame.left = x - 1;
+	     m_recCurFrame.top = y - 1;
+	     m_recCurFrame.width = 2;
+	     m_recCurFrame.height = 2;
+	     m_iInitialOffX = 1;
+	     m_iInitialOffY = 1;
+	     m_iLastX = x;
+	     m_iLastY = y;
+	     m_recOrigLeft.width = 2;
+	     m_recOrigLeft.height = 2;
+	     m_recOrigLeft.left = x-1;
+	     m_recOrigLeft.top = y-1;
+	     GR_Graphics::Cursor cursor = GR_Graphics::GR_CURSOR_DRAGTEXT;
+	     if(isDoingCopy())
+	     {
+	       cursor = GR_Graphics::GR_CURSOR_COPYTEXT;
+	     }
+	     getGraphics()->setCursor(cursor);
+	     return;
+	}
+	m_bNotDraggingImage = false;
 //
 // OK deal with the nice case of the selection just on the single line
 //
@@ -512,7 +560,6 @@ void FV_VisualDragText::getImageFromSelection(UT_sint32 x, UT_sint32 y)
 		m_recCurFrame.top = yy;
 		fp_Line * pNext = pLineLow;
 		UT_sint32 width = 0;
-		UT_sint32 iRightpos = 0;
 		while(pNext && (pNext != pLineHigh))
 		{
 			pRun = pNext->getFirstRun();
@@ -589,6 +636,12 @@ void FV_VisualDragText::mouseCut(UT_sint32 x, UT_sint32 y)
 {
 	getImageFromSelection(x,y);
 	bool bPasteTableCol = (m_pView->getSelectionMode() == FV_SelectionMode_TableColumn);
+
+	// flag up on the document level that we are dragging with the mouse
+	// (in revisions mode the PT needs to be able to make a distinction between normal
+	// cut/delete and the mouse cut)
+	m_pView->getDocument()->setVDNDinProgress(true);
+	
 	if(bPasteTableCol)
 	{
 		m_pView->cmdCut();
@@ -605,6 +658,9 @@ void FV_VisualDragText::mouseCut(UT_sint32 x, UT_sint32 y)
 		m_pView->copyToLocal(pos1,pos2);
 		m_pView->cmdCharDelete(true,1);
 	}
+
+	m_pView->getDocument()->setVDNDinProgress(false);
+	
 	m_pView->updateScreen(false);
 	drawImage();
 }
@@ -650,6 +706,38 @@ PT_DocPosition FV_VisualDragText::getPosFromXY(UT_sint32 x, UT_sint32 y)
 	return posAtXY;
 }
 
+/*!
+ * This method oborts the current visual drag.
+ */
+void   FV_VisualDragText::abortDrag(void)
+{
+        if(m_pAutoScrollTimer != NULL)
+	{
+		m_pAutoScrollTimer->stop();
+		DELETEP(m_pAutoScrollTimer);
+	}
+	bool bDidCopy = m_bDoingCopy;
+	m_bDoingCopy = false;
+	m_bNotDraggingImage = false;
+	clearCursor();
+	if(m_iVisualDragMode != FV_VisualDrag_DRAGGING)
+	{
+//
+// we didn't actually drag anything. Just release the selection.
+//
+	        setMode(FV_VisualDrag_NOT_ACTIVE);
+		return;
+	}
+	getGraphics()->setClipRect(&m_recCurFrame);
+	m_pView->updateScreen(false);
+	getGraphics()->setClipRect(NULL);
+	setMode(FV_VisualDrag_NOT_ACTIVE);
+	if(!bDidCopy)
+	{
+	  m_pView->cmdUndo(1);
+	}
+	return;
+}
 /*!
  * x and y is the location in the document windows of the mouse in logical
  * units.
@@ -662,6 +750,7 @@ void FV_VisualDragText::mouseRelease(UT_sint32 x, UT_sint32 y)
 		DELETEP(m_pAutoScrollTimer);
 	}
 	m_bDoingCopy = false;
+	m_bNotDraggingImage = false;
 	clearCursor();
 	if(m_iVisualDragMode != FV_VisualDrag_DRAGGING)
 	{
@@ -736,6 +825,16 @@ void FV_VisualDragText::mouseRelease(UT_sint32 x, UT_sint32 y)
 
 void FV_VisualDragText::drawImage(void)
 {
+        if(m_bNotDraggingImage)
+	{ 
+	  GR_Graphics::Cursor cursor = GR_Graphics::GR_CURSOR_DRAGTEXT;
+	  if(isDoingCopy())
+	  {
+	      cursor = GR_Graphics::GR_CURSOR_COPYTEXT;
+	  }
+	  getGraphics()->setCursor(cursor);
+	  return;
+	}
 	if(m_pDragImage == NULL)
 	{
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
