@@ -258,7 +258,6 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 			if (!*q)
 			{
 				free(pOrig);
-				UT_DEBUGMSG(("props: %s\n", szValue));
 				return false;
 			}
 
@@ -291,8 +290,22 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 		free(pOrig);
 		return true;
 	}
-	else								// not "PROPS" -- add to attribute list
+	else if (0 == strcmp(szName, PT_XID_ATTRIBUTE_NAME) && *szValue)
 	{
+		// XID is a unique id for the xml element / PT frag. Its function is to facilitate
+		// comparing/merging documents and we do not want it in the AP
+		return true;
+	}
+	else // not "PROPS" -- add to attribute list
+	{
+		UT_UTF8String url;
+		if (szValue && *szValue && (0 == strcmp(szName, "xlink:href") || 0 == strcmp(szName, "href")))
+		{
+			url = szValue;
+			url.decodeURL();
+			szValue = url.utf8_str();
+		}
+		
 		if (!m_pAttributes)
 		{
 			m_pAttributes = new UT_GenericStringMap<XML_Char*>(5);
@@ -560,13 +573,28 @@ bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char
 			    It seems that we also want empty strings for attributes,
 				at least for the 'param' attribute which goes with fields.
 				Is this bogus too? -PL
+
+				Yes. 
+				We use empty strings and NULL in values to indicate that the
+				attribute/property should be absent. Sometimes these values filter down
+				here, so we need to handle this. TF
 			*/
-			UT_return_val_if_fail (p[1] /* && *p[1]*/, false);	// require value for each name
+			// UT_return_val_if_fail (p[1] /* && *p[1]*/, false);	// require value for each name
+
+			// first deal with the case where the value is set to NULL or "" -- we want
+			// that attribute to be absent, not present
 			const XML_Char * szValue = NULL;
-			if (!getAttribute(p[0],szValue))
-				return false;		// item not present
-			if (strcmp(p[1],szValue)!=0)
-				return false;		// item has different value
+
+			if((!p[1] || !*p[1]) && getAttribute(p[0],szValue) && szValue && *szValue)
+				return false;
+			else if(p[1] && *p[1])
+			{
+				if (!getAttribute(p[0],szValue))
+					return false;		// item not present
+				if (strcmp(p[1],szValue)!=0)
+					return false;		// item has different value
+			}
+					
 			p += 2;
 		}
 	}
@@ -580,13 +608,27 @@ bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char
 				Jeff, I weakened the following assert because we
 				*want* to represent no tabstops as an empty string.
 				If this isn't safe, let me know.   -- PCR
+
+				We use empty strings and NULL in values to indicate that the
+				attribute/property should be absent. Sometimes these values filter down
+				here, so we need to handle this. TF
 			*/
-			UT_return_val_if_fail (p[1] /* && *p[1] */, false);	// require value for each name
+			// UT_return_val_if_fail (p[1] /* && *p[1]*/, false);	// require value for each name
+
+			// first deal with the case where the value is set to NULL or "" -- we want
+			// that attribute to be absent, not present
 			const XML_Char * szValue = NULL;
-			if (!getProperty(p[0],szValue))
-				return false;		// item not present
-			if (strcmp(p[1],szValue)!=0)
-				return false;		// item has different value
+
+			if((!p[1] || !*p[1]) && getProperty(p[0],szValue) && szValue && *szValue)
+				return false;
+			else if(p[1] && p[1])
+			{
+				if (!getProperty(p[0],szValue))
+					return false;		// item not present
+				if (strcmp(p[1],szValue)!=0)
+					return false;		// item has different value
+			}
+			
 			p += 2;
 		}
 	}
@@ -688,7 +730,7 @@ bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 
 			v1 = ca1.next();
 			v2 = ca2.next();
-		} while (ca1.is_valid());
+		} while (ca1.is_valid() && ca2.is_valid());
 	}
 
 	if (countMyProps > 0)
@@ -715,7 +757,7 @@ bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 
 			v1 = cp1.next();
 			v2 = cp2.next();
-		} while (cp1.is_valid());
+		} while (cp1.is_valid() && cp2.is_valid());
 
 	#ifdef PT_TEST
 		s_Matches++;
