@@ -86,6 +86,20 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 {
 	if(m_pDocument->isMarkRevisions())
 	{
+		// if the user selected the whole document for deletion, we will not delete the
+		// first block (we need always a visible block in any document); we make an
+		// exception to this in VDND, because in that case the original block is
+		// guaranteed to come back
+		// 
+		// NB: it is possible that the user might delete all contents in several separate
+		// steps; there is no easy way to protect against that
+		bool bWholeDoc = false;
+		if(!m_pDocument->isVDNDinProgress())
+		{
+			pf_Frag * pLast = getFragments().getLast();
+			bWholeDoc = (dpos1 <= 2 && pLast->getPos() == dpos2);
+		}
+		
 		iRealDeleteCount = 0;
 
 		const XML_Char name[] = "revision";
@@ -135,6 +149,12 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				{
 					case PTX_Block:
 						iLen = pf_FRAG_STRUX_BLOCK_LENGTH;
+						if(bWholeDoc && dpos1 == 2)
+						{
+							dpos1 += iLen;
+							continue;
+						}
+						
 						break;
 						
 					case PTX_SectionTable:
@@ -315,7 +335,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 			{
 				case pf_Frag::PFT_Object:
 				case pf_Frag::PFT_Text:
-					if(! _realChangeSpanFmt(PTC_AddFmt, dpos1, dposEnd, ppRevAttrib,NULL))
+					if(! _realChangeSpanFmt(PTC_AddFmt, dpos1, dposEnd, ppRevAttrib,NULL,true))
 						return false;
 					break;
 
@@ -325,7 +345,8 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 					// it the position immediately after the strux; we
 					// only want the one strux changed, so we pass
 					// identical position in both parameters
-					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1 + iLen, dpos1 + iLen /*2*iLen*/, ppRevAttrib,NULL,eStruxType))
+					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1 + iLen, dpos1 + iLen /*2*iLen*/, ppRevAttrib,NULL,
+											 eStruxType,true))
 						return false;
 					break;
 
@@ -580,10 +601,18 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 #endif
 	case PTX_SectionHdrFtr:
 		// if the previous container is a Header/Footersection, then pf_First
-		// must be the first block in the section.
+		// must be the first block or the first Table in the section.
 		UT_return_val_if_fail ((pf_First->getPrev() == pfsContainer),false);
 		UT_return_val_if_fail ((pf_First->getType() == pf_Frag::PFT_Strux),false);
-		UT_return_val_if_fail (((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block),false);
+		UT_return_val_if_fail ((((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block) || (static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_SectionTable),false);
+
+		//
+		// This allows us to delete the first Table in a section
+		//
+		if(static_cast<pf_Frag_Strux *>(pf_First)->getStruxType() == PTX_SectionTable)
+		{
+		     return true;
+		}
 		// since, we cannot delete the first block in a section, we
 		// secretly translate this into a request to delete the section;
 		// the block we have will then be slurped into the previous
