@@ -372,6 +372,14 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 
 
 	case PTX_Section:
+	  //
+	  // We can delete Tables provided there is a block to take it's place later
+	  //
+	        if((pfs->getStruxType() == PTX_SectionTable) || (pfs->getStruxType() == PTX_EndTable))
+		{
+		    _unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		    return true;
+		}
 		// there are no blocks (paragraphs) between this section
 		// and the previous section.  this is not possible.
 		// TODO decide if this should assert or just fail...
@@ -381,6 +389,11 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 
 
 	case PTX_SectionHdrFtr:
+	        if((pfs->getStruxType() == PTX_SectionTable) || (pfs->getStruxType() == PTX_EndTable))
+		{
+		    _unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		    return true;
+		}
 		// there are no blocks (paragraphs) between this section
 		// and the previous section.  this is not possible.
 		// TODO decide if this should assert or just fail...
@@ -407,7 +420,7 @@ bool pt_PieceTable::_deleteStruxWithNotify(PT_DocPosition dpos,
 {
 	PX_ChangeRecord_Strux * pcrs
 		= new PX_ChangeRecord_Strux(PX_ChangeRecord::PXT_DeleteStrux,
-									dpos, pfs->getIndexAP(), pfs->getStruxType());
+									dpos, pfs->getIndexAP(), pfs->getXID(), pfs->getStruxType());
 	UT_return_val_if_fail (pcrs, false);
 
 	if (!_unlinkStrux(pfs,ppfEnd,pfragOffsetEnd))
@@ -427,7 +440,7 @@ bool pt_PieceTable::_deleteStruxWithNotify(PT_DocPosition dpos,
  * This method scans the piecetAble from the section Frag_strux given looking
  * for any Header/Footers that belong to the strux. If it finds them, they
  * are deleted with notifications.
-\params pf_Frag_Strux_Section pfStruxSec the Section strux that might have headers
+\param pf_Frag_Strux_Section pfStruxSec the Section strux that might have headers
  *                                        or footers belonging to it.
  * These must be deleted with notification otherwise they won't be recreated on
  * an undo
