@@ -73,26 +73,29 @@ public:
 //! PTStruxType tells the sub-type of a FragStrux.
 typedef enum _PTStruxType
 {
-	PTX_Section = 0,         // 0 -- maker sure that we can cast into uint
-	PTX_Block,           // 1
-	PTX_SectionHdrFtr,   // 2
-	PTX_SectionEndnote,  // 3
-	PTX_SectionTable,    // 4
-	PTX_SectionCell,     // 5
-	PTX_SectionFootnote, // 6
-	PTX_SectionMarginnote, // 7
-	PTX_SectionFrame,      // 8
-	PTX_EndCell,           // 9
-	PTX_EndTable,          // 10
+	PTX_Section = 0,       	// 0 -- maker sure that we can cast into uint
+	PTX_Block,           	// 1
+	PTX_SectionHdrFtr,   	// 2
+	PTX_SectionEndnote,  	// 3
+	PTX_SectionTable,    	// 4
+	PTX_SectionCell,     	// 5
+	PTX_SectionFootnote, 	// 6
+	PTX_SectionMarginnote, 	// 7
+	PTX_SectionFrame,      	// 8
+	PTX_SectionTOC,			// 9
+	PTX_EndCell,          	// 10
+	PTX_EndTable,
     PTX_EndFootnote,
     PTX_EndMarginnote,
     PTX_EndEndnote,
-    PTX_EndFrame
+    PTX_EndFrame,
+	PTX_EndTOC,
+	PTX_StruxDummy
 } PTStruxType;
 
 //! PTObjectType tells the sub-type of an FragObject.
 
-typedef enum _PTObjectType { PTO_Image = 0, PTO_Field, PTO_Bookmark, PTO_Hyperlink } PTObjectType;
+typedef enum _PTObjectType { PTO_Image = 0, PTO_Field, PTO_Bookmark, PTO_Hyperlink, PTO_Math, PTO_Embed } PTObjectType;
 
 //! PieceTable states
 typedef enum _PTState { PTS_Create=0, PTS_Loading=1, PTS_Editing=2 } PTState;
@@ -134,14 +137,8 @@ typedef const void * PL_StruxFmtHandle;
 #define PT_FOOTERLAST_ATTRIBUTE_NAME	(static_cast<const XML_Char *>("footer-last"))
 #define PT_REVISION_ATTRIBUTE_NAME      (static_cast<const XML_Char *>("revision"))
 #define PT_ID_ATTRIBUTE_NAME            (static_cast<const XML_Char *>("id"))
+#define PT_STRUX_IMAGE_DATAID           (static_cast<const XML_Char *>("strux-image-dataid"))
+#define PT_XID_ATTRIBUTE_NAME           (static_cast<const XML_Char *>("xid"))
 
 
 #endif /* PT_TYPES_H */
-
-
-
-
-
-
-
-
