@@ -365,11 +365,6 @@ bool AP_Win32App::initialize(void)
 	}
 	_findclose( findtag );
 
-	/* SPI modules don't register automatically on loading, so
-	 * now that we've loaded the modules we need to register them:
-	 */
-	XAP_ModuleManager::instance().registerPending ();
-
 	return bSuccess;
 }
 
@@ -1121,7 +1116,7 @@ static LRESULT CALLBACK _SplashWndProc(HWND hWnd, UINT message, WPARAM wParam, L
         break;
         
     default:
-        return (DefWindowProc(hWnd, message, wParam, lParam));
+        return (UT_DefWindowProc(hWnd, message, wParam, lParam));
     }
     return (0);
 }
@@ -1144,26 +1139,15 @@ static GR_Image * _showSplash(HINSTANCE hInstance, const char * szAppName)
 		)
 	{
 		// NB: can't access 'this' members from a static member function
-		WNDCLASSEX  wndclass;
 		ATOM a;
 	
 		sprintf(s_SplashWndClassName, "%sSplash", szAppName /* app->getApplicationName() */);
 
-		// register class for the splash window
-		wndclass.cbSize        = sizeof(wndclass);
-		wndclass.style         = 0;
-		wndclass.lpfnWndProc   = _SplashWndProc;
-		wndclass.cbClsExtra    = 0;
-		wndclass.cbWndExtra    = 0;
-		wndclass.hInstance     = hInstance /* app->getInstance() */;
-		wndclass.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_32)) /* app->getIcon() */;
-		wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
-		wndclass.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
-		wndclass.lpszMenuName  = NULL;
-		wndclass.lpszClassName = s_SplashWndClassName;
-		wndclass.hIconSm       = LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_16)) /* app->getSmallIcon() */;
-
-		a = RegisterClassEx(&wndclass);
+		a = UT_RegisterClassEx(0L, _SplashWndProc, hInstance,
+							   LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_32)),
+							   LoadCursor(NULL, IDC_ARROW), (HBRUSH) GetStockObject(NULL_BRUSH),
+							   LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_16)),
+							   NULL, s_SplashWndClassName);
 		UT_ASSERT_HARMLESS(a);
 
 		// get the extents of the desktop window
@@ -1176,7 +1160,7 @@ static GR_Image * _showSplash(HINSTANCE hInstance, const char * szAppName)
 		UT_PNG_getDimensions(pBB, iSplashWidth, iSplashHeight);
 
 		// create a centered window the size of our bitmap
-		hwndSplash = CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST,s_SplashWndClassName, 
+		hwndSplash = UT_CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST,s_SplashWndClassName, 
 								  NULL, WS_POPUP | WS_BORDER,
 								  (rect.right  / 2) - (iSplashWidth  / 2) - 1, // subtract 1 pixel to account for WS_BORDER
 								  (rect.bottom / 2) - (iSplashHeight / 2) - 1, // subtract 1 pixel to account for WS_BORDER
@@ -1333,7 +1317,7 @@ try
 		}	
 
 		// do dispatch loop
-		while( GetMessage(&msg, NULL, 0, 0) )
+		while(UT_GetMessage(&msg, NULL, 0, 0))
 	    {
    	      	// TranslateMessage is not called because AbiWord
 	      	// has its own way of decoding keyboard accelerators
@@ -1341,7 +1325,7 @@ try
 				continue;
 				
 			TranslateMessage(&msg);	
-	    	DispatchMessage(&msg);	
+			UT_DispatchMessage(&msg);
 	    	
 			// Check for idle condition
 			while( !UT_Win32Idle::_isEmpty() &&
@@ -1740,3 +1724,14 @@ UT_UTF8String	AP_Win32App::s_fromWinLocaleToUTF8(const char* szIn)
 
 	return sRslt;
 }
+
+GR_Graphics * AP_Win32App::newDefaultScreenGraphics() const
+{
+	XAP_Frame * pFrame = findValidFrame();
+	UT_return_val_if_fail( pFrame, NULL );
+	
+	AP_Win32FrameImpl * pFI = (AP_Win32FrameImpl *) pFrame->getFrameImpl();
+	UT_return_val_if_fail( pFI, NULL );
+
+	return pFI->createDocWndGraphics();
+}
