@@ -192,7 +192,6 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 	UT_return_val_if_fail (pApp && pLabel, NULL);
 
 	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_FILE_RECENT_1);
-	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_FILE_RECENT_9);
 
 	UT_uint32 ndx = (id - AP_MENU_ID_FILE_RECENT_1 + 1);
 
@@ -610,7 +609,24 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_HyperlinkOK)
 	    s = EV_MIS_Gray ;
 		return s;
 	}
-
+	if(pBL1->getLength() == 1)
+	{
+	    s = EV_MIS_Gray ;
+	    return s;
+	}
+	PT_DocPosition posTemp =0;
+	if(posStart > posEnd)
+	{
+	  posTemp = posStart;
+	  posStart = posEnd;
+	  posEnd = posStart;
+	}
+	if(posStart < pBL1->getPosition(true))
+	{
+	    s = EV_MIS_Gray ;
+	    return s;
+	}
+	  
 	return s ;
 }
 
@@ -1114,8 +1130,6 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	case AP_MENU_ID_VIEW_RULER:
 		if ( pFrameData->m_bShowRuler && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
-		else if( pFrameData->m_bIsFullScreen )
-			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
@@ -1134,32 +1148,24 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	case AP_MENU_ID_VIEW_TB_1:
 		if ( pFrameData->m_bShowBar[0] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
-		else if( pFrameData->m_bIsFullScreen )
-			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_2:	
 		if ( pFrameData->m_bShowBar[1] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
-		else if( pFrameData->m_bIsFullScreen )
-			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_3:	
 		if ( pFrameData->m_bShowBar[2] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
-		else if( pFrameData->m_bIsFullScreen )
-			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_4:	
 		if ( pFrameData->m_bShowBar[3] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
-		else if( pFrameData->m_bIsFullScreen )
-			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
@@ -1177,8 +1183,6 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
               if ( pFrameData->m_bShowStatusBar &&
 				   !pFrameData->m_bIsFullScreen )
 				  s = EV_MIS_Toggled;
-			  else if( pFrameData->m_bIsFullScreen )
-				  s = EV_MIS_Gray;
 			  else
 				  s = EV_MIS_ZERO;
 			  break;
@@ -1245,7 +1249,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_History)
     return EV_MIS_ZERO;
 }
 
-Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
+Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisionsCheck)
 {
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
@@ -1263,6 +1267,24 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
     return EV_MIS_ZERO;
 }
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->isAutoRevisioning())
+	{
+		return EV_MIS_Gray;
+	}
+
+	if(pView->isMarkRevisions())
+	{
+		return EV_MIS_ZERO;
+	}
+
+    return EV_MIS_Gray;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionsSelectLevel)
 {
 	ABIWORD_VIEW;
@@ -1438,6 +1460,24 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InTable)
 }
 
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableIsRepeat)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->isInTable())
+	{
+	        fp_CellContainer * pCCon = pView->getCellAtPos(pView->getPoint());
+		if(pCCon && pCCon->isRepeated())
+	        {
+		     return EV_MIS_ZERO;
+		}
+		return EV_MIS_Gray;
+	}
+    return EV_MIS_Gray;
+}
+
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_TableOK)
 {
 	ABIWORD_VIEW;
@@ -1520,6 +1560,15 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InImage)
 	{
 		return EV_MIS_ZERO;
 	}
+	if(pView->getFrameEdit()->isActive())
+	{
+	        fl_FrameLayout * pFL = pView->getFrameLayout();
+		if(pFL->getFrameType() == FL_FRAME_TEXTBOX_TYPE)
+		{
+		        return EV_MIS_Gray;
+		}
+		return EV_MIS_ZERO;
+	}
 	return EV_MIS_Gray;
 }
 
