@@ -43,15 +43,34 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Prefs);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_StylesLocked);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ScriptsActive);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_SomethingSelected);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_HyperlinkOK);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Spelling);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ColumnsActive);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_AutoRevision);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisionsCheck);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_HasRevisions);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisions);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfter);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfterPrev);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsBefore);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresent);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresentContext);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionsSelectLevel);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_History);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_FmtHdrFtr);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_BreakOK);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_TOCOK);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_BookmarkOK);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_TableOK);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_TextToTableOK);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InTOC);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InTable);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableIsRepeat);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableMerged);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InFootnote);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InImage);
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InFrame);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_AlwaysDisabled); // REMOVE ME
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Recent);
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Zoom);
