@@ -31,14 +31,31 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_FILE_NEW)
 		MenuItem(AP_MENU_ID_FILE_NEW_USING_TEMPLATE)
 		MenuItem(AP_MENU_ID_FILE_OPEN)
+		MenuItem(AP_MENU_ID_FILE_IMPORTSTYLES)
+
+		Separator()
+
 		MenuItem(AP_MENU_ID_FILE_SAVE)
 		MenuItem(AP_MENU_ID_FILE_SAVEAS)
 //		MenuItem(AP_MENU_ID_FILE_SAVE_TEMPLATE)
-		Separator()
-		MenuItem(AP_MENU_ID_FILE_IMPORT)
+//		MenuItem(AP_MENU_ID_FILE_IMPORT)
 		MenuItem(AP_MENU_ID_FILE_EXPORT)
 		MenuItem(AP_MENU_ID_FILE_REVERT)
 
+		Separator()
+		MenuItem(AP_MENU_ID_FILE_PAGESETUP)
+		MenuItem(AP_MENU_ID_FILE_PRINT_PREVIEW)
+		MenuItem(AP_MENU_ID_FILE_PRINT)
+		Separator()
+		MenuItem(AP_MENU_ID_FILE_PROPERTIES)
+		Separator()
+#ifdef XP_UNIX_TARGET_GTK
+		// GNOME HIG style recent files 
+		MenuItem(AP_MENU_ID_FILE_RECENT_1)
+		MenuItem(AP_MENU_ID_FILE_RECENT_2)
+		MenuItem(AP_MENU_ID_FILE_RECENT_3)
+		MenuItem(AP_MENU_ID_FILE_RECENT_4)
+#else
 		BeginSubMenu(AP_MENU_ID_FILE_RECENT)
 			MenuItem(AP_MENU_ID_FILE_RECENT_1)
 			MenuItem(AP_MENU_ID_FILE_RECENT_2)
@@ -50,14 +67,7 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_FILE_RECENT_8)
 			MenuItem(AP_MENU_ID_FILE_RECENT_9)
 		EndSubMenu()
-
-		Separator()
-		MenuItem(AP_MENU_ID_FILE_PROPERTIES)
-		MenuItem(AP_MENU_ID_FILE_PAGESETUP)
-#if defined(HAVE_GNOME)
-		MenuItem(AP_MENU_ID_FILE_PRINT_PREVIEW)
 #endif
-		MenuItem(AP_MENU_ID_FILE_PRINT)
 		Separator()
 		MenuItem(AP_MENU_ID_FILE_CLOSE)
 		MenuItem(AP_MENU_ID_FILE_EXIT)
@@ -75,6 +85,9 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_EDIT_CLEAR)
 		MenuItem(AP_MENU_ID_EDIT_SELECTALL)
 		Separator()
+		MenuItem(AP_MENU_ID_EDIT_REMOVEHEADER)
+		MenuItem(AP_MENU_ID_EDIT_REMOVEFOOTER)
+		Separator()
 		MenuItem(AP_MENU_ID_EDIT_FIND)
 		MenuItem(AP_MENU_ID_EDIT_REPLACE)
 		MenuItem(AP_MENU_ID_EDIT_GOTO)
@@ -227,7 +240,6 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_FMT_FONT)
 		MenuItem(AP_MENU_ID_FMT_PARAGRAPH)
 		MenuItem(AP_MENU_ID_FMT_BULLETS)
-		MenuItem(AP_MENU_ID_FMT_DOCUMENT)
 		MenuItem(AP_MENU_ID_FMT_FRAME)
         MenuItem(AP_MENU_ID_FMT_IMAGE)
 #if 0 // someone code and turn this back on
@@ -262,37 +274,29 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_FMT_SUBSCRIPT)
 		EndSubMenu()
 
-		BeginSubMenu(AP_MENU_ID_FMT_STYLE)
-			MenuItem(AP_MENU_ID_FMT_STYLE_DEFINE)
-			MenuItem(AP_MENU_ID_FMT_IMPORTSTYLES)
-	        MenuItem(AP_MENU_ID_FMT_STYLIST)
-		EndSubMenu()
-		Separator()
-		MenuItem(AP_MENU_ID_EDIT_REMOVEHEADER)
-		MenuItem(AP_MENU_ID_EDIT_REMOVEFOOTER)
-
 		BeginSubMenu(AP_MENU_ID_FMT_BACKGROUND)
-		    MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_IMAGE)
-		    MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_COLOR)
-	    EndSubMenu()
+			MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_IMAGE)
+			MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_COLOR)
+		EndSubMenu()
 
 	    BeginSubMenu(AP_MENU_ID_FMT_DIRECTION)
-	        MenuItem(AP_MENU_ID_FMT_DIRECTION_DD_RTL)
+			MenuItem(AP_MENU_ID_FMT_DIRECTION_DD_RTL)
         	MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_LTR)
         	MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_RTL)
     	EndSubMenu()
+
+		Separator()
+
+		MenuItem(AP_MENU_ID_FMT_STYLE_DEFINE)
+
 	EndSubMenu()
 
 	BeginSubMenu(AP_MENU_ID_TOOLS)
 
-		BeginSubMenu(AP_MENU_ID_TOOLS_SPELLING)
-			MenuItem(AP_MENU_ID_TOOLS_SPELL)
-			MenuItem(AP_MENU_ID_TOOLS_AUTOSPELL)
-			MenuItem(AP_MENU_ID_TOOLS_SPELLPREFS)
-		EndSubMenu()
-
+		MenuItem(AP_MENU_ID_TOOLS_SPELL)
 		MenuItem(AP_MENU_ID_FMT_LANGUAGE)
 		MenuItem(AP_MENU_ID_TOOLS_WORDCOUNT)
+        MenuItem(AP_MENU_ID_FMT_STYLIST)
 
 		Separator()
 
@@ -300,10 +304,12 @@ BeginLayout(Main,0)
 	        MenuItem(AP_MENU_ID_TOOLS_HISTORY_SHOW)
 	        MenuItem(AP_MENU_ID_TOOLS_REVISIONS_COMPARE_DOCUMENTS)
 		    MenuItem(AP_MENU_ID_TOOLS_REVISIONS_AUTO)
+		    MenuItem(AP_MENU_ID_TOOLS_HISTORY_PURGE)
 		EndSubMenu()
 	
 		BeginSubMenu(AP_MENU_ID_TOOLS_REVISIONS)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_MARK)
+			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_NEW_REVISION)
 		    Separator()
 	        MenuItem(AP_MENU_ID_TOOLS_REVISIONS_SHOW)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_SHOW_AFTER)
@@ -316,6 +322,7 @@ BeginLayout(Main,0)
 	        Separator()
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_ACCEPT_REVISION)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_REJECT_REVISION)
+			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_PURGE)
 #ifdef DEBUG
 	        Separator()
 		    MenuItem(AP_MENU_ID_TOOLS_REVISIONS_MERGE_DOCUMENTS)
@@ -343,11 +350,10 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_COLUMNS_BEFORE)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_COLUMNS_AFTER)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_ROWS_BEFORE)
-			MenuItem(AP_MENU_ID_TABLE_INSERT_ROWS_AFTER)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_SUMCOLS)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_SUMROWS)
 #if 0
-// Not for 2.2
+// Not for 2.4
 			MenuItem(AP_MENU_ID_TABLE_INSERT_CELLS)
 #endif
 		EndSubMenu()
@@ -357,7 +363,7 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_TABLE_DELETE_COLUMNS)
 			MenuItem(AP_MENU_ID_TABLE_DELETE_ROWS)
 #if 0
-// Not for 2.2
+// Not for 2.4
 			MenuItem(AP_MENU_ID_TABLE_DELETE_CELLS)
 #endif
 		EndSubMenu()
@@ -373,11 +379,14 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_TABLE_MERGE_CELLS)
 		MenuItem(AP_MENU_ID_TABLE_SPLIT_CELLS)
 #if 0
-// Not for 2.2
+// Not for 2.4
 		MenuItem(AP_MENU_ID_TABLE_SPLIT_TABLE)
 #endif
 		MenuItem(AP_MENU_ID_TABLE_FORMAT)
-		MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE)
+                BeginSubMenu(AP_MENU_ID_TABLE_TEXTTOTABLE)
+		   MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_ALL)
+		   MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_NOSPACES)
+                EndSubMenu()
 #if DEBUG
 	    BeginSubMenu(AP_MENU_ID_TABLE_SORT)
 	       MenuItem(AP_MENU_ID_TABLE_SORTROWSASCEND)
@@ -386,11 +395,17 @@ BeginLayout(Main,0)
 	       MenuItem(AP_MENU_ID_TABLE_SORTCOLSDESCEND)
 	    EndSubMenu()
 #endif
+	    BeginSubMenu(AP_MENU_ID_TABLE_TABLETOTEXT)
+	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTCOMMAS)
+	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTTABS)
+	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTCOMMASTABS)
+	    EndSubMenu()
 		MenuItem(AP_MENU_ID_TABLE_AUTOFIT)
-
-#if 0
-// Not for 2.2
-		MenuItem(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT)
+#if DEBUG
+	    BeginSubMenu(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT)
+                MenuItem(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT_THIS)
+                MenuItem(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT_REMOVE)
+	    EndSubMenu()
 #endif
 	EndSubMenu()
 
