@@ -48,7 +48,7 @@ dcl(MSG_IE_CouldNotWrite,		"Could not write to file %s")
 dcl(MSG_SpellSelectionDone,		"AbiWord finished checking the selection.")
 dcl(MSG_SpellDone,				"The spelling check is complete.")
 dcl(MSG_PrintStatus,			"Printing page %d of %d")
-dcl(MSG_RevertFile, "Revert file to last unsaved state?")
+dcl(MSG_RevertFile, "Revert file to last saved state?")
 dcl(MSG_PrintingDoc, "Printing Document..")
 dcl(MSG_NoBreakInsideTable, "Can not insert a Break inside a table")
 dcl(MSG_NoBreakInsideFrame, "Can not insert a Break inside a text box")
@@ -184,13 +184,23 @@ dcl(DLG_FormatTOC_Level2,                 "Level 2")
 dcl(DLG_FormatTOC_Level3,                 "Level 3")
 dcl(DLG_FormatTOC_Level4,                 "Level 4")
 
+/* Latex dialog */
+dcl(DLG_Latex_LatexTitle,	"Latex Equation")
 
+/*Equation Menus */
+dcl(MENU_LABEL_INSERT_EQUATION,"Equation")
+dcl(MENU_LABEL_TOOLTIP_INSERT_EQUATION,"Insert Equation")
+dcl(MENU_LABEL_INSERT_EQUATION_FILE,"From File")
+dcl(MENU_LABEL_TOOLTIP_INSERT_EQUATION_FILE,"Insert MathML from a file")
+dcl(MENU_LABEL_INSERT_EQUATION_LATEX,"From Latex")
+dcl(MENU_LABEL_TOOLTIP_INSERT_EQUATION_LATEX,"Insert Equation from a Latex expression")
 
 /* Word Count dialog */
 dcl(DLG_WordCount_WordCountTitle,	"Word Count")
 dcl(DLG_WordCount_Statistics,		"Statistics:")
 dcl(DLG_WordCount_Pages,			"Pages:")
 dcl(DLG_WordCount_Words,			"Words:")
+dcl(DLG_WordCount_Words_No_Notes,			"Words (no footnotes/endnotes):")
 dcl(DLG_WordCount_Characters_No,	"Characters (no spaces):")
 dcl(DLG_WordCount_Characters_Sp,	"Characters (with spaces):")
 dcl(DLG_WordCount_Paragraphs,		"Paragraphs:")
@@ -354,6 +364,10 @@ dcl(DLG_Options_SpellCheckingTitle,		"Spell Checking")
 dcl(DLG_Options_Label_SpellCheckAsType, "Check s&pelling as you type")
 dcl(DLG_Options_Label_SpellHideErrors,	"Hide &spelling errors in the document")
 dcl(DLG_Options_Label_SpellHighlightMisspelledWords, "&Highlight misspelled words") // Opposite of HideErrors
+
+     // Grammar Check
+dcl(DLG_Options_Label_Grammar, "Automatic Grammar Checking")
+dcl(DLG_Options_Label_GrammarCheck, "Check &grammar as you type")
 										      // should be merged one day
 dcl(DLG_Options_Label_SpellSuggest, 	"A&lways suggest corrections")
 dcl(DLG_Options_Label_SpellMainOnly,	"Suggest from &main dictionary only")
@@ -440,11 +454,16 @@ dcl(DLG_Options_Label_Schemes, "Preference Schemes")
 dcl(DLG_Options_Label_Language, "Language")
 
 dcl(DLG_Tab_TabTitle,					"Tabs")
+dcl(DLG_Tab_Label_Existing,				"User-defined Tabs")
+dcl(DLG_Tab_Label_New,					"Set new Tab")
 dcl(DLG_Tab_Label_TabPosition,			"Tab stop position:")
 dcl(DLG_Tab_Label_TabToClear,			"Tab stops to be cleared:")
 dcl(DLG_Tab_Label_DefaultTS,			"Default tab stops:")
 
+dcl(DLG_Tab_Label_Position,				"Position")
+
 dcl(DLG_Tab_Label_Alignment,			"Alignment")
+dcl(DLG_Tab_Radio_NoAlign, 				"None")
 dcl(DLG_Tab_Radio_Left, 				"Left")
 dcl(DLG_Tab_Radio_Center,				"Center")
 dcl(DLG_Tab_Radio_Right,				"Right")
@@ -853,6 +872,7 @@ dcl(MENU_LABEL_FILE,"&File")
 dcl(MENU_LABEL_FILE_NEW,"&New")
 dcl(MENU_LABEL_FILE_NEW_USING_TEMPLATE,"&New using Template")
 dcl(MENU_LABEL_FILE_OPEN,"&Open")
+dcl(MENU_LABEL_FILE_IMPORTSTYLES,"&Import Styles")
 dcl(MENU_LABEL_FILE_CLOSE,"&Close")
 dcl(MENU_LABEL_FILE_SAVE,"&Save")
 dcl(MENU_LABEL_FILE_SAVEAS,"Save &As")
@@ -890,6 +910,9 @@ dcl(MENU_LABEL_EDIT_FIND,"&Find")
 dcl(MENU_LABEL_EDIT_REPLACE,"R&eplace")
 dcl(MENU_LABEL_EDIT_GOTO,"&Go To")
 dcl(MENU_LABEL_EDIT_DELETEFRAME,"Delete Text Box")
+dcl(MENU_LABEL_EDIT_DELETEIMAGE,"Delete Image")
+dcl(MENU_LABEL_EDIT_CUTIMAGE,"Cut Image")
+dcl(MENU_LABEL_EDIT_COPYIMAGE,"Copy Image")
 dcl(MENU_LABEL_EDIT_CUT_FRAME,"Cut Text Box")
 dcl(MENU_LABEL_EDIT_COPY_FRAME,"Copy Text Box")
 dcl(MENU_LABEL_EDIT_SELECT_FRAME,"Select Text Box")
@@ -943,6 +966,7 @@ dcl(MENU_LABEL_INSERT_BOOKMARK,"Boo&kmark")
 dcl(MENU_LABEL_INSERT_HYPERLINK,"&Hyperlink")
 dcl(MENU_LABEL_INSERT_GOTO_HYPERLINK,"&Jump to hyperlink")
 dcl(MENU_LABEL_INSERT_EDIT_HYPERLINK,"&Edit hyperlink")
+dcl(MENU_LABEL_EDIT_COPY_HYPERLINK_LOCATION,"C&opy hyperlink location")
 dcl(MENU_LABEL_INSERT_DELETE_HYPERLINK,"&Delete hyperlink")
 dcl(MENU_LABEL_INSERT_DIRECTIONMARKER, "&Direction Marker")
 dcl(MENU_LABEL_INSERT_DIRECTIONMARKER_LRM, "&LRM")
@@ -965,8 +989,7 @@ dcl(MENU_LABEL_FMT_BACKGROUND_PAGE_COLOR,"Page C&olor")
 dcl(MENU_LABEL_FMT_BACKGROUND_PAGE_IMAGE,"Page Image")
 dcl(MENU_LABEL_FMT_STYLIST, "Sty&list")
 dcl(MENU_LABEL_FMT_STYLE,"St&yle")
-dcl(MENU_LABEL_FMT_STYLE_DEFINE, "&Create and Modify")
-dcl(MENU_LABEL_FMT_IMPORTSTYLES,"&Import Styles")
+dcl(MENU_LABEL_FMT_STYLE_DEFINE, "&Create and Modify Styles")
 dcl(MENU_LABEL_FMT_TABS,"&Tabs")
 dcl(MENU_LABEL_FMT_BOLD,"&Bold")
 dcl(MENU_LABEL_FMT_ITALIC,"&Italic")
@@ -979,6 +1002,7 @@ dcl(MENU_LABEL_FMT_SUPERSCRIPT,"Supe&rscript")
 dcl(MENU_LABEL_FMT_SUBSCRIPT,"&Subscript")
 dcl(MENU_LABEL_FMT_FRAME,"Text Box")
 dcl(MENU_LABEL_FMT_IMAGE, "&Image")
+dcl(MENU_LABEL_FMT_POSIMAGE, "&Image")
 dcl(MENU_LABEL_FMT_DIRECTION, "D&irectional")
 dcl(MENU_LABEL_FMT_DIRECTION_DD_RTL, "RTL &Paragraph")
 dcl(MENU_LABEL_FMT_DIRECTION_DO_LTR, "&Left-to-right text")
@@ -996,6 +1020,7 @@ dcl(MENU_LABEL_TOOLS_SCRIPTS,"S&cripts")
 dcl(MENU_LABEL_TOOLS_MAILMERGE,"&Mail Merge")
 dcl(MENU_LABEL_TOOLS_HISTORY,"&Document History")
 dcl(MENU_LABEL_TOOLS_HISTORY_SHOW,"&Show History")
+dcl(MENU_LABEL_TOOLS_HISTORY_PURGE,"&Purge History")
 dcl(MENU_LABEL_TOOLS_REVISIONS_AUTO,"&Maintain Full History")
 dcl(MENU_LABEL_TOOLS_REVISIONS,"&Revisions")
 dcl(MENU_LABEL_TOOLS_REVISIONS_MARK,"&Mark revisions while typing")
@@ -1010,6 +1035,8 @@ dcl(MENU_LABEL_TOOLS_REVISIONS_FIND_NEXT,"Find &next revision")
 dcl(MENU_LABEL_TOOLS_REVISIONS_FIND_PREV,"Find &previous revision")
 dcl(MENU_LABEL_TOOLS_REVISIONS_COMPARE_DOCUMENTS,"&Compare documents")
 dcl(MENU_LABEL_TOOLS_REVISIONS_MERGE_DOCUMENTS,"M&erge documents")
+dcl(MENU_LABEL_TOOLS_REVISIONS_NEW_REVISION,"Start ne&w revision")
+dcl(MENU_LABEL_TOOLS_REVISIONS_PURGE,"P&urge revisions")
 dcl(MENU_LABEL_TABLE, "T&able")
 dcl(MENU_LABEL_TABLE_INSERT, "&Insert")
 dcl(MENU_LABEL_TABLE_INSERT_TABLE, "&Table")
@@ -1042,7 +1069,15 @@ dcl(MENU_LABEL_TABLE_SPLIT_TABLE, "Spli&t Table")
 dcl(MENU_LABEL_TABLE_FORMAT, "&Format Table")
 dcl(MENU_LABEL_TABLE_AUTOFIT, "&Autofit Table")
 dcl(MENU_LABEL_TABLE_TEXTTOTABLE, "Convert Text to Table")
+dcl(MENU_LABEL_TABLE_TEXTTOTABLE_ALL, "Split text at spaces, commas, or tabs")
+dcl(MENU_LABEL_TABLE_TEXTTOTABLE_NOSPACES, "Split text at commas or tabs")
+dcl(MENU_LABEL_TABLE_TABLETOTEXT, "Convert Table to Text")
+dcl(MENU_LABEL_TABLE_TABLETOTEXTCOMMAS, "Separate with commas")
+dcl(MENU_LABEL_TABLE_TABLETOTEXTTABS, "Separate with tabs")
+dcl(MENU_LABEL_TABLE_TABLETOTEXTCOMMASTABS, "Separate with commas and tabs")
 dcl(MENU_LABEL_TABLE_HEADING_ROWS_REPEAT, "Repeat Row as Heading")
+dcl(MENU_LABEL_TABLE_HEADING_ROWS_REPEAT_THIS, "Set this Row as Heading")
+dcl(MENU_LABEL_TABLE_HEADING_ROWS_REPEAT_REMOVE, "Remove Row as Heading")
 dcl(MENU_LABEL_TABLE_SORT, "So&rt Table")
 dcl(MENU_LABEL_TABLE_SORTROWSASCEND, "Sort Rows in Ascending Order")
 dcl(MENU_LABEL_TABLE_SORTROWSDESCEND, "Sort Rows in Descending Order")
@@ -1148,6 +1183,7 @@ dcl(MENU_STATUSLINE_FILE," ")
 dcl(MENU_STATUSLINE_FILE_NEW,"Create a new document"	)
 dcl(MENU_STATUSLINE_FILE_NEW_USING_TEMPLATE,"Create a new document using a template"	)
 dcl(MENU_STATUSLINE_FILE_OPEN,"Open an existing document")
+dcl(MENU_STATUSLINE_FILE_IMPORTSTYLES,"Import style definitions from a document")
 dcl(MENU_STATUSLINE_FILE_CLOSE,"Close the document")
 dcl(MENU_STATUSLINE_FILE_SAVE,"Save the document")
 dcl(MENU_STATUSLINE_FILE_SAVEAS,"Save the document under a different name")
@@ -1189,6 +1225,9 @@ dcl(MENU_STATUSLINE_EDIT_EDITFOOTER,"Edit the Footer on the current page")
 dcl(MENU_STATUSLINE_EDIT_REMOVEHEADER,"Remove the Header on this page from the Document")
 dcl(MENU_STATUSLINE_EDIT_REMOVEFOOTER,"Remove the Footer on this page from the Document")
 dcl(MENU_STATUSLINE_EDIT_DELETEFRAME,"Remove the Text Box from the Document")
+dcl(MENU_STATUSLINE_EDIT_DELETEIMAGE,"Remove the Image from the Document")
+dcl(MENU_STATUSLINE_EDIT_CUTIMAGE,"Remove the Image and save a copy on the clipboard")
+dcl(MENU_STATUSLINE_EDIT_COPYIMAGE,"Save a copy of the image on the clipboard")
 dcl(MENU_STATUSLINE_EDIT_CUT_FRAME,"Copy the Text Box to the clipboard then remove it")
 dcl(MENU_STATUSLINE_EDIT_COPY_FRAME,"Copy the Text Box to the clipboard")
 dcl(MENU_STATUSLINE_EDIT_SELECT_FRAME,"Select the Text Box")
@@ -1238,6 +1277,7 @@ dcl(MENU_STATUSLINE_INSERT_BOOKMARK,"Insert bookmark")
 dcl(MENU_STATUSLINE_INSERT_HYPERLINK,"Insert hyperlink")
 dcl(MENU_STATUSLINE_INSERT_GOTO_HYPERLINK,"Jump to hyperlink")
 dcl(MENU_STATUSLINE_INSERT_EDIT_HYPERLINK,"Edit hyperlink")
+dcl(MENU_STATUSLINE_EDIT_COPY_HYPERLINK_LOCATION,"Copy hyperlink location")
 dcl(MENU_STATUSLINE_INSERT_DELETE_HYPERLINK,"Delete hyperlink")
 dcl(MENU_STATUSLINE_INSERT_DIRECTIONMARKER, "Insert Unicode direction marker into the document")
 dcl(MENU_STATUSLINE_INSERT_DIRECTIONMARKER_LRM, "Insert left-to-right direction marker (LRM)")
@@ -1262,7 +1302,6 @@ dcl(MENU_STATUSLINE_FMT_BACKGROUND_PAGE_IMAGE,"Set an image as a background for
 dcl(MENU_STATUSLINE_FMT_STYLIST, "Format your document using styles")
 dcl(MENU_STATUSLINE_FMT_STYLE,"Define or apply style for the selection")
 dcl(MENU_STATUSLINE_FMT_STYLE_DEFINE, "Define or apply style for the selection")
-dcl(MENU_STATUSLINE_FMT_IMPORTSTYLES,"Import style definitions from a document")
 dcl(MENU_STATUSLINE_FMT_TABS,"Set tab stops")
 dcl(MENU_STATUSLINE_FMT_BOLD,"Make the selection bold (toggle)")
 dcl(MENU_STATUSLINE_FMT_ITALIC,"Make the selection italic (toggle)")
@@ -1291,6 +1330,7 @@ dcl(MENU_STATUSLINE_TOOLS_MAILMERGE, "Mail Merge")
 dcl(MENU_STATUSLINE_TOOLS_SCRIPTS,"Execute helper scripts")
 dcl(MENU_STATUSLINE_TOOLS_HISTORY,"See history of the current document")
 dcl(MENU_STATUSLINE_TOOLS_HISTORY_SHOW,"View document history")
+dcl(MENU_STATUSLINE_TOOLS_HISTORY_PURGE,"Remove full document history from the document")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS,"Manage changes in document")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_SET_VIEW_LEVEL,"Choose which revision you wish to view")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_MARK,"Mark changes as you type")
@@ -1304,6 +1344,8 @@ dcl(MENU_STATUSLINE_TOOLS_REVISIONS_FIND_NEXT,"Find next visible revision in the
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_FIND_PREV,"Find previous visible revision in the document")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_COMPARE_DOCUMENTS,"Compare active document to another document")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_MERGE_DOCUMENTS,"Merge another document into the active document using revision marks")
+dcl(MENU_STATUSLINE_TOOLS_REVISIONS_NEW_REVISION,"Start revision distinct from the current one")
+dcl(MENU_STATUSLINE_TOOLS_REVISIONS_PURGE,"Remove all revision information from the document")
 dcl(MENU_STATUSLINE_TOOLS_REVISIONS_AUTO,"Retain all document changes")
 dcl(MENU_STATUSLINE_TABLE, "Table")
 dcl(MENU_STATUSLINE_TABLE_INSERT, "Insert")
@@ -1334,9 +1376,17 @@ dcl(MENU_STATUSLINE_TABLE_SPLIT_CELLS, "Split Cells")
 dcl(MENU_STATUSLINE_TABLE_SPLIT_TABLE, "Split Table")
 dcl(MENU_STATUSLINE_TABLE_FORMAT, "Format Table")
 dcl(MENU_STATUSLINE_TABLE_AUTOFIT, "Autofit Table")
-dcl(MENU_STATUSLINE_TABLE_HEADING_ROWS_REPEAT, "Repeat Row as Heading")
+dcl(MENU_STATUSLINE_TABLE_HEADING_ROWS_REPEAT, "Repeat Row as Heading on each new page")
+dcl(MENU_STATUSLINE_TABLE_HEADING_ROWS_REPEAT_THIS, "Set this Row as the Heading on each page")
+dcl(MENU_STATUSLINE_TABLE_HEADING_ROWS_REPEAT_REMOVE, "Remove Row as the page Heading")
 dcl(MENU_STATUSLINE_TABLE_SORT, "Sort Table")
 dcl(MENU_STATUSLINE_TABLE_TEXTTOTABLE, "Convert Selected Text to a Table")
+dcl(MENU_STATUSLINE_TABLE_TEXTTOTABLE_ALL, "Split on all word delimiters including spaces")
+dcl(MENU_STATUSLINE_TABLE_TEXTTOTABLE_NOSPACES, "Split on all delimiters except spaces")
+dcl(MENU_STATUSLINE_TABLE_TABLETOTEXT, "Convert from a Table to Text")
+dcl(MENU_STATUSLINE_TABLE_TABLETOTEXTCOMMAS, "Separate table items with commas")
+dcl(MENU_STATUSLINE_TABLE_TABLETOTEXTTABS, "Separate table items with tabs")
+dcl(MENU_STATUSLINE_TABLE_TABLETOTEXTCOMMASTABS, "Separate table items with commas and tabs")
 dcl(MENU_STATUSLINE_TABLE_SORTROWSASCEND, "Sort Rows in Ascending Order based on selected Column")
 dcl(MENU_STATUSLINE_TABLE_SORTROWSDESCEND, "Sort Rows in Descending Order based on Selected Column")
 dcl(MENU_STATUSLINE_TABLE_SORTCOLSASCEND, "Sort Columns in Ascending Order based on Selected Row")
@@ -1433,6 +1483,7 @@ dcl(MENU_STATUSLINE_AUTOTEXT_EMAIL_4," ")
 dcl(MENU_STATUSLINE_AUTOTEXT_EMAIL_5," ")
 dcl(MENU_STATUSLINE_AUTOTEXT_EMAIL_6," ")
 dcl(MENU_STATUSLINE_FMT_IMAGE, "Resize this image")
+dcl(MENU_STATUSLINE_FMT_POSIMAGE, "Format this image")
 
 /* Toolbar labels */
 dcl(TB_Extra,"Extra")
