@@ -34,9 +34,7 @@ BeginLayout(FileEditOps, AP_STRING_ID_TB_Standard, AP_PREF_KEY_StandardBarVisibl
 
 	Spacer()
 	ToolbarItem(AP_TOOLBAR_ID_FILE_PRINT)
-#if defined(HAVE_GNOME)
 	ToolbarItem(AP_TOOLBAR_ID_FILE_PRINT_PREVIEW)
-#endif
 	ToolbarItem(AP_TOOLBAR_ID_SPELLCHECK)
 
 	Spacer()
