@@ -110,6 +110,13 @@ IE_Exp_AbiWord_1::IE_Exp_AbiWord_1(PD_Document * pDocument, bool isTemplate)
 	: IE_Exp(pDocument), m_bIsTemplate(isTemplate), m_pListener(0)
 {
 	m_error = 0;
+
+	UT_ASSERT_HARMLESS( pDocument );
+
+	// depending on how this document came to be, it might contain fragments without xid;
+	// this is legitimate and desirable while the doc is loade, but once we are saving to
+	// disk, we want to assing xids to any frags that should have them and do not
+	pDocument->fixMissingXIDs();
 }
 
 IE_Exp_AbiWord_1::~IE_Exp_AbiWord_1()
@@ -169,6 +176,7 @@ protected:
 	void				_openSpan(PT_AttrPropIndex apiSpan);
 	void				_openTag(const char * szPrefix, const char * szSuffix,
 								 bool bNewLineAfter, PT_AttrPropIndex api,
+								 UT_uint32 iXID,
 								 bool bIgnoreProperties = false);
 	void				_outputData(const UT_UCSChar * p, UT_uint32 length);
 	void				_outputXMLChar(const XML_Char * data, UT_uint32 length);
@@ -193,6 +201,8 @@ protected:
 	PT_AttrPropIndex	m_apiLastSpan;
     fd_Field *          m_pCurrentField;
 	bool                m_bOpenChar;
+	UT_GenericVector<UT_UTF8String *> m_vecSnapNames;
+
 
 private:
 	UT_Set				m_pUsedImages;
@@ -289,74 +299,104 @@ void s_AbiWord_1_Listener::_openSpan(PT_AttrPropIndex apiSpan)
 	if (!apiSpan)				// don't write tag for empty A/P
 		return;
 
-	_openTag("c","",false,apiSpan);
+	_openTag("c","",false,apiSpan,0);
 	m_bInSpan = true;
 	m_apiLastSpan = apiSpan;
 	return;
 }
 
 void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix,
-								   bool bNewLineAfter, PT_AttrPropIndex api,
-								   bool bIgnoreProperties)
+									bool bNewLineAfter, PT_AttrPropIndex api,
+									UT_uint32 iXID,
+									bool bIgnoreProperties)
 {
 #ifdef ENABLE_RESOURCE_MANAGER
 	UT_ASSERT_HARMLESS (!m_bOpenChar);
 
 	UT_UTF8String tag("<");
+	UT_UTF8String url;
 	tag += szPrefix;
 
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp (api, &pAP);
 	if (bHaveProp && pAP)
+	{
+		const XML_Char * szName = 0;
+		const XML_Char * szValue = 0;
+
+		UT_uint32 k = 0;
+		while (pAP->getNthAttribute (k++, szName, szValue))
 		{
-			const XML_Char * szName = 0;
-			const XML_Char * szValue = 0;
+			tag += " ";
+			tag += szName;
+			tag += "=\"";
 
-			UT_uint32 k = 0;
-			while (pAP->getNthAttribute (k++, szName, szValue))
+			if ((strcmp (szName, "href") == 0) || (strcmp (szName, "xlink:href") == 0))
+			{
+				if(*szValue == '/')
 				{
-					tag += " ";
-					tag += szName;
-					tag += "=\"";
-
-					if ((*szValue == '/') && ((strcmp (szName, "href") == 0) || (strcmp (szName, "xlink:href") == 0)))
-						{
-							XAP_ResourceManager & RM = m_pDocument->resourceManager ();
-							XAP_ExternalResource * re = dynamic_cast<XAP_ExternalResource *>(RM.resource (szValue, false));
-							if (re) tag += re->URL ();
-						}
-					else _outputXMLChar (szValue, strlen (szValue));
-
-					tag += "\"";
+					XAP_ResourceManager & RM = m_pDocument->resourceManager ();
+					XAP_ExternalResource * re = dynamic_cast<XAP_ExternalResource *>(RM.resource (szValue, false));
+					if (re)
+					{
+						url = re->URL ();
+						url.escapeURL();
+						tag += url;
+					}
 				}
-			if (!bIgnoreProperties)
+				else
 				{
-					k = 0;
-					while (pAP->getNthProperty (k++, szName, szValue))
-						{
-							if (k == 1)
-								{
-									tag += " ";
-									tag += PT_PROPS_ATTRIBUTE_NAME;
-									tag += "=\"";
-								}
-							else tag += "; ";
-
-							tag += szName;
-							tag += ":";
-
-							_outputXMLChar (szValue, strlen (szValue));
-						}
-					if (k > 1) tag += "\"";
+					url = szValue;
+					url.escapeURL();
+					tag += url;
 				}
+			}
+			else
+			{
+				tag += szValue;
+			}
+			
+
+			tag += "\"";
 		}
 
+		if(iXID != 0)
+		{
+			// insert xid attribute
+			tag += " ";
+			tag += PT_XID_ATTRIBUTE_NAME;
+			tag += "=\"";
+			UT_UTF8String s;
+			UT_UTF8String_sprintf(s, "%d\"", iXID);
+			tag += s;
+		}
+		
+		if (!bIgnoreProperties)
+		{
+			k = 0;
+			while (pAP->getNthProperty (k++, szName, szValue))
+			{
+				if (k == 1)
+				{
+					tag += " ";
+					tag += PT_PROPS_ATTRIBUTE_NAME;
+					tag += "=\"";
+				}
+				else tag += "; ";
+
+				tag += szName;
+				tag += ":";
+
+				_outputXMLChar (szValue, strlen (szValue));
+			}
+			if (k > 1) tag += "\"";
+		}
+	}
 	if (szSuffix)
 		if (*szSuffix == '/')
 			tag += "/";
 	tag += ">";
 	if (bNewLineAfter) tag += "\n";
-
 	m_pie->write (tag.utf8_str (), tag.byteLength());
 
 	if (strcmp (szPrefix, "c") == 0) m_bOpenChar = true;
@@ -374,6 +414,7 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 	m_pie->write(szPrefix);
 	if (bHaveProp && pAP)
 	{
+		UT_UTF8String url;
 		const XML_Char * szName;
 		const XML_Char * szValue;
 		UT_uint32 k = 0;
@@ -388,9 +429,28 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 			m_pie->write(" ");
 			m_pie->write(static_cast<const char*>(szName));
 			m_pie->write("=\"");
-			_outputXMLChar(szValue, strlen(szValue));
+
+			if ((strcmp (szName, "href") == 0) || (strcmp (szName, "xlink:href") == 0))
+			{
+				url = szValue;
+				url.escapeURL();
+				_outputXMLChar(url.utf8_str(), url.byteLength());
+			}
+			else
+				_outputXMLChar(szValue, strlen(szValue));
+			
 			m_pie->write("\"");
 		}
+		if(iXID != 0)
+		{
+			// insert xid attribute
+			m_pie->write(" ");
+			m_pie->write(PT_XID_ATTRIBUTE_NAME);
+			m_pie->write("=\"");
+			UT_String s;
+			UT_String_sprintf(s, "%d\"", iXID);
+			m_pie->write(s.c_str());
+		}
 		if (!bIgnoreProperties && pAP->getNthProperty(0,szName,szValue))
 		{
 			m_pie->write(" ");
@@ -416,13 +476,68 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 			m_pie->write("\"");
 		}
 	}
-
-	if (szSuffix && *szSuffix)
-		m_pie->write(szSuffix);
-	m_pie->write(">");
-	if (bNewLineAfter)
-		m_pie->write("\n");
-
+	if(UT_strcmp(szPrefix,"math") == 0)
+	{
+		UT_UTF8String tag;
+		const char * szPropVal = NULL;
+		pAP->getAttribute("dataid",szPropVal);
+		if(szPropVal != NULL)
+		{
+			tag = ">";
+			if (bNewLineAfter) tag += "\n";
+			m_pie->write (tag.utf8_str (), tag.byteLength());
+			tag.clear();
+			tag = "<image dataid=";
+			tag += "\"";
+			tag += "snapshot-png-";
+			tag += szPropVal;
+			tag += "\"";
+			tag += " ";
+			tag += PT_PROPS_ATTRIBUTE_NAME;
+			tag += "=\"";
+			bool bFound = pAP->getProperty("height", szPropVal);
+			UT_UTF8String sVal;
+			if(bFound)
+			{
+				double dInch = static_cast<double>(atoi(szPropVal))/UT_LAYOUT_RESOLUTION;
+				UT_UTF8String_sprintf(sVal,"%fin",dInch);
+				tag += "height:";
+				tag += sVal;
+				tag += "; ";
+			}
+			bFound = pAP->getProperty("width", szPropVal);
+			if(bFound)
+			{
+				double dInch = static_cast<double>(atoi(szPropVal))/UT_LAYOUT_RESOLUTION;
+				UT_UTF8String_sprintf(sVal,"%fin",dInch);
+				tag += "width:";
+				tag += sVal;
+				tag += "\"";
+			}
+			tag += "/";
+			tag += ">";
+			tag += "</math";
+			tag += ">";
+		}
+		else
+		{
+			if (szSuffix)
+				if (*szSuffix == '/')
+					tag += "/";
+			tag += ">";
+			if (bNewLineAfter) tag += "\n";
+		}
+		m_pie->write (tag.utf8_str (), tag.byteLength());
+	}
+	else
+	{
+		if (szSuffix)
+			if (*szSuffix == '/')
+				m_pie->write ("/");
+		m_pie->write (">");
+		if (bNewLineAfter)
+			m_pie->write ("\n");
+	}
 	//	m_bInTag = true;
 #endif /* ENABLE_RESOURCE_MANAGER */
 }
@@ -552,14 +667,20 @@ s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
 
 	************************************************************************************/
 
-	const XML_Char *attr[3];
+	// we want to update the XID counter and the template status
+	UT_String s;
+	UT_String_sprintf(s, "%d", pDocument->getTopXID());
+	
+	const XML_Char *attr[5];
 	attr[0] = "template";
 	attr[1] = m_bIsTemplate ? "true" : "false";
-	attr[2] = NULL;
+	attr[2] = "xid-max";
+	attr[3] = s.c_str();
+	attr[4] = NULL;
 	
 	pDocument->setAttributes(attr);
 
-	_openTag("abiword", NULL, true, pDocument->getAttrPropIndex(),false);
+	_openTag("abiword", NULL, true, pDocument->getAttrPropIndex(),false,0);
 
 	// NOTE we output the following preamble in XML comments.
 	// NOTE this information is for human viewing only.
@@ -592,6 +713,7 @@ s_AbiWord_1_Listener::~s_AbiWord_1_Listener()
 	_handleDataItems();
 
 	m_pie->write("</abiword>\n");
+	UT_VECTOR_PURGEALL(UT_UTF8String * ,m_vecSnapNames);
 }
 
 
@@ -647,7 +769,7 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				if (image_name)
 					m_pUsedImages.insert(image_name);
 #endif
-				_openTag("image","/",false,api);
+				_openTag("image","/",false,api,pcr->getXID());
 
 				return true;
 				}
@@ -655,16 +777,58 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
                 {
                     _closeSpan();
                     _closeField();
-                    _openTag("field","",false,api);
+                    _openTag("field","",false,api,pcr->getXID());
                     m_pCurrentField = pcro->getField();
                     UT_ASSERT_HARMLESS(m_pCurrentField);
                     return true;
                 }
-   			case PTO_Bookmark:
+ 			case PTO_Math:
+                {
+                    _closeSpan();
+                    _closeField();
+                    _openTag("math","/",false,api,pcr->getXID());
+					const XML_Char* image_name = getObjectKey(api, static_cast<const XML_Char*>("dataid"));
+					if (image_name)
+					{
+						UT_DEBUGMSG(("resource name #%s# recorded \n",image_name));
+						m_pUsedImages.insert(image_name);
+						UT_UTF8String * sPNGname = new UT_UTF8String("snapshot-png-");
+						m_vecSnapNames.addItem(sPNGname);
+						*sPNGname += image_name;
+						UT_DEBUGMSG(("resource name #%s# recorded \n",sPNGname->utf8_str()));
+						m_pUsedImages.insert(sPNGname->utf8_str());
+					}
+					const XML_Char* latex_name = getObjectKey(api, static_cast<const XML_Char*>("latexid"));
+					if(latex_name)
+					{
+						UT_DEBUGMSG(("resource name #%s# recorded \n",latex_name));
+						m_pUsedImages.insert(latex_name);
+					}
+                    return true;
+                }
+ 			case PTO_Embed:
+                {
+                    _closeSpan();
+                    _closeField();
+                    _openTag("embed","/",false,api,pcr->getXID());
+					const XML_Char* image_name = getObjectKey(api, static_cast<const XML_Char*>("dataid"));
+					if (image_name)
+					{
+						UT_DEBUGMSG(("resource name #%s# recorded \n",image_name));
+						m_pUsedImages.insert(image_name);
+						UT_UTF8String * sPNGname = new UT_UTF8String("snapshot-png-");
+						m_vecSnapNames.addItem(sPNGname);
+						*sPNGname += image_name;
+						UT_DEBUGMSG(("resource name #%s# recorded \n",sPNGname->utf8_str()));
+						m_pUsedImages.insert(sPNGname->utf8_str());
+					}
+                    return true;
+                }
+  			case PTO_Bookmark:
    				{
    					_closeSpan();
    					_closeField();
-   					_openTag("bookmark", "/",false, api,true);
+   					_openTag("bookmark", "/",false, api,pcr->getXID(),true);
    					return true;
    				}
 
@@ -689,7 +853,7 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 					if(bFound)
 					{
 						//this is the start of the hyperlink
-   						_openTag("a", "",false, api,true);
+   						_openTag("a", "",false, api,pcr->getXID(),true);
    						m_bInHyperlink = true;
    					}
    					else
@@ -715,7 +879,7 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 		// attributes and should not be saved
 		if(m_bOpenChar)
 			_closeTag();
-		_openTag("c","",false,pcr->getIndexAP());
+		_openTag("c","",false,pcr->getIndexAP(),0);
 		_closeTag();
 #endif
 		return true;
@@ -750,7 +914,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeHyperlink();
 			_closeBlock();
 			_closeSection();
-			_openTag("section","",true,pcr->getIndexAP());
+			_openTag("section","",true,pcr->getIndexAP(),pcr->getXID());
 			m_bInSection = true;
 			return true;
 		}
@@ -760,7 +924,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
             _closeHyperlink();
 			_closeBlock();
-			_openTag("table","",true,pcr->getIndexAP());
+			_openTag("table","",true,pcr->getIndexAP(),pcr->getXID());
 			m_iInTable++;
 			return true;
 		}
@@ -770,7 +934,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
             _closeHyperlink();
 			_closeBlock();
-			_openTag("cell","",true,pcr->getIndexAP());
+			_openTag("cell","",true,pcr->getIndexAP(),pcr->getXID());
 			m_iInCell++;
 			return true;
 		}
@@ -780,7 +944,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
             _closeHyperlink();
 			m_bInBlock = false;
-			_openTag("foot","",true,pcr->getIndexAP());
+			_openTag("foot","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
 		}
 	case PTX_SectionEndnote:
@@ -789,7 +953,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
             _closeHyperlink();
 			m_bInBlock = false;
-			_openTag("endnote","",true,pcr->getIndexAP());
+			_openTag("endnote","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
 		}
 	case PTX_SectionTOC:
@@ -799,7 +963,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeHyperlink();
 			_closeBlock();
 			m_bInBlock = false;
-			_openTag("toc","",true,pcr->getIndexAP());
+			_openTag("toc","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
 		}
 	case PTX_SectionMarginnote:
@@ -808,7 +972,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
             _closeHyperlink();
 			_closeBlock();
-			_openTag("margin","",true,pcr->getIndexAP());
+			_openTag("margin","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
 		}
 	case PTX_SectionFrame:
@@ -817,7 +981,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
             _closeHyperlink();
 			_closeBlock();
-			_openTag("frame","",true,pcr->getIndexAP());
+			_openTag("frame","",true,pcr->getIndexAP(),pcr->getXID());
 			return true;
 		}
 	case PTX_EndTable:
@@ -890,7 +1054,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
 			_closeHyperlink();
 			_closeBlock();
-			_openTag("p","",false,pcr->getIndexAP());
+			_openTag("p","",false,pcr->getIndexAP(),pcr->getXID());
 			m_bInBlock = true;
 			return true;
 		}
@@ -1023,7 +1187,7 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 		}
 
 		PT_AttrPropIndex api = pStyle->getIndexAP();
-		_openTag("s","/",true,api);
+		_openTag("s","/",true,api,0);
 	}
 
 	for (k=0; m_pDocument->enumStyles(k, &szName, &pStyle); k++)
@@ -1038,7 +1202,7 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 		}
 
 		PT_AttrPropIndex api = pStyle->getIndexAP();
-		_openTag("s","/",true,api);
+		_openTag("s","/",true,api,0);
 	}
 
 	if (bWroteOpenStyleSection)
@@ -1184,7 +1348,7 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 
 	UT_ByteBuf bbEncoded(1024);
 	UT_Set::Iterator end(m_pUsedImages.end());
-
+	UT_DEBUGMSG(("USed images are... \n"));
 	for (UT_uint32 k=0;
 		 (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,reinterpret_cast<const void**>(&szMimeType)));
 		 k++)
@@ -1211,7 +1375,7 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 	   	bool status = false;
 	   	bool encoded = true;
 
-		if (szMimeType && (UT_strcmp(szMimeType, "image/svg-xml") == 0 || UT_strcmp(szMimeType, "text/mathml") == 0))
+		if (szMimeType && (UT_strcmp(szMimeType, "image/svg+xml") == 0 || UT_strcmp(szMimeType, "application/mathml+xml") == 0))
 	    {
 		   bbEncoded.truncate(0);
 		   bbEncoded.append(reinterpret_cast<const UT_Byte*>("<![CDATA["), 9);
@@ -1346,6 +1510,7 @@ void s_AbiWord_1_Listener::_handleHistory(void)
 		const UT_UUID& UID  =  m_pDocument->getHistoryNthUID(k);
 		time_t tStarted     =  m_pDocument->getHistoryNthTimeStarted(k);
 		bool bAuto          =  m_pDocument->getHistoryNthAutoRevisioned(k);
+		UT_uint32 iXID      =  m_pDocument->getHistoryNthTopXID(k);
 		
 		UT_UTF8String s, hUid;
 		UID.toString(hUid);
@@ -1364,8 +1529,8 @@ void s_AbiWord_1_Listener::_handleHistory(void)
 			bWroteOpenSection = true;
 		}
 
-		UT_UTF8String_sprintf(s, "<version id=\"%d\" started=\"%d\" uid=\"%s\" auto=\"%d\"/>\n",
-						  iVersion, tStarted, hUid.utf8_str(),bAuto);
+		UT_UTF8String_sprintf(s, "<version id=\"%d\" started=\"%d\" uid=\"%s\" auto=\"%d\" top-xid=\"%d\"/>\n",
+						  iVersion, tStarted, hUid.utf8_str(),bAuto, iXID);
 		
 		m_pie->write(s.utf8_str());
 	}
