@@ -50,7 +50,7 @@
 #define X_TestParseState(ps)	((m_parseState==(ps)))
 
 #define X_VerifyParseState(ps)	do {  if (!(X_TestParseState(ps)))  \
-									  {  m_error = UT_IE_BOGUSDOCUMENT;	UT_ASSERT(0); \
+									  {  m_error = UT_IE_BOGUSDOCUMENT;	UT_ASSERT_HARMLESS(0); \
 										 return; } } while (0)
 
 #define X_CheckDocument(b)		do {  if (!(b))								\
@@ -175,16 +175,23 @@ IE_Imp_AbiWord_1::~IE_Imp_AbiWord_1()
 	
   if (m_refMap)
   {
-	  UT_HASH_PURGEDATA (UT_UTF8String *, m_refMap, delete);
+	  m_refMap->purgeData();
 	  delete m_refMap;
 	  m_refMap = 0;
   }
 }
 
 IE_Imp_AbiWord_1::IE_Imp_AbiWord_1(PD_Document * pDocument)
-  : IE_Imp_XML(pDocument, true), m_bWroteSection (false),
-    m_bWroteParagraph(false), m_bDocHasLists(false), m_bDocHasPageSize(false),
-    m_iInlineStart(0), m_refMap(new UT_StringPtrMap)
+  : IE_Imp_XML(pDocument, true), 
+	m_bWroteSection (false),
+    m_bWroteParagraph(false), 
+	m_bDocHasLists(false), 
+	m_bDocHasPageSize(false),
+	m_iInlineStart(0), 
+	m_refMap(new UT_GenericStringMap<UT_UTF8String*>),
+	m_bAutoRevisioning(false),
+	m_bInMath(false),
+	m_iImageId(0)
 {
 }
 
@@ -231,6 +238,12 @@ UT_Error IE_Imp_AbiWord_1::importFile(const char * szFilename)
 #define TT_REVISION        29 //<r>
 #define TT_RESOURCE        30 // <resource>
 #define TT_ENDNOTE         31 //<endnote>
+#define TT_HISTORYSECTION  32 //<history>
+#define TT_VERSION         33 //<version>
+#define TT_TOC             34 //<toc> (Table of Contents
+#define TT_MATH            35 //<math> Math Run
+#define TT_EMBED            36 //<embed> Generic Embeded Run
+
 
 /*
   TODO remove tag synonyms.  We're currently accepted
@@ -260,11 +273,13 @@ static struct xmlToIdMapping s_Tokens[] =
 	{	"cell",		    TT_CELL		    },
 	{	"d",			TT_DATAITEM		},
 	{	"data",			TT_DATASECTION	},
+	{   "embed",      TT_EMBED      },
 	{   "endnote",      TT_ENDNOTE      },
 	{	"f",			TT_FIELD		},
 	{	"field",		TT_FIELD		},
 	{	"foot",		    TT_FOOTNOTE	    },
 	{	"frame",		TT_FRAME	    },
+	{   "history",      TT_HISTORYSECTION},
 	{	"i",			TT_IMAGE		},
 	{	"ignoredwords",	TT_IGNOREDWORDS	},
 	{	"image",		TT_IMAGE		},
@@ -273,6 +288,7 @@ static struct xmlToIdMapping s_Tokens[] =
 	{	"lists",		TT_LISTSECTION	},
 	{       "m",        TT_META         },
 	{	"margin",		TT_MARGINNOTE	},
+	{	"math",		    TT_MATH     	},
 	{       "metadata", TT_METADATA     },
 	{	"p",			TT_BLOCK		},
 	{   "pagesize",     TT_PAGESIZE     },
@@ -283,7 +299,10 @@ static struct xmlToIdMapping s_Tokens[] =
 	{	"s",			TT_STYLE		},
 	{	"section",		TT_SECTION		},
 	{	"styles",		TT_STYLESECTION	},
-	{	"table",		TT_TABLE		}
+	{	"table",		TT_TABLE		},
+	{	"toc",		    TT_TOC  		},
+	{   "version",      TT_VERSION      }
+	
 };
 
 #define TokenTableSize	((sizeof(s_Tokens)/sizeof(s_Tokens[0])))
@@ -321,8 +340,9 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 		bool bOK = true;
 		if(pszId)
 		{
-			//UT_uint32 id = atoi(pszId);
-		    bOK = getDoc()->verifySectionID(pszId);
+			UT_uint32 id = atoi(pszId);
+			getDoc()->setMinUID(UT_UniqueId::HeaderFtr, id+1);
+			bOK = getDoc()->verifySectionID(pszId);
 		}
 		if(bOK)
 		{
@@ -362,6 +382,16 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 		// Don't check for id on the footnote. It should match an id on
         // a footnote reference field.
 
+		// Need to set the min Unique id now.
+
+		const XML_Char * pszId = static_cast<const XML_Char*>(_getXMLPropValue("footnote-id", atts));
+		bool bOK = true;
+		if(pszId)
+		{
+			UT_uint32 id = atoi(pszId);
+			bOK = getDoc()->setMinUID(UT_UniqueId::Footnote, id+1);
+		}
+
 		X_CheckError(appendStrux(PTX_SectionFootnote,atts));
 		xxx_UT_DEBUGMSG(("FInished Append footnote strux \n"));
 		return;
@@ -373,11 +403,31 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 		m_parseState = _PS_Sec;
 		m_bWroteSection = true;
 
+		// Need to set the min Unique id now.
+
+		const XML_Char * pszId = static_cast<const XML_Char*>(_getXMLPropValue("endnote-id", atts));
+		bool bOK = true;
+		if(pszId)
+		{
+			UT_uint32 id = atoi(pszId);
+			bOK = getDoc()->setMinUID(UT_UniqueId::Endnote, id+1);
+		}
+
 		// Don't check for id of the endnote strux. It should match the
 		// id of the endnote reference.
 
 		X_CheckError(appendStrux(PTX_SectionEndnote,atts));
-		xxx_UT_DEBUGMSG(("FInished Append Endnote strux \n"));
+		xxx_UT_DEBUGMSG(("Finished Append Endnote strux \n"));
+		return;
+	}
+	case TT_TOC:
+	{
+		// Table of Contents are inside a DocSection
+		X_VerifyParseState(_PS_Sec);
+		m_parseState = _PS_Sec;
+		m_bWroteSection = true;
+		X_CheckError(appendStrux(PTX_SectionTOC,atts));
+		UT_DEBUGMSG(("Finished Append TOC strux \n"));
 		return;
 	}
 	case TT_BLOCK:
@@ -401,7 +451,7 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 			if(!bOK)
 			{
 				UT_DEBUGMSG(("List id %d [%s] already in use\n",id,pszId));
-				UT_ASSERT( UT_SHOULD_NOT_HAPPEN );
+				UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
 			}
 		}
 		X_CheckError(appendStrux(PTX_Block,atts));
@@ -437,6 +487,11 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 
 	case TT_IMAGE:
 	{
+		if(m_bInMath)
+		{
+			UT_DEBUGMSG(("Ignore image in math-tag \n"));
+			return;
+		}
 		X_VerifyParseState(_PS_Block);
 #ifdef ENABLE_RESOURCE_MANAGER
 		X_CheckError(_handleImage (atts));
@@ -451,6 +506,23 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 #endif
 		return;
 	}
+	case TT_MATH:
+	{
+		X_VerifyParseState(_PS_Block);
+		X_CheckError(appendObject(PTO_Math,atts));
+		m_iImageId++;
+		getDoc()->setMinUID(UT_UniqueId::Image, m_iImageId);
+		m_bInMath = true;
+		return;
+	}
+	case TT_EMBED:
+	{
+		X_VerifyParseState(_PS_Block);
+		X_CheckError(appendObject(PTO_Embed,atts));
+		m_iImageId++;
+		getDoc()->setMinUID(UT_UniqueId::Image, m_iImageId);
+		return;
+	}
 	case TT_BOOKMARK:
 		X_VerifyParseState(_PS_Block);
 		X_CheckError(appendObject(PTO_Bookmark,atts));
@@ -485,7 +557,7 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 		// seem a little odd (perhaps an &lf; entity would be better).
 		// Anyway, this distinction from ordinary LF's in the document
 		// (which get mapped into SPACE) keeps the file sanely editable.
-
+	
 	case TT_BREAK:
 		if(X_TestParseState(_PS_Field))
 			return; // just return
@@ -588,25 +660,154 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 	}
 
 	case TT_REVISIONSECTION:
+	{
+				
 		X_VerifyParseState(_PS_Doc);
 		m_parseState = _PS_RevisionSec;
-		// We don't need to notify the piece table of the style section,
-		// it will get the hint when we begin sending styles.
+
+		// parse the attributes ...
+		const XML_Char * szS = UT_getAttribute("show",atts);
+		UT_uint32 i;
+		if(szS)
+		{
+			i = atoi(szS);
+			getDoc()->setShowRevisions(i != 0);			
+		}
+
+		szS = UT_getAttribute("mark",atts);
+		if(szS)
+		{
+			i = atoi(szS);
+			getDoc()->setMarkRevisions(i != 0);
+		}
+		
+		szS = UT_getAttribute("show-level",atts);
+		if(szS)
+		{
+			i = atoi(szS);
+			getDoc()->setShowRevisionId(i);
+		}
+
+		szS = UT_getAttribute("auto",atts);
+		if(szS)
+		{
+			i = atoi(szS);
+			// we cannot call setAutoRevisioning() from here because
+			// it creates a new revision, so we can only call it after
+			// the revisions have been all read it -- we will call it
+			// in the </revisions> processing
+			m_bAutoRevisioning = (i == 1);
+
+			// autorevisioned documents should not have revision
+			// marking turned on
+			if(m_bAutoRevisioning)
+			{
+				getDoc()->setShowRevisionId(PD_MAX_REVISION);
+				getDoc()->setShowRevisions(false);
+			}
+			
+			
+		}
 		return;
+	}
 
 	case TT_REVISION:
 	{
 		X_VerifyParseState(_PS_RevisionSec);
 		m_parseState = _PS_Revision;
 
-		const XML_Char * szId = UT_getAttribute(PT_ID_ATTRIBUTE_NAME,atts);
-		if(szId)
-			m_currentRevisionId = atoi(szId);
+		const XML_Char * szS = UT_getAttribute(PT_ID_ATTRIBUTE_NAME,atts);
+		if(szS)
+		{
+			m_currentRevisionId = atoi(szS);
+			m_currentRevisionTime = 0;
+
+			szS = UT_getAttribute("time-started",atts);
+			if(szS)
+				m_currentRevisionTime = (time_t)atoi(szS);
+
+			szS = UT_getAttribute("version",atts);
+			if(szS)
+				m_currentRevisionVersion = atoi(szS);
+		}
 
 		return;
 	}
 
-   case TT_LISTSECTION:
+	case TT_HISTORYSECTION:
+	{
+				
+		X_VerifyParseState(_PS_Doc);
+		m_parseState = _PS_HistorySec;
+
+		// parse the attributes ...
+		const XML_Char * szS = UT_getAttribute("version",atts);
+		UT_uint32 i;
+		if(szS)
+		{
+			i = atoi(szS);
+			getDoc()->setDocVersion(i);			
+		}
+
+		szS = UT_getAttribute("edit-time",atts);
+		if(szS)
+		{
+			i = atoi(szS);
+			getDoc()->setEditTime(i);
+		}
+		
+		szS = UT_getAttribute("last-saved",atts);
+		if(szS)
+		{
+			i = atoi(szS);
+			getDoc()->setLastSavedTime((time_t)i);
+		}
+		szS = UT_getAttribute("uid",atts);
+		if(szS)
+		{
+			getDoc()->setDocUUID(szS);
+		}
+		
+		return;
+	}
+	
+	case TT_VERSION:
+	{
+		X_VerifyParseState(_PS_HistorySec);
+		m_parseState = _PS_Version;
+
+		const XML_Char * szS = UT_getAttribute(PT_ID_ATTRIBUTE_NAME,atts);
+		if(szS)
+		{
+			UT_uint32 iId = atoi(szS);
+
+			time_t tStarted = 0;
+			szS = UT_getAttribute("started",atts);
+			if(szS)
+				tStarted = (time_t) atoi(szS);
+
+			bool bAuto = false;
+			szS = UT_getAttribute("auto",atts);
+			if(szS)
+				bAuto = (0 != atoi(szS));
+			else
+				bAuto = false;
+
+			UT_uint32 iXID = 0;
+			szS = UT_getAttribute("top-xid",atts);
+			if(szS)
+				iXID = atoi(szS);
+			
+			szS = UT_getAttribute("uid",atts);
+
+			AD_VersionData v(iId, szS, tStarted, bAuto, iXID);
+			getDoc()->addRecordToHistory(v);
+		}
+
+		return;
+	}
+
+		case TT_LISTSECTION:
 		X_VerifyParseState(_PS_Doc);
 		m_parseState = _PS_ListSec;
 		// As per styles, we don't need to notify the piece table.
@@ -660,6 +861,12 @@ void IE_Imp_AbiWord_1::startElement(const XML_Char *name, const XML_Char **atts)
 		X_CheckError(appendStrux(PTX_SectionCell,atts));
 		return;
 
+	case TT_FRAME:
+		m_parseState = _PS_Sec;
+		m_bWroteSection = true;
+		X_CheckError(appendStrux(PTX_SectionFrame,atts));
+		return;
+
 	case TT_OTHER:
 	default:
 		UT_DEBUGMSG(("Unknown tag [%s]\n",name));
@@ -714,6 +921,14 @@ void IE_Imp_AbiWord_1::endElement(const XML_Char *name)
 		m_parseState = _PS_Block;
 		return;
 
+
+	case TT_TOC:
+		X_VerifyParseState(_PS_Sec);
+		m_bWroteSection = true;
+		X_CheckError(appendStrux(PTX_EndTOC,NULL));
+		UT_DEBUGMSG(("Finished Append End TOC strux \n"));
+		return;
+
 	case TT_TABLE:
 		X_VerifyParseState(_PS_Sec);
 		m_bWroteSection = true;
@@ -726,6 +941,12 @@ void IE_Imp_AbiWord_1::endElement(const XML_Char *name)
 		X_CheckError(appendStrux(PTX_EndCell,NULL));
 		return;
 
+	case TT_FRAME:
+		X_VerifyParseState(_PS_Sec);
+		m_bWroteSection = true;
+		X_CheckError(appendStrux(PTX_EndFrame,NULL));
+		return;
+
 	case TT_BLOCK:
 		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		X_VerifyParseState(_PS_Block);
@@ -751,6 +972,19 @@ void IE_Imp_AbiWord_1::endElement(const XML_Char *name)
 		return;
 
 	case TT_IMAGE:						// not a container, so we don't pop stack
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
+		X_VerifyParseState(_PS_Block);
+		m_iImageId++;
+		getDoc()->setMinUID(UT_UniqueId::Image, m_iImageId);
+		return;
+
+	case TT_MATH:						// not a container, so we don't pop stack
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
+		m_bInMath = false;
+		UT_DEBUGMSG(("In math set false \n"));
+		X_VerifyParseState(_PS_Block);
+		return;
+	case TT_EMBED:						// not a container, so we don't pop stack
 		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		X_VerifyParseState(_PS_Block);
 		return;
@@ -839,14 +1073,26 @@ void IE_Imp_AbiWord_1::endElement(const XML_Char *name)
 		return;
 
 	case TT_STYLE:
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		X_VerifyParseState(_PS_Style);
 		m_parseState = _PS_StyleSec;
 		return;
 
+	case TT_HISTORYSECTION:
+		X_VerifyParseState(_PS_HistorySec);
+		m_parseState = _PS_Doc;
+		return;
+		
+	case TT_VERSION:
+		X_VerifyParseState(_PS_Version);
+		m_parseState = _PS_HistorySec;
+		return;
+
 	case TT_REVISIONSECTION:
 		X_VerifyParseState(_PS_RevisionSec);
 		m_parseState = _PS_Doc;
+		getDoc()->setAutoRevisioning(m_bAutoRevisioning);
+		m_bAutoRevisioning = false;
 		return;
 
 	case TT_REVISION:
@@ -856,7 +1102,9 @@ void IE_Imp_AbiWord_1::endElement(const XML_Char *name)
 		{
 			// the revision had no comment associated, so it was not
 			// added to the doc by the xml paraser
-			X_CheckError(getDoc()->addRevision(m_currentRevisionId, NULL, 0));
+			X_CheckError(getDoc()->addRevision(m_currentRevisionId, NULL, 0,
+											   m_currentRevisionTime,
+											   m_currentRevisionVersion));
 			m_currentRevisionId = 0;
 		}
 		
@@ -992,7 +1240,7 @@ bool IE_Imp_AbiWord_1::_handleImage (const XML_Char ** atts)
 
 			new_id = &re_id;
 		}
-	else if ((new_id = reinterpret_cast<const UT_UTF8String *>(m_refMap->pick (old_id))) == 0)
+	else if ((new_id = m_refMap->pick (old_id)) == 0)
 		{
 			/* first occurence of this href/dataid; add to map
 			 */
@@ -1000,7 +1248,7 @@ bool IE_Imp_AbiWord_1::_handleImage (const XML_Char ** atts)
 			if (ri_id)
 				{
 					m_refMap->insert (old_id, ri_id);
-					if ((new_id = reinterpret_cast<const UT_UTF8String *>(m_refMap->pick (old_id))) == 0)
+					if ((new_id = m_refMap->pick (old_id)) == 0)
 						{
 							delete ri_id;
 						}
@@ -1046,6 +1294,8 @@ bool IE_Imp_AbiWord_1::_handleImage (const XML_Char ** atts)
 	*new_attr++ = 0;
 
 	bool success = appendObject (PTO_Image, new_atts);
+	m_iImageId++;
+	getDoc()->setMinUID(UT_UniqueId::Image, m_iImageId);
 
 	free (new_atts);
 
@@ -1109,13 +1359,13 @@ bool IE_Imp_AbiWord_1::_handleResource (const XML_Char ** atts, bool isResource)
 	else
 		{
 			// <d name="ID" mime-type="image/png" base64="yes"> ... </d>
-			// <d name="ID" mime-type="image/svg-xml | text/mathml" base64="no"> <![CDATA[ ... ]]> </d>
+			// <d name="ID" mime-type="image/svg+xml | application/mathxml+xml" base64="no"> <![CDATA[ ... ]]> </d>
 
 			const XML_Char * r_id = 0;
 			const XML_Char * r_64 = 0;
 
-			enum { mt_unknown, mt_png, mt_svg, mt_mathml } mt = mt_unknown;
-
+			enum { mt_unknown, mt_png, mt_svg, mt_mathml,mt_embed } mt = mt_unknown;
+			const XML_Char * pszEmbed = NULL;
 			const XML_Char ** attr = atts;
 			while (*attr)
 				{
@@ -1130,10 +1380,15 @@ bool IE_Imp_AbiWord_1::_handleResource (const XML_Char ** atts, bool isResource)
 
 							if (strcmp (*attr, "image/png") == 0)
 								mt = mt_png;
-							else if (strcmp (*attr, "image/svg-xml") == 0)
+							else if (strcmp (*attr, "image/svg+xml") == 0 || strcmp (*attr, "image/svg"))
 								mt = mt_svg;
-							else if (strcmp (*attr, "text/mathml") == 0)
+							else if (strcmp (*attr, "application/mathml+xml") == 0)
 								mt = mt_mathml;
+							else
+								{
+									pszEmbed = *attr;
+									mt = mt_embed;
+								}
 
 							attr++;
 						}
@@ -1153,7 +1408,7 @@ bool IE_Imp_AbiWord_1::_handleResource (const XML_Char ** atts, bool isResource)
 
 			/* map dataid to new resource ID
 			 */
-			const UT_UTF8String * new_id = reinterpret_cast<const UT_UTF8String *>(m_refMap->pick (r_id));
+			const UT_UTF8String * new_id = m_refMap->pick (r_id);
 			if (new_id == 0) return false;
 
 			XAP_InternalResource * ri = dynamic_cast<XAP_InternalResource *>(RM.resource (new_id->utf8_str (), true));
@@ -1186,6 +1441,14 @@ bool IE_Imp_AbiWord_1::_handleResource (const XML_Char ** atts, bool isResource)
 							add_resource = true;
 						}
 					break;
+				case mt_embed:
+					if (strcmp (r_64, "no") == 0) // hmm, CDATA fun
+						{
+							ri->type (pszEmbed); 
+							m_currentDataItemEncoded = false;
+							add_resource = true;
+						}
+					break;
 				default:
 					break;
 				}
