@@ -32,10 +32,10 @@
 #ifndef UT_TYPES_H
 #include "ut_types.h"
 #endif
+#include "ut_hash.h"
 
 #include "ie_imp_XML.h"
 
-class UT_StringPtrMap;
 
 class PD_Document;
 
@@ -91,7 +91,10 @@ protected:
 
     UT_uint32			m_iInlineStart;
 
-	UT_StringPtrMap *	m_refMap;
+	UT_GenericStringMap<UT_UTF8String *> *	m_refMap;
+	bool                m_bAutoRevisioning;
+	bool                m_bInMath;
+	UT_uint32           m_iImageId;
 };
 
 #endif /* IE_IMP_ABIWORD_1_H */
