@@ -448,7 +448,7 @@ struct ABI_EXPORT RTFStateStore
 {
 	RTFStateStore();
     RTFStateStore * clone();
-//	RTFStateStore& operator=(const RTFStateStore&);
+	// RTFStateStore& operator=(const RTFStateStore& s)
 
 	enum DestinationStateTypes { rdsNorm, rdsSkip, rdsFootnote, rdsHeader, rdsFooter, rdsField };
 	enum InternalStateTypes { risNorm, risBin, risHex };
@@ -462,6 +462,7 @@ struct ABI_EXPORT RTFStateStore
     RTFProps_TableProps     m_tableProps;           // Table properties
 	UT_uint32				m_unicodeAlternateSkipCount;	// value of N in "\ucN"
 	UT_uint32				m_unicodeInAlternate;			// chars left in alternate "\u<u><A>"
+	UT_UTF8String           m_revAttr;              // the revision attribute stored in abirevision
 };
 
 
@@ -696,6 +697,7 @@ private:
 	bool AddTabstop(UT_sint32 stopDist, eTabType tabType, 
                     eTabLeader tabLeader,  RTFProps_ParaProps * pParas);
 
+
 // Paste AbiWord tables
  public:
 	bool HandleAbiTable(void);
@@ -703,6 +705,8 @@ private:
 	bool HandleAbiEndTable(void);
 	bool HandleAbiEndCell(void);
 	bool HandleAbiLists(void);
+	bool HandleAbiMathml(void);
+	bool HandleAbiEmbed(void);
 	bool isPastedTableOpen(void);
 	bool markPasteBlock(void);
 	bool isBlockNeededForPasteTable(void);
@@ -894,7 +898,6 @@ private:
 	UT_BidiCharType       m_iAutoBidiOverride;
 	UT_BidiCharType       m_iBidiLastType;
 	UT_BidiCharType       m_iBidiNextType;
-
 };
 
 #endif /* IE_IMP_RTF_H */
