@@ -350,8 +350,14 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 		case RTFProps_ImageProps::ipstScale:
 			UT_DEBUGMSG (("Scale: x=%d, y=%d, w=%d, h=%d\n", imgProps.scaleX, imgProps.scaleY, imgProps.width, imgProps.height));
 			resize = true;
-			wInch = ((static_cast<double>(imgProps.scaleX) / 100.0f) * imgProps.width);
-			hInch = ((static_cast<double>(imgProps.scaleY) / 100.0f) * imgProps.height);
+			if ((imgProps.wGoal != 0) && (imgProps.hGoal != 0)) {
+				// want image scaled against the w&h specified, not the image's natural size
+				wInch = ((static_cast<double>(imgProps.scaleX) / 100.0f) * imgProps.wGoal/ 1440.0f);
+				hInch = ((static_cast<double>(imgProps.scaleY) / 100.0f) * imgProps.hGoal/ 1440.0f);
+			} else {
+				wInch = ((static_cast<double>(imgProps.scaleX) / 100.0f) * (imgProps.width));
+				hInch = ((static_cast<double>(imgProps.scaleY) / 100.0f) * (imgProps.height));
+			}
 			break;
 		default:
 			resize = false;
@@ -432,6 +438,7 @@ bool IE_Imp_RTF::HandlePicture()
 			}
 			UT_DEBUGMSG(("Doing standard picture stuff with keyword %s \n",keyword));
 			keywordID = KeywordToID(reinterpret_cast<char *>(keyword));
+			UT_ASSERT(RTF_KW_cell  != keywordID);
 			switch (keywordID)
 			{
 			case RTF_KW_pngblip:
@@ -462,25 +469,17 @@ bool IE_Imp_RTF::HandlePicture()
 				}
 				break;
 			case RTF_KW_picscalex:
-				if ((imageProps.sizeType == RTFProps_ImageProps::ipstNone)
-					|| (imageProps.sizeType == RTFProps_ImageProps::ipstScale))
+				if ((parameterUsed) && (parameter != 100))       // scale the image if one of these two keywords appear
 				{
-					if ((parameterUsed) && (parameter != 100))
-					{
-						imageProps.sizeType = RTFProps_ImageProps::ipstScale;
-						imageProps.scaleX = static_cast<unsigned short>(parameter);
-					}
+					imageProps.sizeType = RTFProps_ImageProps::ipstScale;
+					imageProps.scaleX = static_cast<unsigned short>(parameter);
 				}
 				break;
 			case RTF_KW_picscaley:
-				if ((imageProps.sizeType == RTFProps_ImageProps::ipstNone)
-					|| (imageProps.sizeType == RTFProps_ImageProps::ipstScale))
+				if ((parameterUsed) && (parameter != 100))
 				{
-					if ((parameterUsed) && (parameter != 100))
-					{
-						imageProps.sizeType = RTFProps_ImageProps::ipstScale;
-						imageProps.scaleY = static_cast<unsigned short>(parameter);
-					}
+					imageProps.sizeType = RTFProps_ImageProps::ipstScale;
+					imageProps.scaleY = static_cast<unsigned short>(parameter);
 				}
 				break;
 			case RTF_KW_piccropt:
@@ -690,7 +689,8 @@ void RTFProps_FrameProps::_setProperty(const PropertyPair *pair)
 //
 		UT_DEBUGMSG(("Found positioned Image \n"));
 	}
-	else {
+	else if( propName && propValue)
+	{
 		UT_DEBUGMSG(("unknown property %s with value %s\n", propName->utf8_str(),
 					 propValue->utf8_str() ));
 	}
@@ -845,7 +845,7 @@ bool IE_Imp_TextParaPropParser::tokenKeyword(IE_Imp_RTF * ie,
 										   RTF_KEYWORD_ID kwID, 
 										   UT_sint32 param, bool paramUsed)
 {
-	UT_DEBUGMSG(("IE_Imp_TextParPropParser::tokenKeyword()\n"));
+	xxx_UT_DEBUGMSG(("IE_Imp_TextParPropParser::tokenKeyword()\n"));
 	return ie->TranslateKeywordID(kwID, param, paramUsed);
 }
 
@@ -1158,17 +1158,12 @@ void IE_Imp_RTF::addFrame(RTFProps_FrameProps & frame)
 	if(!bUseInsertNotAppend())
 	{
 		getDoc()->appendStrux(PTX_SectionFrame,attribs);
-		getDoc()->appendStrux(PTX_Block,NULL);
 	}
 	else
 	{
 		insertStrux(PTX_SectionFrame,attribs,NULL);
-		UT_DEBUGMSG((" Insert Block at Frame \n"));
-		markPasteBlock();
-		insertStrux(PTX_Block);
 	}
 	m_bFrameStruxIn = true;
-
 }
 
 /*!
@@ -1213,6 +1208,7 @@ void IE_Imp_RTF::HandleShapePict()
 		case RTF_TOKEN_KEYWORD:
 		{
 			keywordID = KeywordToID(reinterpret_cast<char *>(keyword));
+			UT_ASSERT(RTF_KW_cell  != keywordID);
 			switch (keywordID)
 			{
 			case RTF_KW_pict:
@@ -1276,6 +1272,7 @@ bool IE_Imp_RTF::HandleObject()
 		case RTF_TOKEN_KEYWORD:
 		{
 			keywordID = KeywordToID(reinterpret_cast<char *>(keyword));
+			UT_ASSERT(RTF_KW_cell  != keywordID);
 			switch (keywordID)
 			{
 			case RTF_KW_rsltrtf:
