@@ -43,7 +43,22 @@ EMBED_LIB_DIR=@executable_path/../Frameworks
 #  -iconv
 #  -bz2
 
-DYLIBS="libwpd-0.8.dylib libwpd-stream-0.8.dylib libgsf-1.1.dylib libgthread-2.0.dylib libgobject-2.0.dylib libglib-2.0.dylib libxml2.2.dylib libintl.1.dylib libiconv.2.dylib libbz2.1.dylib"
+WPDLIBS="libwpd-0.8.dylib libwpd-stream-0.8.dylib"
+libwpd_count = 0
+DYLIBS="libgsf-1.1.dylib libgthread-2.0.dylib libgobject-2.0.dylib libglib-2.0.dylib libxml2.2.dylib libintl.1.dylib libiconv.2.dylib libbz2.1.dylib"
+
+# add optional libwpd...
+for i in $WPDLIBS ; do
+    if [ -f $i ] ; then
+        libwpd_count = $$liwpd_count + 1
+    fi
+done
+
+if [ $liwpd_count = "2" ] ; then
+    DYLIBS="$DYLIBS $WPDLIBS"
+else
+    echo "Can't find $WPDLIBS. WordPerfect plugin not built."
+fi
 
 for i in $DYLIBS ; do
     if [ ! -f $FINK_LIB_DIR/$i ] ; then
