@@ -32,29 +32,6 @@
 
 #include "ut_debugmsg.h"
 
-static void
-oo_gsf_output_write(GsfOutput *output, size_t num_bytes, guint8 const *data)
-{
-  if (!gsf_output_write(output, num_bytes, data)) {
-    UT_DEBUGMSG(("DOM: gsf_output_write() failed\n"));
-    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-  }
-}
-
-static void
-oo_gsf_output_close(GsfOutput *output)
-{
-  if (!gsf_output_close(output)) {
-    const GError * err = gsf_output_error(output);
-    UT_DEBUGMSG(("DOM: gsf_output_close() failed\n"));
-    if (err) {
-      UT_DEBUGMSG(("DOM: reason: %s\n", err->message));
-    }
-    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-  }
-  g_object_unref(output);
-}
-
 /*****************************************************************************/
 /*****************************************************************************/
 
@@ -107,17 +84,17 @@ bool IE_Exp_OpenWriter_Sniffer::getDlgLabels(const char ** pszDesc,
 static void writeToStream (GsfOutput * stream, const char * const message [], size_t nElements)
 {
     for(UT_uint32 k = 0; k < nElements; k++)
-      oo_gsf_output_write(stream, strlen(message[k]), reinterpret_cast<const guint8 *>(message[k]));
+      gsf_output_write(stream, strlen(message[k]), reinterpret_cast<const guint8 *>(message[k]));
 }
 
 static void writeString (GsfOutput * output, const UT_String & str)
 {
-   oo_gsf_output_write (output, str.length(), reinterpret_cast<const guint8*>(str.c_str()));
+   gsf_output_write (output, str.length(), reinterpret_cast<const guint8*>(str.c_str()));
 }
 
 static void writeUTF8String (GsfOutput * output, const UT_UTF8String & str)
 {
-  oo_gsf_output_write (output, str.byteLength(), reinterpret_cast<const guint8*>(str.utf8_str()));
+  gsf_output_write (output, str.byteLength(), reinterpret_cast<const guint8*>(str.utf8_str()));
 }
 
 static void outputCharData (GsfOutput * output, const UT_UCSChar * data, UT_uint32 length)
@@ -187,12 +164,14 @@ OO_WriterImpl::OO_WriterImpl(GsfOutfile *pOutfile, OO_StylesContainer *pStylesCo
 	};
 	writeToStream(m_pContentStream, preamble, NrElements(preamble));
 
+
 	// in my sample files content.xml, styles.xml font-decl sections are the same
-	UT_UTF8String fontDecls = "<office:font-decls>\n";
+	UT_UTF8String fontDecls = "<office:font-decls>";
 	OO_StylesWriter::addFontDecls(fontDecls, *m_pStylesContainer);
-	fontDecls += "</office:font-decls>\n";
+	fontDecls += "</office:font-decls>";
 	writeUTF8String(m_pContentStream, fontDecls);
 	
+
 	writeUTF8String(m_pContentStream, "<office:automatic-styles>\n");
 	
 	int *styleNum = NULL;
@@ -211,8 +190,8 @@ OO_WriterImpl::OO_WriterImpl(GsfOutfile *pOutfile, OO_StylesContainer *pStylesCo
 	   UT_DEBUGMSG(("%s", styleString.c_str()));
 	   UT_DEBUGMSG(("\"%s\"\n", styleProps->c_str()));
 	}
-	DELETEP(tempStylesKeysList);
-	DELETEP(tempStylesValuesList);
+	FREEP(tempStylesKeysList);
+	FREEP(tempStylesValuesList);
 
 	// block styles
 	UT_GenericVector<const UT_String*> *tempBlockStylesKeysList = m_pStylesContainer->getBlockStylesKeys();
@@ -226,7 +205,7 @@ OO_WriterImpl::OO_WriterImpl(GsfOutfile *pOutfile, OO_StylesContainer *pStylesCo
 		styleString += UT_String_sprintf("</style:style>");
 		writeString(m_pContentStream, styleString);
 	}
-	DELETEP(tempBlockStylesKeysList);
+	FREEP(tempBlockStylesKeysList);
 
     //m_acc.writeStylePreamble(m_contentStream);
     
@@ -254,7 +233,8 @@ OO_WriterImpl::~OO_WriterImpl()
       };
     writeToStream(m_pContentStream, postamble, NrElements(postamble));
 
-    oo_gsf_output_close(m_pContentStream);
+    gsf_output_close(m_pContentStream);
+    g_object_unref(G_OBJECT(m_pContentStream));
 }
 
 void OO_WriterImpl::insertText(const UT_UCSChar * data, UT_uint32 length)
@@ -279,12 +259,12 @@ void OO_WriterImpl::openBlock(UT_String & styleAtts, UT_String & styleProps, UT_
 	if (bIsHeading) 
 	{
 		tag = "<text:h " + props + ">";
-		m_blockEnd = "</text:h>\n";
+		m_blockEnd = "</text:h>";
 	}
 	else
 	{
 		tag = "<text:p " + props + ">";
-		m_blockEnd = "</text:p>\n";
+		m_blockEnd = "</text:p>";
 	}	
 
 	writeUTF8String(m_pContentStream, tag);
@@ -492,12 +472,12 @@ bool OO_Listener::change(PL_StruxFmtHandle sfh,
 }
    
 bool OO_Listener::insertStrux(PL_StruxFmtHandle sfh,
-			      const PX_ChangeRecord * pcr,
-			      PL_StruxDocHandle sdh,
-			      PL_ListenerId lid,
-			      void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
-						      PL_ListenerId lid,
-						      PL_StruxFmtHandle sfhNew))
+				      const PX_ChangeRecord * pcr,
+				      PL_StruxDocHandle sdh,
+				      PL_ListenerId lid,
+				      void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
+							      PL_ListenerId lid,
+							      PL_StruxFmtHandle sfhNew))
 {
    UT_ASSERT_NOT_REACHED();
    return true;
@@ -634,17 +614,17 @@ public:
 
     if (pDoc->getMetaDataProp(PD_META_KEY_DATE, meta_val) && meta_val.size()) {
       val = UT_UTF8String_sprintf("<dc:date>%s</dc:date>\n", meta_val.utf8_str());
-      oo_gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
+      gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
     }
     if (pDoc->getMetaDataProp(PD_META_KEY_LANGUAGE, meta_val) && meta_val.size()) {
       val = UT_UTF8String_sprintf("<dc:language>%s</dc:language>\n", meta_val.utf8_str());
-      oo_gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
+      gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
     }
 
     writeToStream(meta, postamble, NrElements(postamble));
 
-    oo_gsf_output_close(meta);
-
+    gsf_output_close(meta);
+    g_object_unref(G_OBJECT(meta));
     return true;
   }
 
@@ -681,7 +661,8 @@ public:
 
     writeToStream (settings, preamble, NrElements(preamble));
 
-    oo_gsf_output_close(settings);
+    gsf_output_close(settings);
+    g_object_unref(G_OBJECT(settings));
 
     return true;
   }
@@ -715,13 +696,14 @@ public:
       {
 	// create individual pictures
 	UT_String name = UT_String_sprintf("IMG-%d.png", k);
-	GsfOutput * img = gsf_outfile_new_child(GSF_OUTFILE(pictures), name.c_str(), FALSE);	
-	oo_gsf_output_write(img, pByteBuf->getLength(),  pByteBuf->getPointer(0));
-
-	oo_gsf_output_close(img);
+	GsfOutput * img = gsf_outfile_new_child(GSF_OUTFILE(pictures), name.c_str(), FALSE);
+	gsf_output_write(img, pByteBuf->getLength(),  pByteBuf->getPointer(0));
+	gsf_output_close(img);
+	g_object_unref(G_OBJECT(img));
       }
 
-    oo_gsf_output_close(pictures);
+    gsf_output_close(pictures);
+    g_object_unref(G_OBJECT(pictures));
 
     return true;
   }
@@ -767,7 +749,6 @@ public:
 	"</manifest:manifest>\n"
       };
 
-
     writeToStream (manifest, preamble, NrElements(preamble));
 
     const char * szName;
@@ -777,17 +758,19 @@ public:
       {
 	if (k == 0) {
 	  name = "<manifest:file-entry manifest:media-type='' manifest:full-path='Pictures/'/>\n";
-	  oo_gsf_output_write(manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
+	  gsf_output_write(manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
 	}
 	  
 	name = UT_String_sprintf("<manifest:file-entry manifest:media-type='%s' manifest:full-path='Pictures/IMG-%d.png'/>\n", szMimeType, k);
-	oo_gsf_output_write (manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
+	gsf_output_write (manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
       }
 
     writeToStream (manifest, postamble, NrElements(postamble));
 
-    oo_gsf_output_close(manifest);
-    oo_gsf_output_close(meta_inf);
+    gsf_output_close(manifest);
+    g_object_unref(G_OBJECT(manifest));
+    gsf_output_close(meta_inf);
+    g_object_unref(G_OBJECT(meta_inf));
 
     return true;
   }
@@ -801,105 +784,108 @@ private:
 
 bool OO_StylesWriter::writeStyles(PD_Document * pDoc, GsfOutfile * oo, OO_StylesContainer & stylesContainer)
 {
-  GsfOutput * styleStream = gsf_outfile_new_child (oo, "styles.xml", FALSE);
-  
-  static const char * const preamble [] = 
-    {	   
-      "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n",
-      "<!DOCTYPE office:document-styles PUBLIC \"-//OpenOffice.org//DTD OfficeDocument 1.0//EN\" \"office.dtd\">\n",
-      "<office:document-styles xmlns:office=\"http://openoffice.org/2000/office\" xmlns:style=\"http://openoffice.org/2000/style\" xmlns:text=\"http://openoffice.org/2000/text\" xmlns:table=\"http://openoffice.org/2000/table\" xmlns:draw=\"http://openoffice.org/2000/drawing\" xmlns:fo=\"http://www.w3.org/1999/XSL/Format\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:number=\"http://openoffice.org/2000/datastyle\" xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns:chart=\"http://openoffice.org/2000/chart\" xmlns:dr3d=\"http://openoffice.org/2000/dr3d\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\" xmlns:form=\"http://openoffice.org/2000/form\" xmlns:script=\"http://openoffice.org/2000/script\" office:version=\"1.0\">\n"
-    };
-  
-  static const char * const midsection [] = 
-    {	   
-      "<office:styles>\n",
-      "<style:default-style style:family=\"graphics\">\n",
-      "<style:properties draw:start-line-spacing-horizontal=\"0.283cm\" draw:start-line-spacing-vertical=\"0.283cm\" draw:end-line-spacing-horizontal=\"0.283cm\" draw:end-line-spacing-vertical=\"0.283cm\" fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"simple\" style:line-break=\"strict\">\n",
-      "<style:tab-stops/>\n",
-      "</style:properties>\n",
-      "</style:default-style>\n",
-      "<style:default-style style:family=\"paragraph\">\n",
-      "<style:properties fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" fo:hyphenate=\"false\" fo:hyphenation-remain-char-count=\"2\" fo:hyphenation-push-char-count=\"2\" fo:hyphenation-ladder-count=\"no-limit\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"hanging\" style:line-break=\"strict\" style:tab-stop-distance=\"2.205cm\"/>\n",
-      "</style:default-style>\n"
-    };
-  
-  UT_UTF8String styles;
-  const PD_Style * pStyle = NULL;
-  UT_GenericVector<PD_Style *> vecStyles;
-  pDoc->getAllUsedStyles(&vecStyles);
-  UT_uint32 k = 0;
-  UT_UTF8String styleAtts, propAtts, font;
-  for (k=0; k < vecStyles.getItemCount(); k++)
-    {
-      pStyle = vecStyles.getNthItem(k);
-      PT_AttrPropIndex api = pStyle->getIndexAP();
-      const PP_AttrProp * pAP = NULL;
-      bool bHaveProp = pDoc->getAttrProp (api, &pAP);
-      
-      if (bHaveProp && pAP) 
+     GsfOutput * styleStream = gsf_outfile_new_child (oo, "styles.xml", FALSE);
+     
+     static const char * const preamble [] = 
+	{	   
+	   "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n",
+	   "<!DOCTYPE office:document-styles PUBLIC \"-//OpenOffice.org//DTD OfficeDocument 1.0//EN\" \"office.dtd\">\n",
+	   "<office:document-styles xmlns:office=\"http://openoffice.org/2000/office\" xmlns:style=\"http://openoffice.org/2000/style\" xmlns:text=\"http://openoffice.org/2000/text\" xmlns:table=\"http://openoffice.org/2000/table\" xmlns:draw=\"http://openoffice.org/2000/drawing\" xmlns:fo=\"http://www.w3.org/1999/XSL/Format\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:number=\"http://openoffice.org/2000/datastyle\" xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns:chart=\"http://openoffice.org/2000/chart\" xmlns:dr3d=\"http://openoffice.org/2000/dr3d\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\" xmlns:form=\"http://openoffice.org/2000/form\" xmlns:script=\"http://openoffice.org/2000/script\" office:version=\"1.0\">\n"
+	};
+
+     static const char * const midsection [] = 
+	{	   
+	   "<office:styles>\n",
+	   "<style:default-style style:family=\"graphics\">\n",
+	   "<style:properties draw:start-line-spacing-horizontal=\"0.283cm\" draw:start-line-spacing-vertical=\"0.283cm\" draw:end-line-spacing-horizontal=\"0.283cm\" draw:end-line-spacing-vertical=\"0.283cm\" fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"simple\" style:line-break=\"strict\">\n",
+	   "<style:tab-stops/>\n",
+	   "</style:properties>\n",
+	   "</style:default-style>\n",
+	   "<style:default-style style:family=\"paragraph\">\n",
+	   "<style:properties fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" fo:hyphenate=\"false\" fo:hyphenation-remain-char-count=\"2\" fo:hyphenation-push-char-count=\"2\" fo:hyphenation-ladder-count=\"no-limit\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"hanging\" style:line-break=\"strict\" style:tab-stop-distance=\"2.205cm\"/>\n",
+	   "</style:default-style>\n"
+	};
+
+	UT_UTF8String styles;
+	const PD_Style * pStyle = NULL;
+	UT_GenericVector<PD_Style *> vecStyles;
+	pDoc->getAllUsedStyles(&vecStyles);
+	UT_uint32 k = 0;
+	UT_UTF8String styleAtts, propAtts, font;
+	for (k=0; k < vecStyles.getItemCount(); k++)
 	{
-	  OO_StylesWriter::map(pAP, styleAtts, propAtts, font);
-	  
-	  styles += "<style:style " + styleAtts + ">\n";
-	  styles += "<style:properties " + propAtts + "/>\n";
-	  styles += "</style:style>\n";
+		pStyle = vecStyles.getNthItem(k);
+		PT_AttrPropIndex api = pStyle->getIndexAP();
+		const PP_AttrProp * pAP = NULL;
+		bool bHaveProp = pDoc->getAttrProp (api, &pAP);
+		
+		if (bHaveProp && pAP) 
+		{
+			OO_StylesWriter::map(pAP, styleAtts, propAtts, font);
+
+			styles += "<style:style " + styleAtts + ">\n";
+			styles += "<style:properties " + propAtts + "/>\n";
+			styles += "</style:style>\n";
+		}
+
+		if (font.size())
+		{
+			UT_String f = font.utf8_str();
+			stylesContainer.addFont(f);
+			font.clear();
+		}
 	}
-      
-      if (font.size())
+
+	static const char * const postamble [] = 
 	{
-	  UT_String f = font.utf8_str();
-	  stylesContainer.addFont(f);
-	  font.clear();
-	}
-    }
-  
-  static const char * const postamble [] = 
-    {
-      "<text:outline-style>\n",
-      "<text:outline-level-style text:level=\"1\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"2\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"3\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"4\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"5\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"6\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"7\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"8\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"9\" style:num-format=\"\"/>\n",
-      "<text:outline-level-style text:level=\"10\" style:num-format=\"\"/>\n",
-      "</text:outline-style>\n",
-      "<text:footnotes-configuration style:num-format=\"1\" text:start-value=\"0\" text:footnotes-position=\"page\" text:start-numbering-at=\"document\"/>\n",
-      "<text:endnotes-configuration style:num-format=\"i\" text:start-value=\"0\"/>\n",
-      "<text:linenumbering-configuration text:number-lines=\"false\" text:offset=\"0.499cm\" style:num-format=\"1\" text:number-position=\"left\" text:increment=\"5\"/>\n",
-      "</office:styles>\n",
-      "<office:automatic-styles>\n",
-      "<style:page-master style:name=\"pm1\">\n",
-      "<style:properties fo:page-width=\"21.59cm\" fo:page-height=\"27.94cm\" style:num-format=\"1\" style:print-orientation=\"portrait\" fo:margin-top=\"2.54cm\" fo:margin-bottom=\"2.54cm\" fo:margin-left=\"3.175cm\" fo:margin-right=\"3.175cm\" style:footnote-max-height=\"0cm\">\n",
-      "<style:footnote-sep style:width=\"0.018cm\" style:distance-before-sep=\"0.101cm\" style:distance-after-sep=\"0.101cm\" style:adjustment=\"left\" style:rel-width=\"25%\" style:color=\"#000000\"/>\n",
-      "</style:properties>\n",
-      "<style:header-style/>\n",
-      "<style:footer-style/>\n",
-      "</style:page-master>\n",
-      "</office:automatic-styles>\n",
-      "<office:master-styles>\n",
-      "<style:master-page style:name=\"Standard\" style:page-master-name=\"pm1\"/>\n",
-      "</office:master-styles>\n",
-      "</office:document-styles>\n"
-    };
-  
-  writeToStream(styleStream, preamble, NrElements(preamble));
-  
-  UT_UTF8String fontDecls = "<office:font-decls>\n";
-  OO_StylesWriter::addFontDecls(fontDecls, stylesContainer);
-  fontDecls += "</office:font-decls>\n";
-  writeUTF8String(styleStream, fontDecls.utf8_str());
-  
-  writeToStream(styleStream, midsection, NrElements(midsection));
-  writeUTF8String(styleStream, styles.utf8_str());
-  writeToStream(styleStream, postamble, NrElements(postamble));
-  
-  oo_gsf_output_close(styleStream);
-  
-  return true;
+	   "<text:outline-style>\n",
+	   "<text:outline-level-style text:level=\"1\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"2\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"3\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"4\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"5\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"6\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"7\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"8\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"9\" style:num-format=\"\"/>\n",
+	   "<text:outline-level-style text:level=\"10\" style:num-format=\"\"/>\n",
+	   "</text:outline-style>\n",
+	   "<text:footnotes-configuration style:num-format=\"1\" text:start-value=\"0\" text:footnotes-position=\"page\" text:start-numbering-at=\"document\"/>\n",
+	   "<text:endnotes-configuration style:num-format=\"i\" text:start-value=\"0\"/>\n",
+	   "<text:linenumbering-configuration text:number-lines=\"false\" text:offset=\"0.499cm\" style:num-format=\"1\" text:number-position=\"left\" text:increment=\"5\"/>\n",
+	   "</office:styles>\n",
+	   "<office:automatic-styles>\n",
+	   "<style:page-master style:name=\"pm1\">\n",
+	   "<style:properties fo:page-width=\"21.59cm\" fo:page-height=\"27.94cm\" style:num-format=\"1\" style:print-orientation=\"portrait\" fo:margin-top=\"2.54cm\" fo:margin-bottom=\"2.54cm\" fo:margin-left=\"3.175cm\" fo:margin-right=\"3.175cm\" style:footnote-max-height=\"0cm\">\n",
+	   "<style:footnote-sep style:width=\"0.018cm\" style:distance-before-sep=\"0.101cm\" style:distance-after-sep=\"0.101cm\" style:adjustment=\"left\" style:rel-width=\"25%\" style:color=\"#000000\"/>\n",
+	   "</style:properties>\n",
+	   "<style:header-style/>\n",
+	   "<style:footer-style/>\n",
+	   "</style:page-master>\n",
+	   "</office:automatic-styles>\n",
+	   "<office:master-styles>\n",
+	   "<style:master-page style:name=\"Standard\" style:page-master-name=\"pm1\"/>\n",
+	   "</office:master-styles>\n",
+	   "</office:document-styles>\n"
+      };
+     
+
+	writeToStream(styleStream, preamble, NrElements(preamble));
+
+	UT_UTF8String fontDecls = "<office:font-decls>\n";
+	OO_StylesWriter::addFontDecls(fontDecls, stylesContainer);
+	fontDecls += "</office:font-decls>\n";
+    writeUTF8String(styleStream, fontDecls.utf8_str());
+
+    writeToStream(styleStream, midsection, NrElements(midsection));
+    writeUTF8String(styleStream, styles.utf8_str());
+    writeToStream(styleStream, postamble, NrElements(postamble));
+     
+     
+    gsf_output_close(styleStream);
+    g_object_unref(G_OBJECT(styleStream));
+     
+    return true;
 }
 
 void OO_StylesWriter::addFontDecls(UT_UTF8String & buffer, OO_StylesContainer & stylesContainer)
@@ -910,10 +896,10 @@ void OO_StylesWriter::addFontDecls(UT_UTF8String & buffer, OO_StylesContainer &
 		// check here by using pango?
 		const XML_Char * pitch = "variable";
 		const UT_String * font = vecFonts->getNthItem(i);
-		buffer += UT_UTF8String_sprintf("<style:font-decl style:name=\"%s\" fo:font-family=\"'%s'\" style:font-pitch=\"%s\"/>\n", 
-						font->c_str(), font->c_str(), pitch);
+		buffer += UT_UTF8String_sprintf("<style:font-decl style:name=\"%s\" fo:font-family=\"'%s'\" style:font-pitch=\"%s\"/>", 
+										font->c_str(), font->c_str(), pitch);
 	}
-	DELETEP(vecFonts);
+	FREEP(vecFonts);
 }
 
 void OO_StylesWriter::map(const PP_AttrProp * pAP, UT_UTF8String & styleAtts, UT_UTF8String & propAtts, UT_UTF8String & font) 
@@ -1088,8 +1074,6 @@ IE_Exp_OpenWriter::~IE_Exp_OpenWriter()
 {
 }
 
-#define SXW_MIMETYPE "application/vnd.sun.xml.writer"
-
 /*!
  * This writes out our AbiWord file as an OpenOffice
  * compound document
@@ -1097,16 +1081,7 @@ IE_Exp_OpenWriter::~IE_Exp_OpenWriter()
 UT_Error IE_Exp_OpenWriter::_writeDocument(void)
 {
   UT_return_val_if_fail(m_oo, UT_ERROR);
-
-  {
-    GsfOutput * mimetype = gsf_outfile_new_child (m_oo, "mimetype", FALSE);
-    if (!mimetype)
-      return UT_ERROR;
-
-    oo_gsf_output_write(mimetype, strlen(SXW_MIMETYPE), (const guint8 *)SXW_MIMETYPE);
-    oo_gsf_output_close(mimetype);
-  }
-
+  
   if (!OO_MetaDataWriter::writeMetaData(getDoc(), m_oo))
     return UT_ERROR;
 
@@ -1160,7 +1135,8 @@ bool IE_Exp_OpenWriter::_openFile(const char * szFilename)
 bool IE_Exp_OpenWriter::_closeFile(void)
 {
   if(m_oo) {
-    oo_gsf_output_close(GSF_OUTPUT(m_oo));
+    gsf_output_close(GSF_OUTPUT(m_oo));
+    g_object_unref(G_OBJECT(m_oo));
     m_oo = 0;
   }
 
