@@ -96,8 +96,6 @@ public:
   */
   const XML_Char ** getAbiPageAtts(const XML_Char * masterName)
   {
-    UT_return_val_if_fail(masterName != NULL, (const XML_Char **)m_pageAtts);
-
     if (UT_strcmp (m_name.c_str(), masterName))
       // FIXME can there be more than one master-page?
       UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -614,8 +612,6 @@ const XML_Char* IE_Imp_OpenWriter::mapStyle (const XML_Char * name) const
 
 const OO_Style * IE_Imp_OpenWriter::mapStyleObj (const XML_Char * name) const
 {
-  if (!name)
-    return NULL;
   return m_styleBucket.pick((const char *)name);
 }
 
@@ -864,13 +860,8 @@ UT_Error IE_Imp_OpenWriter::_handleMimetype ()
 
   GsfInput * input = gsf_infile_child_by_name(m_oo, "mimetype");
 
-  if (!input) {
-    UT_DEBUGMSG(("Error: didn't get a mimetype. Assuming that it's a application/vnd.sun.xml.writer document\n"));
-    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-
-    m_bOpenDocument = FALSE;
-    return UT_OK;
-  }
+  if (!input)
+    return UT_ERROR;
 
   UT_UTF8String mimetype = "";
   if (gsf_input_size (input) > 0) {
@@ -969,16 +960,6 @@ public:
   
   virtual ~OpenWriter_StylesStream_Listener ()
   {
-    m_styleNameMap.purgeData();
-  }
-
-  UT_UTF8String getStyleName(const UT_UTF8String & in) const
-  {
-    UT_UTF8String * name = m_styleNameMap.pick(in.utf8_str());
-    if (!name)
-      return UT_UTF8String(in);
-    else
-      return *name;
   }
   
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
@@ -999,29 +980,15 @@ public:
       if (attr)
 	m_name = attr;
 
-      attr = UT_getAttribute("style:display-name",atts);
-      if (attr)
-	m_displayName = attr;
-
       if (m_name != "Standard") {
 	attr = UT_getAttribute("style:parent-style-name",atts);
-	if (attr) {
-	  if (!UT_strcmp(attr, "Standard")) {
-	    m_parent = "Normal";
-	  } else {
-	    m_parent = attr;
-	  }
-	}
+	if (attr)
+	  m_parent = attr;
 	
 	attr = UT_getAttribute("style:next-style-name",atts);
-	if (attr) {
-	  if (!UT_strcmp(attr, "Standard")) {
-	    m_next = "Normal";
-	  } else {
-	    m_next = attr;
-	  }
-	}
-
+	if (attr)
+	  m_next = attr;
+	
 	attr = UT_getAttribute("style:family",atts);
 	if (attr && strcmp (attr, "paragraph") != 0)
 	  m_type = CHARACTER;
@@ -1035,25 +1002,25 @@ public:
 	m_type = PARAGRAPH;
       }
       DELETEP(m_ooStyle);
-      m_ooStyle = NULL;
+	  m_ooStyle = NULL;
     }
     else if (/* SXW || ODT */
-	     (!strcmp (name, "style:properties") || !strcmp (name, "style:page-layout-properties")) && 
-	     m_pageMaster) {
+			(!strcmp (name, "style:properties") || !strcmp (name, "style:page-layout-properties")) && 
+			m_pageMaster) {
       // page setup, because we are in page-master subtree
       m_ooPageStyle.appendPageMaster(m_pageMaster, atts);
     }
     else if (!strcmp (name, "style:properties") || 				/* SXW */
-	     !strcmp (name, "style:text-properties") || 		/* ODT */
-	     !strcmp (name, "style:paragraph-properties")) {
-      
-      if (m_ooStyle == NULL) {
-	getDocument()->getStyle (m_parent.utf8_str(), &m_pParentStyle);
-	m_ooStyle = new OO_Style (atts, m_pParentStyle, m_bOpenDocument);
-      }
-      else {
-	m_ooStyle->parse(atts);
-      }
+			 !strcmp (name, "style:text-properties") || 		/* ODT */
+			 !strcmp (name, "style:paragraph-properties")) {
+
+		if (m_ooStyle == NULL) {
+	      getDocument()->getStyle (m_parent.utf8_str(), &m_pParentStyle);
+    	  m_ooStyle = new OO_Style (atts, m_pParentStyle, m_bOpenDocument);
+		}
+		else {
+		  m_ooStyle->parse(atts);
+		}
     }
   } 
 
@@ -1069,16 +1036,8 @@ public:
 	const XML_Char * atts[11];
 	atts[propCtr++] = "type";
 	atts[propCtr++] = (m_type == PARAGRAPH ? "P" : "C");
-
 	atts[propCtr++] = "name";
-	if (m_displayName.size()) {	 
-	  atts[propCtr++] = m_displayName.utf8_str();
-	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_displayName));
-	}
-	else {
-	  atts[propCtr++] = m_name.utf8_str();
-	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_name));
-	}
+	atts[propCtr++] = m_name.utf8_str();
 
 	if (m_ooStyle) {
 	  atts[propCtr++] = "props";
@@ -1101,11 +1060,10 @@ public:
       }
 
       m_name.clear ();
-      m_displayName.clear ();
       m_parent.clear ();
       m_next.clear ();
       DELETEP(m_ooStyle);
-      m_ooStyle = NULL;
+	  m_ooStyle = NULL;
     }
   }
   
@@ -1126,7 +1084,6 @@ public:
 private:
 
   UT_UTF8String m_name;
-  UT_UTF8String m_displayName;
   UT_UTF8String m_parent;
   UT_UTF8String m_next;
   enum { CHARACTER, PARAGRAPH } m_type;
@@ -1135,8 +1092,6 @@ private:
   OO_PageStyle m_ooPageStyle;
   const XML_Char * m_pageMaster;
   const bool m_bOpenDocument;
-
-  UT_GenericStringMap<UT_UTF8String *> m_styleNameMap;
 };
 
 /*!
@@ -1160,8 +1115,6 @@ private:
   UT_UCS4String m_charData;
   bool m_bAcceptingText;
   bool m_bInSection;  
-  bool m_bSectionHasPara;
-  bool m_bInTOC;
 
   UT_UTF8String m_curStyleName;
 
@@ -1179,7 +1132,7 @@ private:
 
 public:
   OpenWriter_ContentStream_Listener ( IE_Imp_OpenWriter * importer, OpenWriter_StylesStream_Listener * pSSListener, bool bOpenDocument )
-    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), m_bSectionHasPara(false), m_bInTOC(false), 
+    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), 
       m_pSSListener(pSSListener), m_imgCnt(0), m_row(0), m_col(0), m_cel(0), m_bOpenDocument(bOpenDocument)
   {
   }
@@ -1190,10 +1143,6 @@ public:
 
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
   {
-    // if we're inside a TOC, ignore the contents since SXW/ODT include a copy of the TOC
-    if (m_bInTOC)
-      return;
-
     if (!UT_strcmp(name, "office:body"))
       {
       }
@@ -1204,8 +1153,6 @@ public:
 
 	_appendSection(abi_sty);
 	m_bInSection = true;
-	m_bSectionHasPara = false;
-	m_bAcceptingText = false;
       }
     else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
@@ -1213,10 +1160,9 @@ public:
 	  {
 	      _appendSection(NULL);
 	      m_bInSection = true;
-	      m_bSectionHasPara = false;
 	  }
-	UT_UTF8String oo_sty;
-	const OO_Style * abi_sty = _mapStyleObj (UT_getAttribute ("text:style-name", atts), oo_sty);
+	const XML_Char * oo_sty = UT_getAttribute ("text:style-name", atts);
+	const OO_Style * abi_sty = _mapStyleObj (oo_sty);
 
 	if (abi_sty) {
 	  // append an empty paragraph with this one char
@@ -1233,10 +1179,10 @@ public:
 	  
 	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
 	}
-	else if (oo_sty.size()) {
+	else if (oo_sty) {
 	  const XML_Char * props[3];
 	  props[0] = "style";
-	  props[1] = oo_sty.utf8_str();
+	  props[1] = oo_sty;
 	  props[2] = 0;
 
 	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
@@ -1244,7 +1190,6 @@ public:
 	else
 	  getDocument()->appendStrux(PTX_Block, NULL);
 	m_bAcceptingText = true;
-	m_bSectionHasPara = true;
       }
     else if (!UT_strcmp(name, "text:span"))
       {
@@ -1294,21 +1239,6 @@ public:
       _insertBookmark (name, "start");
       _insertBookmark (name, "end");
     }
-    else if (!UT_strcmp(name, "text:table-of-content"))
-    {
-      _flush ();
-
-      if(!m_bAcceptingText)
-	{
-	  getDocument()->appendStrux(PTX_Block, NULL);
-	  m_bAcceptingText = true ;
-	}
-      
-      getDocument()->appendStrux(PTX_SectionTOC, NULL);
-      getDocument()->appendStrux(PTX_EndTOC, NULL);
-
-      m_bInTOC = true;
-    }
     else if (!UT_strcmp(name, "draw:image"))
       {
 	_flush ();
@@ -1320,14 +1250,7 @@ public:
 	  {
 	      _appendSection(NULL);
 	      m_bInSection = true;
-	      m_bSectionHasPara = false;
 	  }
-	// FIXME rob: workaround for #6725 "AbiWord Crashes Importing OpenWriter 1.1 Document Containing Table"
-	if (!m_bSectionHasPara)
-	  {
-	      getDocument()->appendStrux(PTX_Block, NULL);
-	      m_bSectionHasPara = true;
- 	  }
 	_openTable (atts);
       }
     else if (!UT_strcmp(name, "table:table-column"))
@@ -1405,7 +1328,7 @@ public:
     else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
 	_flush ();
-	m_bAcceptingText = false;
+	m_bAcceptingText= false;
       }
     else if (!UT_strcmp(name, "text:span"))
       {
@@ -1418,9 +1341,6 @@ public:
 	_flush ();
 	getDocument()->appendObject(PTO_Hyperlink, NULL);
       }
-    else if (!UT_strcmp(name, "text:table-of-content")) {
-      m_bInTOC = false;
-    }
     else if (!UT_strcmp(name, "table:table"))
       {
 	_closeTable ();
@@ -1458,7 +1378,7 @@ public:
 
   virtual void charData (const XML_Char * buffer, int length)
   {
-    if (buffer && length && m_bAcceptingText && !m_bInTOC)
+    if (buffer && length && m_bAcceptingText)
       m_charData += UT_UCS4String (buffer, length, true);
   }
 
@@ -1622,8 +1542,6 @@ private:
   void _closeTable ()
   {
     getDocument()->appendStrux(PTX_EndTable, NULL);
-
-    m_row = m_cel = m_col = 0;
   }
 
   void _closeColumn ()
@@ -1648,14 +1566,12 @@ private:
 
   const XML_Char* _mapStyle (const XML_Char * name) const
   {
-    UT_UTF8String styleName = m_pSSListener->getStyleName(name);
-    return getImporter ()->mapStyle(styleName.utf8_str());
+    return getImporter ()->mapStyle(name);
   }
 
-  const OO_Style * _mapStyleObj (const XML_Char * name, UT_UTF8String & oo_sty) const
+  const OO_Style * _mapStyleObj (const XML_Char * name) const
   {
-    oo_sty = m_pSSListener->getStyleName(name);
-    return getImporter ()->mapStyleObj (oo_sty.utf8_str());
+    return getImporter ()->mapStyleObj (name);
   }
 
   bool _pushInlineFmt(const XML_Char ** atts)
