@@ -51,11 +51,11 @@ IE_Exp_DocBook_Sniffer::IE_Exp_DocBook_Sniffer (const char * name) :
 
 bool IE_Exp_DocBook_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix,".dbk") ||!UT_stricmp(szSuffix,".xml"));
+	return (UT_stricmp(szSuffix,".dbk") == 0);
 }
 
 UT_Error IE_Exp_DocBook_Sniffer::constructExporter(PD_Document * pDocument,
-						   IE_Exp ** ppie)
+													 IE_Exp ** ppie)
 {
 	IE_Exp_DocBook * p = new IE_Exp_DocBook(pDocument);
 	*ppie = p;
@@ -66,8 +66,8 @@ bool IE_Exp_DocBook_Sniffer::getDlgLabels(const char ** pszDesc,
 											const char ** pszSuffixList,
 											IEFileType * ft)
 {
-	*pszDesc = "DocBook (.dbk, .xml)";
-	*pszSuffixList = "*.dbk; *.xml";
+	*pszDesc = "DocBook (.dbk)";
+	*pszSuffixList = "*.dbk";
 	*ft = getFileType();
 	return true;
 }
@@ -695,7 +695,6 @@ void s_DocBook_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 			pData++;
 			break;
 
-		case UCS_VTAB:					// column break
 		case UCS_LF:					// LF -- representing a Forced-Line-Break
 			if (m_iBlockType != BT_PLAINTEXT)
 				sBuf += "<br/>";
@@ -956,7 +955,7 @@ void s_DocBook_Listener::_openTable(PT_AttrPropIndex api)
 
   m_pie->write("\n<table frame=\"all\">\n");
 
-  UT_String tgroup(UT_String_sprintf("<tgroup cols='%d' align='left' colsep='1' rowsep='1'>\n", nCols));
+  UT_String tgroup(UT_String_sprintf("<colgroup cols='%d' align='left' colsep='1' rowsep='1'>\n", nCols));
   m_pie->write(tgroup.c_str(), tgroup.size());
 
   for (int i = 0; i < nCols; i++)
@@ -1196,9 +1195,9 @@ void s_DocBook_Listener::_handleDataItems(void)
 			/* int result = */
 			m_pDocument->getApp()->makeDirectory(fname.c_str(), 0750);
 			
-			if (!UT_strcmp(szMimeType, "image/svg-xml"))
+			if (!UT_strcmp(szMimeType, "image/svg+xml"))
 				UT_String_sprintf(fname, "%s/%s_%d.svg", fname.c_str(), szName, loc);
-			if (!UT_strcmp(szMimeType, "text/mathml"))
+			if (!UT_strcmp(szMimeType, "application/mathml+xml"))
 				UT_String_sprintf(fname, "%s/%s_%d.mathml", fname.c_str(), szName, loc);
 			else // PNG Image
 			{  
