@@ -101,7 +101,7 @@ public:
 
 		if (it.is_valid())
 		{
-			m_pan = reinterpret_cast<fl_AutoNum* const> (static_cast<const UT_Pair*> (it.value())->second());
+			m_pan = reinterpret_cast<fl_AutoNum* const> (static_cast<const UT_Pair<void*,void*>*> (it.value())->second());
 			UT_ASSERT(m_pan);
 			m_iNextNb = m_pan->getStartValue32();
 
@@ -391,26 +391,13 @@ s_XSL_FO_Listener::s_XSL_FO_Listener(PD_Document * pDocument,
 	  m_iImgCnt(0),
 	  mTableHelper(pDocument)
 {
-	// Be nice to XML apps.  See the notes in _outputData() for more 
-	// details on the charset used in our documents.  By not declaring 
-	// any encoding, XML assumes we're using UTF-8.  Note that US-ASCII 
-	// is a strict subset of UTF-8. 
-
-	if (!XAP_EncodingManager::get_instance()->cjk_locale() &&
-	    (XAP_EncodingManager::get_instance()->try_nativeToU(0xa1) != 0xa1)) {
-	    // use utf8 for CJK locales and latin1 locales and unicode locales
-	    m_pie->write("<?xml version=\"1.0\" encoding=\"");
-	    m_pie->write(XAP_EncodingManager::get_instance()->getNativeEncodingName());
-	    m_pie->write("\"?>\n");
-	} else {
-	    m_pie->write("<?xml version=\"1.0\"?>\n");
-	}
+        m_pie->write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
 
 	m_pie->write("<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n\n");
 
 	m_pie->write("<!-- This document was created by AbiWord -->\n");
 	m_pie->write("<!-- AbiWord is a free, Open Source word processor. -->\n");
-	m_pie->write("<!-- You may obtain more information about AbiWord at www.abisource.com -->\n\n");
+	m_pie->write("<!-- You may obtain more information about AbiWord at http://www.abisource.com -->\n\n");
 
 	_handleLists();
 }
@@ -837,9 +824,9 @@ void s_XSL_FO_Listener::_handleDataItems()
 	  FILE *fp;
 	  UT_String fname;
 	  
-	  if (!UT_strcmp(szMimeType, "image/svg-xml"))
+	  if (!UT_strcmp(szMimeType, "image/svg+xml"))
 	    UT_String_sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
-	  if (!UT_strcmp(szMimeType, "text/mathml"))
+	  if (!UT_strcmp(szMimeType, "application/mathml+xml"))
 	    UT_String_sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
 	  else // PNG Image
 	    UT_String_sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
