@@ -335,7 +335,8 @@ wvDecodeComplex (wvParseStruct * ps)
     U32 beginfc, endfc;
 	U32 stream_size;
     U32 begincp, endcp;
-    U8 chartype;
+    int ichartype;
+    U8  chartype;
     U16 eachchar;
     U32 para_fcFirst, para_fcLim = 0xffffffffL;
     U32 dummy, nextpara_fcLim = 0xffffffffL;
@@ -365,12 +366,12 @@ wvDecodeComplex (wvParseStruct * ps)
     BKL *bkl;
     U32 *posBKL;
     U32 bkl_intervals;
-
+    wvVersion ver = wvQuerySupported (&ps->fib, NULL);
     external_wvReleasePAPX_FKP ();
     external_wvReleaseCHPX_FKP ();
 
     /*dop */
-    wvGetDOP (wvQuerySupported (&ps->fib, NULL), &ps->dop, ps->fib.fcDop,
+    wvGetDOP (ver, &ps->dop, ps->fib.fcDop,
 	      ps->fib.lcbDop, ps->tablefd);
 
 #if 0
@@ -380,7 +381,7 @@ unknown, the rest is a ordinary unicode string, is the time and date and saved b
 encoded into the first 22 bytes.
 */
     STTBF versioning;
-    if (wvQuerySupported (&ps->fib, NULL) == 0)
+    if (ver == 0)
       {
 	  U16 *str;
 	  wvError (("into the versions\n"));
@@ -410,8 +411,8 @@ encoded into the first 22 bytes.
     wvGetSTSH (&ps->stsh, ps->fib.fcStshf, ps->fib.lcbStshf, ps->tablefd);
 
     /* get font list */
-    if ((wvQuerySupported (&ps->fib, NULL) == WORD6)
-	|| (wvQuerySupported (&ps->fib, NULL) == WORD7))
+    if ((ver == WORD6)
+	|| (ver == WORD7))
 	wvGetFFN_STTBF6 (&ps->fonts, ps->fib.fcSttbfffn, ps->fib.lcbSttbfffn,
 			 ps->tablefd);
     else
@@ -419,8 +420,8 @@ encoded into the first 22 bytes.
 			ps->tablefd);
 
     /*we will need the table of names to answer questions like the name of the doc */
-    if ((wvQuerySupported (&ps->fib, NULL) == WORD6)
-	|| (wvQuerySupported (&ps->fib, NULL) == WORD7))
+    if ((ver == WORD6)
+	|| (ver == WORD7))
       {
 	  wvGetSTTBF6 (&ps->anSttbfAssoc, ps->fib.fcSttbfAssoc,
 		       ps->fib.lcbSttbfAssoc, ps->tablefd);
@@ -467,7 +468,7 @@ encoded into the first 22 bytes.
     wvGetFDOA_PLCF (&ps->fdoa, &ps->fdoapos, &ps->nooffdoa,
 		    ps->fib.fcPlcdoaMom, ps->fib.lcbPlcdoaMom, ps->tablefd);
 
-    wvGetCLX (wvQuerySupported (&ps->fib, NULL), &ps->clx,
+    wvGetCLX (ver, &ps->clx,
 	      (U32) ps->fib.fcClx, ps->fib.lcbClx, (U8) ps->fib.fExtChar,
 	      ps->tablefd);
 
@@ -483,8 +484,8 @@ encoded into the first 22 bytes.
        we will need the paragraph and character bounds table to make decisions as 
        to where a table begins and ends
      */
-    if ((wvQuerySupported (&ps->fib, NULL) == WORD6)
-	|| (wvQuerySupported (&ps->fib, NULL) == WORD7))
+    if ((ver == WORD6)
+	|| (ver == WORD7))
       {
 	  wvGetBTE_PLCF6 (&btePapx, &posPapx, &para_intervals,
 			  ps->fib.fcPlcfbtePapx, ps->fib.lcbPlcfbtePapx,
@@ -510,7 +511,8 @@ encoded into the first 22 bytes.
     wvInitPAPX_FKP (&para_fkp);
     wvInitCHPX_FKP (&char_fkp);
 
-    wvHandleDocument (ps, DOCBEGIN);
+    if(wvHandleDocument (ps, DOCBEGIN))
+		goto  finish_processing;
 
 	/*get stream size for bounds checking*/
 	stream_size = wvStream_size(ps->mainfd);
@@ -518,11 +520,11 @@ encoded into the first 22 bytes.
     /*for each piece */
     for (piececount = 0; piececount < ps->clx.nopcd; piececount++)
       {
-	  chartype =
+	  ichartype =
 	      wvGetPieceBoundsFC (&beginfc, &endfc, &ps->clx, piececount);
-	  if(chartype==-1)
+	  if(ichartype==-1)
 		  break;
-
+	  chartype = (U8) ichartype;
 	  /*lvm007@aha.ru fix antiloop: check stream size */
 	  if(beginfc>stream_size || endfc>stream_size){
 		  wvError (
@@ -580,8 +582,7 @@ encoded into the first 22 bytes.
 		if ((section_fcLim == 0xffffffff) || (section_fcLim == j))
 		  {
 		      section_dirty =
-			  wvGetSimpleSectionBounds (wvQuerySupported
-						    (&ps->fib, NULL), ps,
+			  wvGetSimpleSectionBounds (ver, ps,
 						    &sep, &section_fcFirst,
 						    &section_fcLim, i,
 						    &ps->clx, sed, &spiece,
@@ -590,7 +591,7 @@ encoded into the first 22 bytes.
 						    &ps->stsh, ps->mainfd);
 		      section_dirty =
 			  (wvGetComplexSEP
-			   (wvQuerySupported (&ps->fib, NULL), &sep, spiece,
+			   (ver, &sep, spiece,
 			    &ps->stsh, &ps->clx) ? 1 : section_dirty);
 		  }
 
@@ -609,8 +610,7 @@ encoded into the first 22 bytes.
 			       ("cp and fc are %x(%d) %x\n", i, i,
 				wvConvertCPToFC (i, &ps->clx)));
 		      cpiece =
-			  wvGetComplexParaBounds (wvQuerySupported
-						  (&ps->fib, NULL), &para_fkp,
+			  wvGetComplexParaBounds (ver, &para_fkp,
 						  &para_fcFirst, &para_fcLim,
 						  wvConvertCPToFC (i,
 								   &ps->clx),
@@ -643,14 +643,10 @@ encoded into the first 22 bytes.
 		if (j == para_fcFirst)
 		  {
 		      para_dirty =
-			  wvAssembleSimplePAP (wvQuerySupported
-					       (&ps->fib, NULL), &apap,
-					       para_fcLim, &para_fkp,
-					       &ps->stsh, ps->data);
+			  wvAssembleSimplePAP (ver, &apap, para_fcLim, &para_fkp, ps);
 		      para_dirty =
 			  (wvAssembleComplexPAP
-			   (wvQuerySupported (&ps->fib, NULL), &apap, cpiece,
-			    &ps->stsh, &ps->clx, ps->data) ? 1 : para_dirty);
+			   (ver, &apap, cpiece, ps) ? 1 : para_dirty);
 #ifdef SPRMTEST
 		      {
 			  int p;
@@ -669,8 +665,7 @@ encoded into the first 22 bytes.
 			       ("cp and fc are %x(%d) %x\n", i, i,
 				wvConvertCPToFC (i, &ps->clx)));
 		      npiece =
-			  wvGetComplexParaBounds (wvQuerySupported
-						  (&ps->fib, NULL), &para_fkp,
+			  wvGetComplexParaBounds (ver, &para_fkp,
 						  &dummy, &nextpara_fcLim,
 						  para_fcLim, &ps->clx,
 						  btePapx, posPapx,
@@ -681,16 +676,8 @@ encoded into the first 22 bytes.
 				para_fcLim));
 		      if (npiece > -1)
 			{
-			    wvAssembleSimplePAP (wvQuerySupported
-						 (&ps->fib, NULL),
-						 &ps->nextpap, nextpara_fcLim,
-						 &para_fkp, &ps->stsh,
-						 ps->data);
-			    wvAssembleComplexPAP (wvQuerySupported
-						  (&ps->fib, NULL),
-						  &ps->nextpap, npiece,
-						  &ps->stsh, &ps->clx,
-						  ps->data);
+			    wvAssembleSimplePAP (ver, &ps->nextpap, nextpara_fcLim, &para_fkp, ps);
+			    wvAssembleComplexPAP (ver, &ps->nextpap, npiece,ps);
 			}
 		      else
 			  wvInitPAP (&ps->nextpap);
@@ -743,8 +730,7 @@ encoded into the first 22 bytes.
 		  {
 		      wvReleaseCHPX_FKP (&char_fkp);
 		      /*try this without using the piece of the end char for anything */
-		      wvGetComplexCharBounds (wvQuerySupported
-					      (&ps->fib, NULL), &char_fkp,
+		      wvGetComplexCharBounds (ver, &char_fkp,
 					      &char_fcFirst, &char_fcLim,
 					      wvConvertCPToFC (i, &ps->clx),
 					      &ps->clx, bteChpx, posChpx,
@@ -780,17 +766,16 @@ encoded into the first 22 bytes.
 		if (j == char_fcFirst)
 		  {
 		      /* a CHP's base style is in the para style */
-		      achp.istd = apap.istd;
+		      /*achp.istd = apap.istd;*/
 		      wvTrace (("getting chp\n"));
 		      char_dirty =
-			  wvAssembleSimpleCHP (wvQuerySupported
-					       (&ps->fib, NULL), &achp,
+				  wvAssembleSimpleCHP (ver, &achp, &apap,
 					       char_fcLim, &char_fkp,
 					       &ps->stsh);
 		      wvTrace (("getting complex chp\n"));
 		      char_dirty =
 			  (wvAssembleComplexCHP
-			   (wvQuerySupported (&ps->fib, NULL), &achp, cpiece,
+			   (ver, &achp, cpiece,
 			    &ps->stsh, &ps->clx) ? 1 : char_dirty);
 		      wvHandleElement (ps, CHARPROPBEGIN, (void *) &achp,
 				       char_dirty);
@@ -850,6 +835,7 @@ encoded into the first 22 bytes.
 #endif
       }
 
+ finish_processing:
     if (char_pendingclose)
       {
 	  wvInitCHP (&achp);
@@ -913,6 +899,7 @@ encoded into the first 22 bytes.
 	  wvFree (ps->vmerges);
       }
     wvFree (ps->cellbounds);
+	wvOLEFree(ps);
     tokenTreeFreeAll ();
 }
 
@@ -1003,8 +990,7 @@ CLX.  If that grpprl contains any paragraph sprms, they should be applied to
 the local PAP.
 */
 int
-wvAssembleComplexPAP (wvVersion ver, PAP * apap, U32 cpiece, STSH * stsh,
-		      CLX * clx, wvStream * data)
+wvAssembleComplexPAP (wvVersion ver, PAP * apap, U32 cpiece, wvParseStruct *ps)
 {
     int ret = 0;
     U16 sprm, pos = 0, i = 0;
@@ -1014,46 +1000,46 @@ wvAssembleComplexPAP (wvVersion ver, PAP * apap, U32 cpiece, STSH * stsh,
     U8 val;
     Sprm RetSprm;
 
-    if (clx->pcd[cpiece].prm.fComplex == 0)
+    if (ps->clx.pcd[cpiece].prm.fComplex == 0)
       {
-	  val = clx->pcd[cpiece].prm.para.var1.val;
+	  val = ps->clx.pcd[cpiece].prm.para.var1.val;
 	  pointer = &val;
 #ifdef SPRMTEST
-	  wvError (("singleton\n", clx->pcd[cpiece].prm.para.var1.isprm));
+	  wvError (("singleton\n", ps->clx.pcd[cpiece].prm.para.var1.isprm));
 #endif
 	  RetSprm =
 	      wvApplySprmFromBucket (ver,
-				     (U16) wvGetrgsprmPrm ( (U16) clx->pcd[cpiece].prm.
+				     (U16) wvGetrgsprmPrm ( (U16) ps->clx.pcd[cpiece].prm.
 						     para.var1.isprm), apap,
-				     NULL, NULL, stsh, pointer, &pos, data);
+				     NULL, NULL, &ps->stsh, pointer, &pos, ps->data);
 	  if (RetSprm.sgc == sgcPara)
 	      ret = 1;
       }
     else
       {
-	  index = clx->pcd[cpiece].prm.para.var2.igrpprl;
+	  index = ps->clx.pcd[cpiece].prm.para.var2.igrpprl;
 #ifdef SPRMTEST
 	  wvError (("HERE-->\n"));
 	  fprintf (stderr, "\n");
-	  for (i = 0; i < clx->cbGrpprl[index]; i++)
-	      fprintf (stderr, "%x ", *(clx->grpprl[index] + i));
+	  for (i = 0; i < ps->clx.cbGrpprl[index]; i++)
+	      fprintf (stderr, "%x ", *(ps->clx.grpprl[index] + i));
 	  fprintf (stderr, "\n");
 	  i = 0;
 #endif
-	  while (i < clx->cbGrpprl[index])
+	  while (i < ps->clx.cbGrpprl[index])
 	    {
 		if (ver == WORD8)
-		    sprm = bread_16ubit (clx->grpprl[index] + i, &i);
+		    sprm = bread_16ubit (ps->clx.grpprl[index] + i, &i);
 		else
 		  {
-		      sprm8 = bread_8ubit (clx->grpprl[index] + i, &i);
+		      sprm8 = bread_8ubit (ps->clx.grpprl[index] + i, &i);
 		      sprm = (U16) wvGetrgsprmWord6 (sprm8);
 		      wvTrace (("sprm is %x\n", sprm));
 		  }
-		pointer = clx->grpprl[index] + i;
+		pointer = ps->clx.grpprl[index] + i;
 		RetSprm =
-		    wvApplySprmFromBucket (ver, sprm, apap, NULL, NULL, stsh,
-					   pointer, &i, data);
+		    wvApplySprmFromBucket (ver, sprm, apap, NULL, NULL, &ps->stsh,
+					   pointer, &i, ps->data);
 		if (RetSprm.sgc == sgcPara)
 		    ret = 1;
 	    }
